subroutine gconne(line,error)
  use greg_dependencies_interfaces
  use greg_interfaces, except_this=>gconne
  use gildas_def
  use sic_types
  use greg_kernel
  use greg_xyz
  use gbl_format
  !---------------------------------------------------------------------
  ! @ private
  ! GREG Support routine for command
  !   CONNECT/BLANKING Bval Eval
  !---------------------------------------------------------------------
  character(len=*), intent(in)    :: line   ! Command line
  logical,          intent(inout) :: error  ! Logical error flag
  ! Global
  include 'gbl_memory.inc'
  ! Local
  integer(kind=size_length) :: ixy
  integer(kind=4) :: form,narg
  type(sic_descriptor_t) :: xinca,yinca
  integer(kind=address_length) :: ipx,ipy
  real(kind=8) :: bval,eval
  real(kind=4) :: r4bv,r4ev
  !
  save xinca,yinca
  !
  ! /FILL option
  if (sic_present(2,0)) then
    call grfill (line,error)
    return
  endif
  !
  eval = eblank
  call sic_r8 (line,1,2,eval,.false.,error)
  if (error) return
  bval = cblank
  call sic_r8 (line,1,1,bval,.false.,error)
  if (error) return
  narg = sic_narg(0)
  if (narg.eq.0 .or. narg.eq.2) then
    form = 0
    call get_incarnation('CONNECT',line,form,ixy,xinca,yinca,error)
    if (error) return
    ipx = gag_pointer(xinca%addr,memory)
    ipy = gag_pointer(yinca%addr,memory)
    if (form.eq.fmt_r8) then
      call gr8_connect(ixy,memory(ipx),memory(ipy),bval,eval)
    else
      r4bv=bval
      r4ev=eval
      call gr4_connect(ixy,memory(ipx),memory(ipy),r4bv,r4ev)
    endif
    call sic_volatile(xinca)
    call sic_volatile(yinca)
  endif
end subroutine gconne
!
subroutine ghisto(line,error)
  use gildas_def
  use gbl_format
  use phys_const
  use sic_types
  use greg_dependencies_interfaces
  use greg_interfaces, except_this=>ghisto
  use greg_kernel
  use greg_pen
  use greg_types
  use greg_xyz
  !---------------------------------------------------------------------
  ! @ private
  ! Support routine for command
  !   HISTOGRAM [Array_X Array_Y]
  ! 1   [/BLANKING  Bval Eval]
  ! 2   [/FILL [Icolor]]
  ! 3   [/BASE [Ybase]]
  ! 4   [/HATCH [Ipen] [Angle] [Separ] [Phase]]
  !---------------------------------------------------------------------
  character(len=*), intent(in)    :: line   !
  logical,          intent(inout) :: error  !
  ! Global
  include 'gbl_memory.inc'
  ! Local
  character(len=*), parameter :: rname='HISTOGRAM'
  integer(kind=size_length) :: ixy
  integer(kind=4) :: form,narg,icol_old,ipen_old
  type(sic_descriptor_t) :: xinca,yinca
  integer(kind=address_length) :: ipx,ipy
  real(kind=8) :: bval,eval,ybase
  type(polygon_drawing_t) :: drawing
  !
  save xinca,yinca
  !
  ! Command line parsing
  narg = sic_narg(0)
  if (narg.ne.0 .and. narg.ne.2)  return
  !
  ! /BLANK options
  bval = cblank
  call sic_r8 (line,1,1,bval,.false.,error)
  if (error) return
  eval = eblank
  call sic_r8 (line,1,2,eval,.false.,error)
  if (error) return
  !
  ! /FILL and /HATCH options
  if (penupd) call setpen(cpen)
  call inqcol(icol_old)
  call inqpen(ipen_old)
  call parse_polygon_drawing(rname,line,2,4,drawing,error)
  if (error)  return
  !
  ! /BASE options
  ybase = 0.d0
  call sic_r8 (line,3,1,ybase,.false.,error)
  if (error)  return
  !
  form = fmt_r8
  call get_incarnation('HISTOGRAM',line,form,ixy,xinca,yinca,error)
  if (error) return
  ipx = gag_pointer(xinca%addr,memory)
  ipy = gag_pointer(yinca%addr,memory)
  !
  call gr_segm(rname,error)
  if (error)  return
  !
  if (drawing%hatched) then  ! /HATCH (+ maybe /FILL)
    if (drawing%filled) then
      ! Hatch+Fill = color filled histogram (base assumed)
      call setcol(drawing%fcolor)
      call gr8_hfill(ixy,memory(ipx),memory(ipy),bval,eval,ybase)
      !
      ! New segment (may have different colors)
      call gtsegm_close(error)
      call setcol(icol_old)  ! Reset color *after* closing the segment
      call gr_segm(rname,error)
    endif
    call setpen(drawing%hpen)
    call gr8_histo_hatch(ixy,memory(ipx),memory(ipy),bval,eval,ybase,drawing,error)
    if (error)  continue
    !
  elseif (sic_present(3,0)) then  ! /BASE
    if (drawing%filled) then  ! /FILL
      ! Base+Fill = color filled histogram
      call setcol(drawing%fcolor)
      call gr8_hfill(ixy,memory(ipx),memory(ipy),bval,eval,ybase)
    else
      call gr8_histo_base(ixy,memory(ipx),memory(ipy),bval,eval,ybase)
    endif
    !
  else
    ! No Base (maybe fill): top of histogram (maybe colored)
    if (drawing%filled)  call setcol(drawing%fcolor)
    if (eval.ge.0.d0) then
      call gr8_histo(ixy,memory(ipx),memory(ipy),bval,eval)
    else
      call histo1(memory(ipx),memory(ipy),ixy)
    endif
  endif
  !
  call gtsegm_close(error)
  !
  ! Reset pen *after* closing the segment
  call setpen(ipen_old)
  call setcol(icol_old)
  !
  call sic_volatile(xinca)
  call sic_volatile(yinca)
end subroutine ghisto
!
subroutine get_incarnation(command,line,form,ixy,xinca,yinca,error)
  use greg_dependencies_interfaces
  use greg_interfaces, except_this=>get_incarnation
  use gildas_def
  use sic_types
  use greg_xyz
  use gbl_message
  use gbl_format
  !---------------------------------------------------------------------
  ! @ private
  ! GreG Internal routine
  !   Get incarnation of variables in command line, defaulting to X Y
  ! Warning: this routine may be called with more than 3 arguments on
  ! the command line (see RGDATA command).
  !---------------------------------------------------------------------
  character(len=*),          intent(in)    :: command  !
  character(len=*),          intent(in)    :: line     !
  integer(kind=4),           intent(inout) :: form     !
  integer(kind=size_length), intent(out)   :: ixy      !
  type(sic_descriptor_t),    intent(out)   :: xinca    !
  type(sic_descriptor_t),    intent(out)   :: yinca    !
  logical,                   intent(inout) :: error    !
  ! Local
  character(len=80) :: var,chain
  logical :: found
  integer(kind=4) :: narg,nc
  type(sic_descriptor_t) :: xdesc,ydesc
  !
  data found/.false./
  !
  narg = sic_narg(0)
  if (narg.ge.2) then
    call sic_ch (line,0,1,var,nc,.true.,error)
    if (error) return
  else
    var = 'X'
  endif
  found = .true.
  call sic_materialize(var,xdesc,found)
  if (.not.found) then
    chain = 'Unknown variable '//var
    call greg_message(seve%e,command,chain)
    error = .true.
    return
  endif
  if (narg.ge.2) then
    call sic_ch (line,0,2,var,nc,.true.,error)
    if (error) return
  else
    var = 'Y'
  endif
  found = .true.
  call sic_materialize(var,ydesc,found)
  if (.not.found) then
    chain = 'Unknown variable '//var
    call greg_message(seve%e,command,chain)
    call sic_volatile(xdesc)   ! It may have been transposed...
    error = .true.
    return
  endif
  if (form.eq.0) then
    if (xdesc%type.eq.fmt_r4 .and. ydesc%type.eq.fmt_r4) then
      form = fmt_r4
    else
      form = fmt_r8
    endif
  endif
  call sic_incarnate(form,xdesc,xinca,error)
  if (error) then
    write(*,*) 'Error! ',var ! xdesc,xinca
    return
  endif
  call sic_incarnate(form,ydesc,yinca,error)
  if (error) then
    ! Must free both XDESC and XINCA if they are Transposed and Incarnated
    ! but they could point to the same virtual memory ...
    if (sic_notsamedesc(xdesc,xinca)) call sic_volatile(xdesc)
    call sic_volatile(xinca)
    return
  endif
  !
  if (yinca%size.ne.xinca%size) then
    call greg_message(seve%w,command,'Arrays have different sizes')
  endif
  ixy = min(xinca%size,yinca%size)
  if (form.eq.fmt_r8 .or. form.eq.fmt_i8) ixy = ixy/2
  ! Must free both XDESC and YDESC if they are Transposed
  ! but they could point to the same virtual memory as their Incarnation...
  if (sic_notsamedesc(xdesc,xinca)) call sic_volatile(xdesc)
  if (sic_notsamedesc(ydesc,yinca)) call sic_volatile(ydesc)
end subroutine get_incarnation
