subroutine iniloo (line,nline,error)
  use sic_structures
  use sic_dictionaries
  use sic_interfaces, except_this=>iniloo
  use gbl_message
  !---------------------------------------------------------------------
  ! @ private
  ! SIC   Internal routine
  !       Support for command
  !               FOR Variable n1 TO n2 ....
  !       or
  !               FOR /WHILE Logical_Expression
  !---------------------------------------------------------------------
  character(len=*), intent(in)  :: line   ! Command line
  integer(kind=4),  intent(in)  :: nline  !
  logical,          intent(out) :: error  ! Logical error flag
  ! Local
  character(len=*), parameter :: rname='FOR'
  integer(kind=4) :: istart
  character(len=message_length) :: mess
  !
  error=.false.
  !
  if (compil) then
    ! Compilation of a new nested FOR command, increment the buffer level
    if (aloop.ge.mcloop) then
      write(mess,101) mcloop
      call sic_message(seve%e,rname,mess)
      error=.true.
      return
    endif
    lasllo(cloop) = loolen
    lasblo(cloop) = bulend
    ! Encode loop number after command line
    call insloo(line,nline,error)
    if (error) return          ! No more room for loop command.
    loobuf (bulend-1) = aloop+1
    ! Check syntax
    if (sic_present(1,0)) then  ! FOR /WHILE
      ! Can not check: since this FOR /WHILE is nested in another FOR loop,
      ! its logical expression may involve variables not yet defined. No
      ! worry, the expression will be checked at execution time.
      ! call check_loop_logical(aloop+1,line,error)
    elseif (sic_present(0,2)) then  ! FOR I 1 TO 3
      call check_loop_variable(aloop+1,line,error)
    else
      call sic_message(seve%e,rname,'No variable or list')
    endif
    if (error) then            ! GD thinks this is veeery wise...
      call desinsloo
      loobuf (bulend-1) = aloop
      return
    endif
    aloop = aloop+1            ! Number of loops
    ploop(aloop) = cloop       ! Backward pointer to current loop
    cloop = aloop              ! Current loop
    firllo(cloop) = loolen
    firblo(cloop) = bulend
    !
  elseif (mlire(nlire).lt.-1) then
    ! Execution of a FOR command, dynamical interpretation,
    ! Get loop number from loop buffer
    cloop = loobuf(nloo-1)
    kloo(cloop)=0              ! Compteur de Boucles d'indices
    if_loop_level (cloop) = if_current
    if (sic_present(1,0)) then
      call check_loop_logical(cloop,line,error)
      if (error) goto 89
    elseif (sic_present(0,2)) then
      istart = sic_start(0,2)-1
      call sic_parse_listr8(rname,line(istart:),loop_index(cloop),mloop,error)
      if (error) goto 89
      call check_loop_variable(cloop,line,error)
      if (error) goto 89
      loop_length(cloop) = 0
    else
      call sic_message(seve%e,rname,'No variable or list')
      goto 89
    endif
    !
  else
    ! First loop of a new level, compile the command line and set COMPIL
    if (aloop.ge.mcloop) then
      write(mess,101) mcloop
      call sic_message(seve%e,rname,mess)
      error=.true.
      return
    endif
    if (nlire.ge.maxlev-1) then
      error = .true.
      call sic_message(seve%e,rname,'Input level too deep')
      return
    endif
    if (sic_present(1,0)) then
      call check_loop_logical(aloop+1,line,error)
      if (error) return
    elseif (sic_present(0,2)) then
      istart = sic_start(0,2)-1
      call sic_parse_listr8(rname,line(istart:),loop_index(aloop+1),mloop,error)
      if (error) return
      call check_loop_variable(aloop+1,line,error)
      if (error) return
      loop_length(aloop+1) = 0
    else
      call sic_message(seve%e,rname,'No variable or list')
      goto 89
    endif
    !
    ! OK...
    ifloop(nlire+1) = aloop
    curbyt(nlire+1) = nloo     ! Save Byte pointer
    curlin(nlire+1) = jloo     ! Save Line pointer
    aloop = aloop+1            ! Number of loops
    kloo(aloop) = 0
    compil  = .true.
    ploop(aloop) = cloop       ! Backward pointer to previous level
    cloop   = aloop            ! Current loop number
    firllo(cloop) = loolen
    firblo(cloop) = bulend
  endif
  return
  !
89 error = .true.
  if (nerr(var_level).eq.e_cont) then
    nloo  = lasblo(cloop)
    jloo  = lasllo(cloop)
    error = .false.
  endif
  if (lverif) then
    write(mess,102) cloop
    call sic_message(seve%i,rname,mess)
  endif
  loop_length(cloop) = 0
  cloop = ploop(cloop)
  return
  !
101 format('Only ',i2,' levels of FOR - NEXT loops')
102 format('Loop ',i2,' has finished')
end subroutine iniloo
!
subroutine insloo (line,nline,error)
  use sic_dependencies_interfaces
  use sic_interfaces, except_this=>insloo
  use gildas_def
  use sic_structures
  use gbl_message
  !---------------------------------------------------------------------
  ! @ private
  !  Insert a command line into the current loop buffer
  !---------------------------------------------------------------------
  character(len=*), intent(in)    :: line   !
  integer(kind=4),  intent(in)    :: nline  !
  logical,          intent(inout) :: error  !
  !
  mlen = (nline+3)/4+3
  !
  ! Loop buffer full
  if (mlen+bulend.ge.looend) then
    call sic_message(seve%e,'FOR','Loop buffer overflow')
    error = .true.
    return
  endif
  loobuf(bulend) = nline
  loobuf(bulend+1) = locwrd(loobuf(bulend+2))
  call ctodes (line,nline,loobuf(bulend+1))
  bulend = bulend + mlen
  loolen = loolen + 1
  !
end subroutine insloo
!
subroutine desinsloo
  use sic_structures
  !---------------------------------------------------------------------
  ! @ private
  ! This if an error occurs, to be used immediatly to forget erroneous
  ! line.
  !---------------------------------------------------------------------
  loolen = loolen - 1
  bulend = bulend - mlen
end subroutine desinsloo
!
subroutine exeloo (error)
  use sic_interfaces, except_this=>exeloo
  use sic_structures
  use gbl_message
  !---------------------------------------------------------------------
  ! @ private
  ! SIC   Internal routine
  !       Execute a FOR/NEXT loop : activated by a NEXT command
  !---------------------------------------------------------------------
  logical, intent(out) :: error  !
  ! Local
  character(len=*), parameter :: rname='FOR'
  logical :: logi
  character(len=loop_string_length) :: chain
  integer(kind=4) :: ls
  !
  ! Retrieves next index value
  error = .false.
  !
  ! An IF level must be defined even for null loops, otherwise errors occur
  ! at close time if they contain a test.
  if_loop_level (cloop) = if_current
  !
  if (loop_length(cloop).gt.0) then
    ! FOR /WHILE: evaluate the logical expression. See comment in check_loop_logical.
    call sic_shape(chain,loop_string(cloop),1,loop_length(cloop),ls,error)
    if (error)  return
    call sic_math_logi(chain,ls,logi,error)
    !
    ! Once more
    if (.not.error .and. logi) then
      nloo = firblo(cloop)     ! Reset Byte pointer
      jloo = firllo(cloop)     ! Reset Line pointer
      return
    endif
  else
    call list2(loop_index(cloop),kloo(cloop),indice(cloop),error)
    !
    ! Once more
    if (.not.error) then
      nloo = firblo(cloop)     ! Reset Byte pointer
      jloo = firllo(cloop)     ! Reset Line pointer
      if (lverif) then
        write(chain,101) cloop,indice(cloop)
        call sic_message(seve%i,rname,chain)
      endif
      return
    endif
  endif
  !
  ! End of loop
  error = .false.
  call finloo()
  !
101 format('Loop ',i2,' is running with index ',1pg24.17)
end subroutine exeloo
!
subroutine finloo
  use gbl_message
  use sic_interfaces, except_this=>finloo
  use sic_structures
  !----------------------------------------------------------------------
  ! @ private
  ! End of a loop execution
  !----------------------------------------------------------------------
  ! Local
  character(len=*), parameter :: rname='FOR'
  character(len=message_length) :: mess
  integer(kind=4) :: i,lloop
  !
  ! Anomalous end of a loop (QUIT or BREAK) : reset pointer
  nloo  = lasblo(cloop)
  jloo  = lasllo(cloop)
  if (loop_length(cloop).eq.0) then
    call delete_loop_variable(cloop)
  endif
  !
  ! Reset active IF_blocks
  do i=if_loop_level(cloop)+1,if_current
    if_active(i) = .false.
    if_elsefound(i) = .false.
    if_finished(i) = .true.
  enddo
  if_current = if_loop_level(cloop)
  if_last = if_current
  !
  indice(cloop) = 0.0
  kloo(cloop)  = 0
  if (lverif) then
    write(mess,102) cloop
    call sic_message(seve%i,rname,mess)
  endif
  loop_length(cloop) = 0
  lloop = cloop
  cloop = ploop(cloop)
  if (cloop.le.ifloop(nlire)) then
    aloop = ifloop(nlire)
    nloo = curbyt(nlire)       ! Restore Byte pointer
    jloo = curlin(nlire)       ! Save Line pointer
    bulend = firblo(lloop)
    loolen = firllo(lloop)
    nlire = nlire-1
  endif
  !
102 format('Loop ',i2,' has finished')
end subroutine finloo
!
subroutine aboloo
  use gbl_message
  use sic_interfaces, except_this=>aboloo
  use sic_structures
  !----------------------------------------------------------------------
  ! @ private
  ! Abort a loop compilation : decrease loop pointers and exit
  ! compilation mode if needed
  !----------------------------------------------------------------------
  ! Local
  character(len=*), parameter :: rname='FOR'
  character(len=message_length) :: mess
  integer(kind=4) :: lloop,i
  !
  write(mess,103) cloop
  call sic_message(seve%i,rname,mess)
  lloop = cloop
  do i=cloop,aloop
    if (loop_length(i).eq.0) then
      call delete_loop_variable(i)
    else
      loop_length(i) = 0
    endif
  enddo
  cloop = ploop(cloop)
  aloop = cloop
  !
  if (cloop.eq.0) then
    ! Exit compilation mode
    compil=.false.
    bulend = 1
    loolen = 0
  else
    ! Reset end of buffer pointers as for FINLOO
    loolen = firllo(lloop)
    bulend = firblo(lloop)
    if (aloop.eq.ifloop(nlire+1)) compil = .false.
  endif
  return
  !
103 format('Loop ',i2,' compilation aborted')
end subroutine aboloo
!
subroutine check_loop_variable(loop_number,line,error)
  use sic_dependencies_interfaces
  use sic_interfaces, except_this=>check_loop_variable
  use gildas_def
  use sic_structures
  use sic_dictionaries
  use gbl_message
  use gbl_format
  !---------------------------------------------------------------------
  ! @ private
  !  Check that the loop variable specified in command FOR does not
  ! already exist.
  !---------------------------------------------------------------------
  integer(kind=4),  intent(in)    :: loop_number  !
  character(len=*), intent(in)    :: line         !
  logical,          intent(inout) :: error        !
  ! Local
  character(len=*), parameter :: rname='VARIABLE'
  type(sic_identifier_t) :: var
  integer(kind=4) :: loop_in(mcloop)
  integer(kind=4) :: in,ier,nc
  save loop_in
  ! Data
  data loop_in/mcloop*empty/
  !
  call sic_ke (line,0,1,var%name,in,.true.,error)
  if (error) return
  var%lname = len_trim(var%name)
  !
  error = .true.
  var%level = var_level
  ier = sic_hasfin(maxvar,pfvar,pnvar,dicvar,var,in)
  if (ier.eq.1) then
    call sic_message(seve%e,rname,'Variable '//trim(var%name)//  &
    ' already exists')
    return
  endif
  !
  ! Insert in variable dictionnary
  ier = sic_hasins(rname,maxvar,pfvar,pnvar,dicvar,var,in)
  if (ier.eq.0 .or. ier.eq.2)  return
  !
  dicvar(in)%desc%addr = locwrd(indice(loop_number))
  dicvar(in)%desc%type = fmt_r8
  dicvar(in)%desc%readonly = .true.
  dicvar(in)%desc%ndim = 0
  dicvar(in)%desc%dims(:) = 1         ! Normally useless, but...
  dicvar(in)%desc%size = 2            ! Don't remove this one!!!
  dicvar(in)%desc%status = program_defined
  loop_in(loop_number) = in
  error = .false.
  return
  !
entry delete_loop_variable(loop_number)
  !---------------------------------------------------------------------
  ! Delete loop variable
  !---------------------------------------------------------------------
  in = loop_in(loop_number)
  ! FOR /WHILE loops do not define loop variables (nothing to delete here)
  if (in.ne.empty) then
    dicvar(in)%desc%status = empty       ! Mandatory
    ier = sic_hasdel(maxvar,pfvar,pnvar,dicvar,dicvar(in)%id)
    loop_in(loop_number) = empty
  endif
  return
  !
entry verify_loop_variable(loop_number,line,error)
  error = .false.
  in = loop_in(loop_number)
  if (in.eq.empty) then
    call sic_message(seve%e,rname,'No variable for FOR /WHILE loops')
    error = .true.
  else
    call sic_ke (line,0,1,var%name,nc,.true.,error)
    if (dicvar(in)%id%name.ne.var%name(1:nc)) then
      call sic_message(seve%e,rname,'Wrong loop variable '// &
        var%name(1:nc)//' expected '//trim(dicvar(in)%id%name))
      error = .true.
    else
      error = .false.
    endif
  endif
end subroutine check_loop_variable
!
subroutine check_loop_logical(loop_number,line,error)
  use sic_interfaces, except_this=>check_loop_logical
  use sic_structures
  use gbl_message
  !---------------------------------------------------------------------
  ! @ private
  !  Check that the expression specified in /WHILE option of FOR command
  ! is a valid logical expression.
  !---------------------------------------------------------------------
  integer(kind=4),  intent(in)    :: loop_number  !
  character(len=*), intent(in)    :: line         !
  logical,          intent(inout) :: error        !
  ! Local
  integer(kind=4), parameter :: optwhile=1
  integer(kind=4) :: is,ls
  logical :: logi
  character(len=loop_string_length) :: chain
  !
  ! 1) Retrieve the raw logical expression. No evaluation! For example with
  !     FOR /WHILE test['j+1']
  ! 'j+1' is not translated, it will be re-evaluated at each iteration.
  ls = sic_len(optwhile,1)
  if (ls.gt.loop_string_length) then
    call sic_message(seve%e,'FOR','Logical expression is too long')
    error = .true.
    return
  endif
  loop_length(loop_number) = ls
  is = sic_start(optwhile,1)
  loop_string(loop_number) =  line(is:is+ls-1)
  !
  ! 2) Check this is a valid logical expression
  call sic_shape(chain,loop_string(loop_number),1,loop_length(loop_number),ls,error)
  if (error)  return
  call sic_math_logi(chain,ls,logi,error)
  if (error)  return
  !
end subroutine check_loop_logical
