subroutine noema_list(line,error)
  use gbl_message
  use gkernel_interfaces
  use astro_interfaces, except_this=>noema_list
  use astro_types
  use my_receiver_globals
  use frequency_axis_globals
  !-----------------------------------------------------------------------
  ! @ private
  ! List current state of the setup definition
  !
  !-----------------------------------------------------------------------
  character(len=*), intent(inout) :: line        ! command line
  logical, intent(inout) :: error
  ! local
  character(len=*), parameter :: rname='LIST'
  integer(kind=4), parameter :: optconflict = 1  ! option /CONFLICT
  integer(kind=4), parameter :: optspw = 2   ! option /SPW
  integer(kind=4), parameter :: optchunk = 3 ! option /CHUNK
  integer(kind=4), parameter :: optfreq = 4  ! option /FREQUENCY
  integer(kind=4) :: is1,is2
  logical :: dospw,dofreq,dochunk,dopchanged,doconflict
  !
  if (rec%n_tunings.eq.0) then
    call astro_message(seve%e,rname,'No tuning found')
    error = .true.
    return
  endif
  !
  !check that doppler did not changed since last tuning
  call rec_check_doppler(rec%source,dopchanged,error)
  if (dopchanged) then
    call astro_message(seve%e,rname,'Doppler changed since last action')
    call rec_display_error('Doppler changed since last action',error)
    error = .true.
    return
  endif
  !
  if (spw%out%n_spw.eq.0) then
    call astro_message(seve%e,rname,'No SPW defined')
    error = .true.
    return
  endif
  !
  doconflict = sic_present(optconflict,0)
  dospw = sic_present(optspw,0)
  dofreq = sic_present(optfreq,0)
  dochunk = sic_present(optchunk,0)
  if (.not.dospw.and..not.dofreq.and..not.dochunk) then
    dospw=.true.
  endif
  !
  if (dospw) then ! LIST the spw
    if (sic_narg(0).eq.0) then
      is1=1
      is2=spw%out%n_spw
    else if (sic_narg(0).eq.2) then
      call sic_i0(line,0,1,is1,.true.,error)
      if (error) return
      call sic_i0(line,0,2,is2,.true.,error)
      if (error) return
    else
      call astro_message(seve%e,rname,'LIST /SPW accepts only 0 or 2 (ispw1 and ispw2) arguments')
      error=.true.
      return
    endif
    call noema_list_spw(spw%out,freq_axis%main,rec%source,doconflict,is1,is2,error)
    if (error) return
  else if (dochunk) then ! LIST the chunks
    call astro_message(seve%e,rname,'Not implemented yet')
    error=.true.
    return
  else if (dofreq) then ! LIST the frequencies
    call astro_message(seve%e,rname,'Not implemented yet')
    error=.true.
    return
  endif
  !
end subroutine noema_list
!
subroutine noema_list_spw(spwout,fr_axis,rsou,onlyconflict,w1,w2,error)
  use gbl_message
  use gkernel_interfaces
  use astro_interfaces, except_this=>noema_list_spw
  use astro_types
  !-----------------------------------------------------------------------
  ! @ private
  ! list all the spwctral window already defined
  !
  !-----------------------------------------------------------------------
  type(spw_output_t), intent(inout) :: spwout ! list of spw
  type(receiver_source_t), intent(in) :: rsou ! source parameters (for freq conversion)
  character(len=*), intent(in) :: fr_axis     ! current main frequency axis
  logical, intent(in) :: onlyconflict         ! list only conflicts
  integer(kind=4)       ::w1,w2               ! sublist limits
  logical, intent(inout) :: error
  ! Local
  character(len=*), parameter :: rname='LIST'
  integer(kind=4)       :: iw,nch,ic1,ic2
  character(len=64)    :: mess1,mess2,mess3,mess4,mess5,mess6
  character(len=512)    :: mess
  !
  if (onlyconflict) then
    write (mess,'(a)') 'Spectral windows in conflict are:'
  else
    write (mess,'(i0,1x,a)') spwout%n_spw,'spectral windows defined:'
  endif
  call astro_message(seve%i,rname,mess)
  !
  if (w1.le.0.or.w2.le.0.or.w2.lt.w1) then
    call astro_message(seve%e,rname,'Problem with index of spectral windows to list')
    error=.true.
    return
  endif
  do iw=w1,w2
    if (onlyconflict.and. &
        .not.spwout%win(iw)%conflict.and. & 
        .not.spwout%win(iw)%overload) cycle
    ic1=min(spwout%win(iw)%ich2,spwout%win(iw)%ich1)
    ic2=max(spwout%win(iw)%ich2,spwout%win(iw)%ich1)
    nch = abs(spwout%win(iw)%ich2-spwout%win(iw)%ich1)+1
    write (mess1,'(a,1x,i3,1x,a,1x,a,a)')  'SPW',iw,'in',trim(spwout%win(iw)%label),':'
    write (mess2,'(a,1x,f8.3,1x,a)') 'df =',spwout%win(iw)%resol,'kHz,'
    write (mess3,'(a,1x,i2,1x,a,1x,i2)') 'Chunks',ic1,'to',ic2
    ! get output freq in the current main frequency axis
    call noema_list_outputfreq(spwout%win(iw),fr_axis,rsou,mess4,error)
    if (error) return
    write (mess,'(a,1x,a,1x,a,a)') trim(mess1),trim(mess2),trim(mess3),trim(mess4)
    if (len_trim(spwout%win(iw)%user_label).gt.0) then
      write (mess5,'(a,a,a)') '(' , trim(spwout%win(iw)%user_label) , ')'
      write (mess,'(a,1x,a)') trim(mess),trim(mess5)
    endif
    if (spwout%win(iw)%conflict.or.spwout%win(iw)%overload) then
      write (mess6,'(a)') '!Conflict!'
      write (mess,'(a,1x,a)') trim(mess),trim(mess6)
    endif
    call astro_message(seve%r,rname,mess)
  enddo
  !
end subroutine noema_list_spw
!
subroutine noema_list_outputfreq(spwu,fr_axis,rsou,mess,error)
  use gbl_message
  use gkernel_interfaces
  use astro_interfaces, except_this=>noema_list_outputfreq
  use astro_types
  !-----------------------------------------------------------------------
  ! @ private
  ! convert the frequency limits of the spw unit into the output freq frame
  !
  !-----------------------------------------------------------------------
  type(spw_unit_t), intent(inout) :: spwu ! list of spw
  character(len=*), intent(in) :: fr_axis     ! current main frequency axis
  type(receiver_source_t), intent(in) :: rsou ! source parameters (for freq conversion)
  character(len=*), intent(inout)     :: mess !output message with right frequencies
  logical, intent(inout) :: error
  ! Local
  character(len=*), parameter :: rname='LIST'
  real(kind=8)  :: fmin,fmax
  !
  call rec_resttooutput(rname,spwu%restmin,fr_axis,rsou,fmin,error)
  if (error) return
  call rec_resttooutput(rname,spwu%restmax,fr_axis,rsou,fmax,error)
  if (error) return
  write (mess,'(a,1x,a,1x,f10.2,1x,a,1x,f10.2,1x,a)') &
              ',',trim(fr_axis),fmin,'to',fmax,'MHz'
  !
end subroutine noema_list_outputfreq
!
subroutine noema_reset(line,error)
  use gbl_message
  use gkernel_interfaces
  use astro_interfaces, except_this=>noema_reset
  use astro_types
  use my_receiver_globals
  use plot_molecules_globals
  use frequency_axis_globals
  !-----------------------------------------------------------------------
  ! @ private
  ! Reset one or several spw
  !
  !-----------------------------------------------------------------------
  character(len=*), intent(inout) :: line        ! command line
  logical, intent(inout) :: error
  ! local
  character(len=*), parameter :: rname='RESET'
  integer(kind=4) :: is,ir,izoom,nkey,nc,n_to_reset,iarg,i1
  integer(kind=4) :: spw_to_reset(m_spw)
  logical :: dopchanged,dolast,dolist
  integer(kind=4), parameter :: mkeys=2
  character(len=16) :: keys(mkeys),arg,key
  character(len=256) :: mess,spwstring
  data keys/'LAST','ALL'/
  !
  if (rec%n_tunings.eq.0) then
    call astro_message(seve%e,rname,'No tuning found. Nothing to reset')
    error = .true.
    return
  endif
  !
  !check that doppler did not changed since last tuning
  call rec_check_doppler(rec%source,dopchanged,error)
  if (dopchanged) then
    call astro_message(seve%e,rname,'Doppler changed since last action')
    call rec_display_error('Doppler changed since last action',error)
    error = .true.
    return
  endif
  !
  if (spw%out%n_spw.eq.0) then
    call astro_message(seve%e,rname,'Nothing to reset (no SPW defined)')
    error = .true.
    return
  endif
  !
  dolist=.false.
  dolast=.false.
  n_to_reset=0
  ! First parse the command line
  ! 1- Look for known keywords
  arg=''
  call sic_ke(line,0,1,arg,nc,.false.,error)
  if (error) return
  !
  if (nc.ne.0) then
    call sic_ambigs_sub(rname,arg,key,nkey,keys,mkeys,error)
    if (error) return
    if (nkey.eq.0) dolist=.true.
  else
    ! no arg
    dolast=.true.
  endif
  !
  if (dolist) then
  ! not a recognized keyword --> a sic list
    i1=1
    ! Put list in a string
    do iarg=1,sic_narg(0)
      ! Translate the arguments (as string, because of "TO" in "12 TO 34") and
      ! save them in a string
      call sic_ke(line,0,iarg,spwstring(i1:),nc,.true.,error)
      if (error)  return
      i1 = i1+nc+1
    enddo
    ! Build the list of each individual element selected
    call sic_build_listi4(spw_to_reset,n_to_reset,m_spw,spwstring,rname,error)
    if (error)  return
    !
  else if (key.eq.'LAST'.or.dolast) then
    ! reset the spectral windows created by the last spw command
    call noema_last_spw(spw%out,n_to_reset,spw_to_reset,error)
    if (error)  return
    if (n_to_reset.eq.0) return
  else if (key.eq.'ALL') then
    !reset all flexible spectral windows
    n_to_reset=0
    do is=1,spw%out%n_spw
      if (.not.(spw%out%win(is)%flexible)) cycle
      n_to_reset=n_to_reset+1
      spw_to_reset(n_to_reset)=is
    enddo
  else
    call astro_message(seve%e,rname,'Could not understand what to do')
    error=.true.
    return
  endif
  !
  !Check that all spw to reset exist and can be reset before starting to reset
  do ir=1,n_to_reset
    if (spw_to_reset(ir).gt.spw%out%n_spw.or.spw_to_reset(ir).le.0) then
      write (mess,'(a,i0,1x,a)') 'SPW #',spw_to_reset(ir),'is not defined'
      call astro_message(seve%e,rname,mess)
      error=.true.
    endif
    if (.not.(spw%out%win(spw_to_reset(ir))%flexible)) then
      write (mess,'(a,i0,1x,a)') 'Fixed SPW #',spw_to_reset(ir),'cannot be reset'
      call astro_message(seve%e,rname,mess)
      error=.true.
    endif
  enddo
  if (error) then
    call astro_message(seve%e,rname,'Nothing done')
    return
  endif
  !
  !Do the reset
  do ir=1,n_to_reset
    write (mess,'(a,1x,i0)') 'Resetting Spectral Window #',spw_to_reset(ir)
    call astro_message(seve%i,rname,mess)
    call noema_reset_spw(pfx,spw%out%win(spw_to_reset(ir)),error)
    if (error) exit
  enddo
  ! Remove empty windows from spw%out type
  call noema_compress_spw(spw%out,error)
  if (error) return
  ! look for conflicts
  call noema_check_conflicts(spw%out,pfx,error)
  if (error) return
  ! Display the spw list again (since numbers changed)
  call noema_list_spw(spw%out,freq_axis%main,rec%source,.false.,1,spw%out%n_spw,error)
  if (error) return
  ! Redo the plot
  if (cplot%plotmode.eq.'SUMMARY_R') then
    call rec_get_itune(rec,cplot%box(1)%iband,izoom,error)
    if (error) return
    call rec_plot_sidebands(rec,izoom,cplot,molecules,error)
    if (error) return
    call noema_draw_summary(spw%out,pfx,rec,cplot,molecules,error)
    if (error) return
  else 
    call noema_plot_selpfx(rec,noema,pfx,spw,cplot,molecules,error)
    if (error) return
  endif
  !
end subroutine noema_reset
!
subroutine noema_last_spw(spwout,n_last,spw_last,error)
  use gbl_message
  use gkernel_interfaces
  use astro_interfaces, except_this=>noema_last_spw
  use astro_types
  !-----------------------------------------------------------------------
  ! @ private
  ! find the latest spw that have been defined
  !-----------------------------------------------------------------------
  type(spw_output_t), intent(inout)  :: spwout
  integer(kind=4), intent(inout)  :: n_last
  integer(kind=4), intent(inout)  :: spw_last(m_spw)
  logical, intent(inout)            :: error
  ! Local
  integer(kind=4)       :: iw,ir
  integer(kind=4)       :: sort(spwout%n_spw)
  integer(kind=8)       :: lasttime
  !
  !Sort spw by time
  !
  do iw=1,spwout%n_spw
    sort(iw) = iw
  enddo
  !
  call gi0_quicksort_index_with_user_gtge(sort,spwout%n_spw,  &
    sort_spw_time_gt,sort_spw_time_ge,error)
  if (error)  return
  !
  lasttime=spwout%win(sort(spwout%n_spw))%ctime
  !
  n_last=0
  do iw=spwout%n_spw,1,-1
    ir=sort(iw)
    if (spwout%win(ir)%ctime.ne.lasttime.or..not.spwout%win(ir)%flexible) exit
    n_last=n_last+1
    spw_last(n_last)=ir
  enddo
  if (n_last.eq.0) then
    call astro_message(seve%i,'RESET','No more flexible SPW to reset')
    return
  endif
  !
end subroutine noema_last_spw
!
function sort_spw_time_gt(m,l)
  use my_receiver_globals
  !---------------------------------------------------------------------
  ! @ private
  ! Sorting function for noema_last_spw (greater than) [sort by time]
  ! ---
  !  NB: BASED ON THE GLOBAL SPW STRUCTURE SHARED THROUGH MODULE
  !      MY_RECEIVER_GLOBALS!!!
  !---------------------------------------------------------------------
  logical :: sort_spw_time_gt
  integer(kind=4), intent(in) :: m,l
  !
  ! Sort by creation time
  sort_spw_time_gt = spw%out%win(m)%ctime.gt.spw%out%win(l)%ctime
  !
end function sort_spw_time_gt
!
function sort_spw_time_ge(m,l)
  use my_receiver_globals
  !---------------------------------------------------------------------
  ! @ private
  ! Sorting function for noema_sort_spw (greater or equal)
  ! ---
  !  NB: THIS SORTS THE GLOBAL SPW STRUCTURE SHARED THROUGH MODULE
  !      MY_RECEIVER_GLOBALS!!!
  !---------------------------------------------------------------------
  logical :: sort_spw_time_ge
  integer(kind=4), intent(in) :: m,l
  !
  ! Sort by creation time
  sort_spw_time_ge = spw%out%win(m)%ctime.ge.spw%out%win(l)%ctime
  !
end function sort_spw_time_ge
!
subroutine pfx_reset_unit(pfxu,spwout,error)
  use astro_interfaces, except_this=>pfx_reset_unit
  use astro_types
  !-----------------------------------------------------------------------
  ! @ private
  ! defines polyfix backends
  !-----------------------------------------------------------------------
  type(pfx_unit_t), intent(inout)    :: pfxu
  type(spw_output_t), intent(inout)  :: spwout
  logical, intent(inout)            :: error
  ! Local
  integer(kind=4)       :: j,k,ich
  !
  ! First pfx unit characteristics
  do j=1,pfxu%n_modes
    do k=1,pfxu%mode(j)%n_types
      pfxu%mode(j)%chtype(k)%usage = 0
      do ich=1,m_ch
        pfxu%mode(j)%chtype(k)%chunks(ich) = 0
      enddo
    enddo
  enddo
  ! Then associated spectral windows
  do j=1,spwout%n_spw
    if (spwout%win(j)%label.ne.pfxu%label) cycle
    call noema_null_spw(spwout%win(j),error)
    if (error) return
  enddo
  !
end subroutine pfx_reset_unit
!
subroutine noema_reset_spw(pfx,spwin,error)
  use astro_interfaces, except_this=>noema_reset_spw
  use astro_types
  !-----------------------------------------------------------------------
  ! @ private
  ! reset a spw and free the associated chunks in pfx corresponding unit
  !-----------------------------------------------------------------------
  type(pfx_t), intent(inout)    :: pfx
  type(spw_unit_t), intent(inout)  :: spwin
  logical, intent(inout)            :: error
  ! Local
  character(len=*), parameter :: rname='RESET'
  integer(kind=4)       :: i,k,ich,ic1,ic2,imode
  !
  ic1=spwin%ich1
  ic2=spwin%ich2
  !
  ! Find and free the chunks related to the resetted spw
  do i=1,pfx%n_units
    if (pfx%unit(i)%label.ne.spwin%label) cycle
    imode=pfx%unit(i)%imode
    do k=1,pfx%unit(i)%mode(imode)%n_types
      if (.not.(pfx%unit(i)%mode(imode)%chtype(k)%move_chunk)) cycle
      do ich=ic1,ic2
        pfx%unit(i)%mode(imode)%chtype(k)%chunks(ich)=pfx%unit(i)%mode(imode)%chtype(k)%chunks(ich)-1
        if (pfx%unit(i)%mode(imode)%chtype(k)%chunks(ich).eq.0) then
          pfx%unit(i)%mode(imode)%chtype(k)%usage = pfx%unit(i)%mode(imode)%chtype(k)%usage-1
        endif
      enddo !chunks
    enddo  !types
  enddo !pfx units
  !
  !Then reset spw attributes
  call noema_null_spw(spwin,error)
  if (error) return
  !
end subroutine noema_reset_spw
!
subroutine noema_null_spw(win,error)
  use astro_interfaces, except_this=>noema_null_spw
  use astro_types
  !------------------------------------------------------------------------
  ! @ private
  ! put all attributes of a spectral window to undefined
  !------------------------------------------------------------------------
  type(spw_unit_t), intent(inout)       :: win
  logical, intent(inout)                :: error
  !
  win%restmin = undef_freq
  win%restmax = -1d0
  win%ich2 = -1
  win%ich1 = -1
  win%resol = -1.0
  win%iband = -1
  win%sb_code = -1
  win%bb_code = -1
  win%pol_code = -1
  win%label = ''
  win%user_label = ''
  win%flexible = .false.
  win%ctime = -1
 !
end subroutine noema_null_spw
!
subroutine noema_reset_backend(pfx,spwout,error)
  use astro_interfaces, except_this=>noema_reset_backend
  use astro_types
  !-----------------------------------------------------------------------
  ! @ private
  ! defines polyfix backends
  !-----------------------------------------------------------------------
  type(pfx_t), intent(inout)    :: pfx
  type(spw_output_t)            :: spwout
  logical, intent(inout)            :: error
  ! Local
  integer(kind=4)       :: i
  !
  ! Reset polyfix type settings
  do i=1,pfx%n_units
    pfx%unit(i)%iband = 0
    pfx%unit(i)%sb_code = 0
    pfx%unit(i)%bb_code = 0
    pfx%unit(i)%pol_code = 0
    pfx%unit(i)%label = ''
    pfx%unit(i)%imode = -1
    call pfx_reset_unit(pfx%unit(i),spwout,error)
  enddo
  !
  ! Reset spectral window output information
  spwout%n_spw=0
  !
end subroutine noema_reset_backend
!
subroutine noema_compress_spw(spwout,error)
  use astro_interfaces, except_this=>noema_compress_spw
  use astro_types
  !---------------------------------------------------------------------
  ! @ private
  ! Compress spw list (to remove spwindows that have been reset)
  !---------------------------------------------------------------------
  type(spw_output_t), intent(inout) :: spwout
  logical,            intent(inout) :: error
  ! Local
  integer(kind=4) :: ispw,jspw
  !
  jspw = 0
  do ispw=1,spwout%n_spw
    if (spwout%win(ispw)%restmin.eq.undef_freq) then
      cycle
    else
      jspw = jspw+1
    endif
    if (ispw.ne.jspw) then
      call noema_copy_spw(spwout%win(ispw),spwout%win(jspw),error)
      if (error)  return
    endif
  enddo ! ispw
  !
  spwout%n_spw = jspw
  !
end subroutine noema_compress_spw
!
subroutine noema_copy_spw(in,out,error)
  use astro_interfaces, except_this=>noema_copy_spw
  use astro_types
  !---------------------------------------------------------------------
  ! @ private
  ! Copy a spw_unit_t into another one
  !---------------------------------------------------------------------
  type(spw_unit_t), intent(in)    :: in
  type(spw_unit_t), intent(out)   :: out
  logical,          intent(inout) :: error
  !
  out = in
  !
end subroutine noema_copy_spw
!
subroutine noema_check_conflicts(spw,pfx,error)
  use gbl_message
  use astro_interfaces, except_this=>noema_check_conflicts
  use astro_types
  !---------------------------------------------------------------------
  ! @ private
  ! Copy a spw_unit_t into another one
  !---------------------------------------------------------------------
  type(spw_output_t), intent(inout)    :: spw
  type(pfx_t), intent(in)       :: pfx
  logical, intent(inout)   :: error
  ! Local
  character(len=*), parameter   :: rname='SPW'
  integer(kind=4)        :: ispw,iu,imode,it,ic1,ic2
  character(len=256)    :: mess
  !
  do ispw=1,spw%n_spw ! look for spw involved in the conflict
    do iu=1,pfx%n_units ! correlator units
      imode=pfx%unit(iu)%imode
      if (spw%win(ispw)%label.ne.pfx%unit(iu)%label) cycle ! spw is not in the current unit
      ic1=spw%win(ispw)%ich1
      ic2=spw%win(ispw)%ich2
      do it=1,pfx%unit(iu)%mode(imode)%n_types ! correlator types
        if (pfx%unit(iu)%mode(imode)%chtype(it)%df_chunks.ne.spw%win(ispw)%resol/1d3) cycle
        if (any(pfx%unit(iu)%mode(imode)%chtype(it)%chunks(ic1:ic2).gt.1)) then
          spw%win(ispw)%conflict = .true.
          write (mess,'(a,i0,1x,a)') 'SPW #',ispw,'uses conflicting chunk(s)'
          call astro_message(seve%w,rname,mess)
        else
          spw%win(ispw)%conflict = .false.
        endif
        if (pfx%unit(iu)%mode(imode)%chtype(it)%move_chunk .and. &
            pfx%unit(iu)%mode(imode)%chtype(it)%usage.gt. &
            pfx%unit(iu)%mode(imode)%chtype(it)%use_chunks) then
          spw%win(ispw)%overload = .true.
        else
          spw%win(ispw)%overload = .false.
        endif
      enddo ! chunk types (i.e. resolutions of same mode)
    enddo ! pfx units
  enddo ! defined spectral windows
  !
end subroutine noema_check_conflicts
!
subroutine noema_plot(line,error)
  use gbl_message
  use gkernel_interfaces
  use astro_interfaces, except_this=>noema_plot
  use astro_types
  use my_receiver_globals
  use plot_molecules_globals
  use frequency_axis_globals
  use string_parser_types
  !-----------------------------------------------------------------------
  ! @ private
  ! Plot the current setup of receiver\backend at Bure
  !
  !-----------------------------------------------------------------------
  character(len=*), intent(inout) :: line        ! command line
  logical, intent(inout) :: error
  ! local
  character(len=*), parameter :: rname='PLOT'
  integer(kind=4), parameter :: optrec=1        ! /RECEIVER
  integer(kind=4), parameter :: optpms=2        ! /PROPOSAL option
  integer(kind=4), parameter :: optline=3        ! /LINES option
  type(plot_molecules_t) :: cata
  type(string_parser_t) :: sp
  character(len=5)      :: selcode
  logical :: dopchanged,dorec,dopms,domol,dofreq,dodf
  integer(kind=4)       :: lpar,iu,isel,izoom
  real(kind=8)  :: frepin,frep
  character(len=256) :: mess
  integer(kind=4), parameter :: ipband=1
  integer(kind=4), parameter :: ippol=2
  integer(kind=4), parameter :: ipsb=3
  integer(kind=4), parameter :: ipbb=4
  integer(kind=4), parameter :: nband=3
  integer(kind=4), parameter :: npol=2
  integer(kind=4), parameter :: nsb=2
  integer(kind=4), parameter :: nbb=2
  integer(kind=4)       :: parsecode(4)
  character(len=2) :: bb_band(nband)
  character(len=1) :: bb_pol(npol),bb_sb(nsb),bb_bb(nbb)
  type(info_pms_t) :: infopms
  data bb_band/'B1','B2','B3'/
  data bb_pol/'H','V'/
  data bb_sb/'U','L'/
  data bb_bb/'O','I'/
  !
  !Preliminary checks
  if (rec%n_tunings.eq.0) then
    call astro_message(seve%e,rname,'No tuning found')
    error = .true.
    return
  endif
  if (.not.noema%ifproc%defined) then
    call astro_message(seve%e,rname,'Problem with IF Processor part')
    error = .true.
    return
  endif
  !
  !check that doppler did not changed since last tuning
  call rec_check_doppler(rec%source,dopchanged,error)
  if (dopchanged) then
    call astro_message(seve%e,rname,'Doppler changed since last tuning')
    call astro_message(seve%i,rname,'Nothing done')
    call rec_display_error('Doppler changed since last tuning',error)
    error = .true.
    return
  endif
  !
  dorec=sic_present(optrec,0)
  dopms=sic_present(optpms,0)
  dofreq=sic_present(optpms,1)
  dodf=sic_present(optpms,2)
  domol=sic_present(optline,0)
  !
  if (dorec.and.dopms) then
    call astro_message(seve%e,rname,'Options /RECEIVER and /PROPOSAL are exculsive from each other')
    error = .true.
    return
  endif
  if (domol.and..not.(dopms)) then
    call astro_message(seve%e,rname,'Options /LINE only available in combination with /PROPOSAL')
    error = .true.
    return
  endif
  !
  if (dorec) then
    ! Global view of the receiver
    izoom=1
    call sic_i0(line,optrec,1,izoom,.false.,error)
    if (error) return
    call rec_plot_sidebands(rec,izoom,cplot,molecules,error)
    if (error) return
    cplot%plotmode='SUMMARY_R'
    call noema_draw_summary(spw%out,pfx,rec,cplot,molecules,error)
    if (error) return
  else if (dopms) then
    ! plot on a single horizontal line to fit well in PMS pages
    call noema_default_pms(infopms,error)
    if (error) return
    izoom=1
    call sic_i0(line,optrec,1,izoom,.false.,error)
    if (error) return
    call noema_info_pms(rec%tune(izoom),rec%source,pfx,infopms,error)
    if (error) return
    ! prepare for molecules
    if (domol) then
      cata%catalog=molecules%catalog
      cata%doplot=.true.
      cata%profile='BOXCAR'
      cata%width=0d0
    else
      cata%doplot=.false.
    endif
    ! Indicate a representative frequency
    if (dofreq) then
      call sic_r8(line,optpms,1,frepin,.true.,error)
      if (error) return
      frepin=frepin*mhzperghz
      call rec_inputtorest(rname,frepin,freq_axis%main,rec%source,frep,error)
      if (error) return
      infopms%frep=frep*ghzpermhz
      if (dodf) then
        call sic_r8(line,optpms,2,infopms%resrep,.true.,error)
        if (error) return
      endif
    endif
    call rec_plot_tuning(rec,cata,izoom,cplot,error)
    cplot%plotmode='SUMMARY_P'
    call noema_draw_summary(spw%out,pfx,rec,cplot,cata,error)
    if (error) return
    call noema_lsrrange(spw%out,pfx,rec,cplot,infopms,error)
    if (error) return
    if (dofreq) then
      call noema_draw_frep(frep,spw%out,cplot,infopms,error)
      if (error) return
    endif
    call noema_print_pms(infopms,error)
    if (error) return
    !
  else
    ! Baseband selection
    selcode=''
    call sic_ke(line,0,1,selcode,lpar,.false.,error)
    if (error) return
    !Parse the user code
    if (len_trim(selcode).ne.0) then
      call string_parser_addlist(sp,ipband,bb_band,error)
      if (error) return
      call string_parser_addlist(sp,ippol,bb_pol,error)
      if (error) return
      call string_parser_addlist(sp,ipsb,bb_sb,error)
      if (error) return
      call string_parser_addlist(sp,ipbb,bb_bb,error)
      if (error) return
      call string_parser_parse('BASEBAND',sp,selcode,.false.,parsecode,error)
      if (error) return
    else 
      parsecode(ipband)=0
      parsecode(ippol)=0
      parsecode(ipsb)=0
      parsecode(ipbb)=0
    endif
    !
    if (parsecode(ippol).eq.0) then
      noema%selunit%polmode = "B" ! BOTH polar same config
    else
      noema%selunit%polmode = "S"
    endif
    !
    ! Loop over pfx%units to put the right numbers in ifsel() and define the mode when needed
    isel=0
    do iu=1,pfx%n_units
      if (pfx%unit(iu)%iband.ne.parsecode(ipband).and.parsecode(ipband).ne.0) cycle
      if (pfx%unit(iu)%pol_code.ne.parsecode(ippol).and.parsecode(ippol).ne.0) cycle
      if (pfx%unit(iu)%sb_code.ne.parsecode(ipsb).and.parsecode(ipsb).ne.0) cycle
      if (pfx%unit(iu)%bb_code.ne.parsecode(ipbb).and.parsecode(ipbb).ne.0) cycle
      isel=isel+1
      noema%selunit%usel(isel)=iu
    enddo !iu
    if (isel.eq.0) then
      call astro_message(seve%e,rname,'The selection returns nothing')
      error = .true.
      return
    endif
    !
    noema%selunit%n_ifsel=isel
    write (mess,'(a)') 'Selected Basebands are:'
    do iu=1,m_backunits
      if (noema%selunit%usel(iu).ne.-1) then
        write (mess,'(a,1x,a)') trim(mess),trim(noema%ifproc%ifc(noema%selunit%usel(iu))%label)
      endif
    enddo !iu
    call astro_message(seve%i,rname,mess)
    !
    call noema_plot_selpfx(rec,noema,pfx,spw,cplot,molecules,error)
    if (error) return
    cplot%plotmode='SUMMARY_B'
    !
  endif
  !
end subroutine noema_plot
!
subroutine noema_default_pms(pms,error)
  use astro_interfaces, except_this=>noema_default_pms
  use astro_types
  !-----------------------------------------------------------------------
  ! @ private
  ! Reset the structure containing info to be passed to Proposal Management System
  !-----------------------------------------------------------------------
  type(info_pms_t), intent(inout)   :: pms
  logical, intent(inout) :: error
  ! Local
  integer(kind=4) :: ir
  !
  pms%redshift=.false.
  pms%tuning_freq=0d0
  pms%tuning_ongrid=.false.
  pms%frep=0d0
  pms%npolrep=0d0
  pms%resrep=0d0
  pms%lsrrange=0d0
  pms%zrange=0d0
  pms%mflex=0d0
  pms%nflex=0d0
  pms%nspw=0
  pms%nresol=0
  pms%fcont=0d0
  pms%dfcont=0d0
  pms%rescont=0d0
  do ir=1,m_chtypes
    pms%resol(ir)=-1d0
    pms%npol(ir)=-1
  enddo !ir
  !
end subroutine noema_default_pms
!
subroutine noema_info_pms(rtune,rsou,pfx,pms,error)
  use gbl_message
  use astro_interfaces, except_this=>noema_info_pms
  use astro_types
  !-----------------------------------------------------------------------
  ! @ private
  ! Fill the structure containing info
  ! to be passed to Proposal Management System
  !-----------------------------------------------------------------------
  type(receiver_tune_t), intent(in)      :: rtune
  type(receiver_source_t), intent(in)      :: rsou
  type(pfx_t), intent(in)    :: pfx
  type(info_pms_t), intent(inout)   :: pms
  logical, intent(inout) :: error
  ! Local
  character(len=*), parameter :: rname='PLOT'
  integer(kind=4) :: iu,imode,it
  integer(kind=4), parameter :: lowrestype=1 ! low res always in type=1
  real(kind=8) :: dfconth,dfcontv
  !
  !Tuning
  pms%tuning_freq=rtune%frest/(1+rsou%z)*ghzpermhz ! In PMS we don't want rest in case of redshift
  pms%tuning_ongrid=rtune%ongrid
  !
  !Continuum + Correlator usage
  pms%fcont = pms%tuning_freq ! could be adapted later
  pms%dfcont = 0d0
  pms%rescont=-1d0
  dfconth = 0d0
  dfcontv = 0d0
  do iu=1,pfx%n_units
    imode=pfx%unit(iu)%imode
    !correlator usage
    do it=1,pfx%unit(iu)%mode(imode)%n_types
      if (.not.(pfx%unit(iu)%mode(imode)%chtype(it)%move_chunk)) cycle
      pms%mflex=pms%mflex+pfx%unit(iu)%mode(imode)%chtype(it)%use_chunks
      pms%nflex=pms%nflex+pfx%unit(iu)%mode(imode)%chtype(it)%usage
    enddo ! it
    ! continuum
    if (pfx%unit(iu)%pol_code.eq.1) then ! H polar
      dfconth=dfconth+pfx%unit(iu)%mode(imode)%chtype(lowrestype)%use_chunks* &
               pfx%unit(iu)%mode(imode)%chtype(lowrestype)%width_chunk- &
                pfx%unit(iu)%mode(imode)%chtype(lowrestype)%width_chunk/2d0
    else if (pfx%unit(iu)%pol_code.eq.2) then ! V polar
      dfcontv=dfcontv+pfx%unit(iu)%mode(imode)%chtype(lowrestype)%use_chunks* &
               pfx%unit(iu)%mode(imode)%chtype(lowrestype)%width_chunk- &
                pfx%unit(iu)%mode(imode)%chtype(lowrestype)%width_chunk/2d0 ! chunk #1 is 32 instead of 64
    endif
    if (pfx%unit(iu)%mode(imode)%chtype(lowrestype)%df_chunks.le.pms%rescont) cycle
    pms%rescont=pfx%unit(iu)%mode(imode)%chtype(lowrestype)%df_chunks
  enddo ! iu
  if (dfconth.lt.0.or.dfcontv.lt.0) then
    call astro_message(seve%e,rname,'Problem with continuum and polarizations')
    error=.true.
    return
  endif
  !
  if (dfconth.eq.dfcontv) then ! 2 polar
    pms%dfcont=dfconth
    pms%npolcont=2
  else if (dfconth.eq.0.and.dfcontv.gt.0) then ! full V
    pms%dfcont=dfcontv
    pms%npolcont=1
  else if (dfcontv.eq.0.and.dfconth.gt.0) then ! full H
    pms%dfcont=dfconth
    pms%npolcont=1
  else  ! mixed case
    pms%dfcont=(dfconth+dfcontv)/2d0
    pms%npolcont=2
  endif
  !
end subroutine noema_info_pms
!
subroutine noema_print_pms(infopms,error)
  use gbl_message
  use astro_interfaces, except_this=>noema_print_pms
  use astro_types
  !-----------------------------------------------------------------------
  ! @ private
  ! Print all info relevant for Proposal Management System
  !-----------------------------------------------------------------------
  type(info_pms_t), intent(in)   :: infopms
  logical, intent(inout) :: error
  ! Local
  character(len=*), parameter :: rname='PLOT'
  integer(kind=4) :: ir
  character(len=256) :: mess1
  !
  ! Tuning parameters
  write (mess1,'(a,1x,f0.6,1x,a)')  'INFO PMS: Tuning Frequency =',infopms%tuning_freq,'GHz'
  call astro_message(seve%r,rname,trim(mess1))
  !
  if (infopms%tuning_ongrid) then
    mess1='INFO PMS: Tuning ON grid'
  else
    mess1='INFO PMS: Tuning OFF grid'
  endif
  call astro_message(seve%r,rname,trim(mess1))
  ! LSR/z ranges
  if (infopms%redshift) then
    write (mess1,'(a,1x,f0.3,1x,a)') 'INFO PMS: Half the most narrow SPW is equivalent to an offset of', &
                        infopms%zrange,'in redshift'
  else
    write (mess1,'(a,1x,f0.3,1x,a)') 'INFO PMS: Half the most narrow SPW is equivalent to an offset of', &
                        infopms%lsrrange,'km/s in LSR velocity'
  endif
  call astro_message(seve%r,rname,mess1)
  ! Correlator usage
  if (infopms%mflex.ne.0) then
    write (mess1,'(a,1x,i0,a)') 'INFO PMS: Flexible part of the correlator is used at',nint(100.*infopms%nflex/infopms%mflex),'%'
    call astro_message(seve%r,rname,trim(mess1))
  endif
  ! Continuum part
  write (mess1,'(a,1x,f0.6,1x,a)')  &
                'INFO PMS: Representative frequency for continuum =',infopms%fcont,'GHz'
  call astro_message(seve%r,rname,trim(mess1))
  write (mess1,'(a,i0,1x,a,1x,f0.6,1x,a)') &
                'INFO PMS: Total bandwidth for continuum (',infopms%npolcont,' polar) =',infopms%dfcont,'MHz'
  call astro_message(seve%r,rname,trim(mess1))
  write (mess1,'(a,1x,f0.6,1x,a)') &
               'INFO PMS: Channel spacing for continuum =',infopms%rescont,'MHz'
  call astro_message(seve%r,rname,trim(mess1))
  ! Representative frequency
  if (infopms%nresol.le.0) return
  write (mess1,'(a,1x,f0.6,1x,a,1x,i0,1x,a)') &
                'INFO PMS: Representative Frequency =',infopms%frep, &
                'GHz covered by',infopms%nspw,' spectral windows'
  call astro_message(seve%r,rname,mess1)
  do ir=1,infopms%nresol
    write (mess1,'(a,1x,i0,1x,a,1x,f0.1,1x,a)') &
        'INFO PMS:',infopms%npol(ir), &
        'polarizations at hardware df=',infopms%resol(ir),'kHz'
    call astro_message(seve%r,rname,trim(mess1))
  enddo
  if (infopms%resrep.ne.0) then
    write (mess1,'(a,1x,i0,1x,a,1x,f0.1,1x,a)') &
          'INFO PMS:',infopms%npolrep, &
          'polarizations at requested df=',infopms%resrep,'kHz'
    call astro_message(seve%r,rname,trim(mess1))
  endif
  !
end subroutine noema_print_pms
!
subroutine noema_draw_frep(freq,spwout,cplot,infopms,error)
  use gbl_message
  use gkernel_interfaces
  use astro_interfaces, except_this=>noema_draw_frep
  use astro_types
  !-----------------------------------------------------------------------
  ! @ private
  ! Draw the position of a repreentative freq (might be different form tuning freq)
  ! Needed for PMS
  !-----------------------------------------------------------------------
  real(kind=8), intent(in)      :: freq
  type(spw_output_t), intent(in) :: spwout
  type(current_boxes_t), intent(inout) :: cplot
  type(info_pms_t), intent(inout)   :: infopms
  logical, intent(inout) :: error
  ! local
  character(len=*), parameter :: rname='PLOT'
  integer(kind=4), parameter :: mres=2
  integer(kind=4) :: ib,iw,nspw,nres,ir,npol(mres)
  integer(kind=4) :: repspw(spwout%n_spw),reppol(spwout%n_spw)
  character(len=5) :: replabel(spwout%n_spw)
  real(kind=8)  :: repres(spwout%n_spw),res(mres)
  type(draw_line_t) :: fline
  character(len=256) :: comm,mess,mess1
  !
  !Search the spw covering the representative frequency
  do iw=1,spwout%n_spw
    repspw(iw)=0
    repres(iw)=0
    reppol(iw)=0
  enddo
  do ir=1,mres
    npol(ir)=0
  enddo
  nspw=0
  nres=0
  do iw=1,spwout%n_spw
    if (freq.le.spwout%win(iw)%restmin.or.freq.ge.spwout%win(iw)%restmax) cycle
    nspw=nspw+1
    repspw(nspw)=iw
    repres(nspw)=spwout%win(iw)%resol
    reppol(nspw)=spwout%win(iw)%pol_code
    replabel(nspw)=spwout%win(iw)%label
    if (nspw.eq.1.or.spwout%win(iw)%resol.ne.repres(nspw-1)) then
      nres=nres+1
      res(nres)=spwout%win(iw)%resol
      infopms%resol(nres)=spwout%win(iw)%resol
    endif
  enddo
  infopms%nresol=nres
  infopms%nspw=nspw
  do ir=1,nres
    do iw=1,nspw
      if (repres(iw).eq.res(ir)) then
        npol(ir)=npol(ir)+1
        infopms%npol(ir)=npol(ir)
      endif
    enddo
  enddo
  if (nspw.eq.0) then
    call noema_default_pms(infopms,error)
    if (error) return
    call astro_message(seve%e,rname,'Representative frequency is not in the observed range')
    error=.true.
    return
  endif
  if (infopms%resrep.ne.0) then
    if (infopms%nresol.gt.1) then
      ! Required resolution wider than widest channel
      if (infopms%resrep.ge.maxval(infopms%resol(1:infopms%nresol))) then
        infopms%npolrep=maxval(infopms%npol)
      ! Required resolution narrower than narrowest channel
      else if (infopms%resrep.lt.minval(infopms%resol(1:infopms%nresol))) then
        call astro_message(seve%e,rname,'Spectral resolution is not achievable')
        error=.true.
        return
      ! Required resolution is intermediate
      else
        infopms%npolrep=0
        do ir=1,infopms%nresol
          if (infopms%resol(ir).le.infopms%resrep) then
            if (infopms%npol(ir).gt.infopms%npolrep) then
              infopms%npolrep=infopms%npol(ir)
            endif
          endif
        enddo
      endif
    else if (infopms%nresol.eq.1) then
      if (infopms%resrep.lt.infopms%resol(1)) then
        call astro_message(seve%e,rname,'Resolution not achievable at representative frequency')
        error = .true.
        return
      endif
      infopms%npolrep=infopms%npol(1)
    else
      call astro_message(seve%e,rname,'Problem with resolution at representative frequency')
      error=.true.
      return
    endif
  endif
  write (mess,'(a,1x,i0,1x,a)') 'Representative frequency is covered by',nspw,'spectral windows'
!  call astro_message(seve%i,rname,mess)
!   do iw=1,nspw
!     write (mess,'(a,i3,1x,a,1x,a,a,1x,f8.1,a)') &
!             'SPW #',repspw(iw),'in',replabel(iw),', df=',repres(iw),'kHz'
!     call astro_message(seve%i,rname,mess)
!   enddo
  write (mess1,'(a,1x,i0,1x,a,1x,f0.1,1x,a)') &
      'Representative frequency is covered by',npol(1),'polarizations at df=',res(1),'kHz'
  if (nres.eq.2) then
    write (mess1,'(a,1x,a,1x,i0,1x,a,1x,f0.1,1x,a)') &
      trim(mess1),'and',npol(2),'polarizations at df=',res(2),'kHz'
  endif
!  call astro_message(seve%i,rname,mess1)
  ! Do the plot
  do ib=1,cplot%nbox
    if (freq.lt.cplot%box(ib)%rest%xmin.or.freq.gt.cplot%box(ib)%rest%xmax) cycle
    write (comm,'(a,i0)') 'CHANGE DIRECTORY BOX',ib
    !Draw the arrow
    call gr_execl(comm)
    fline%xmin=freq
    fline%xmax=freq
    fline%ymin=cplot%box(ib)%rest%ymin
    fline%ymax=cplot%box(ib)%rest%ymin+(cplot%box(ib)%rest%ymax-cplot%box(ib)%rest%ymin)*9./16.
    fline%col=afrepcol
    fline%dash=1
    call rec_draw_arrow(fline,cplot%box(ib)%rest,error)
    if (error) return
    !Write the information
    write (comm,'(a,a,a)') 'DRAW TEXT 0 -6 "',trim(mess1),'" 6 0 /CHARACTER 1'
    call gr_pen(colour=afrepcol,error=error)
    call gr_exec1(comm)
    call gr_pen(colour=adefcol,error=error)
    call gr_execl('CHANGE DIRECTORY')
  enddo
  !
end subroutine noema_draw_frep
!
subroutine noema_draw_summary(spwout,pfx,rec,cplot,molecules,error)
  use gbl_message
  use gkernel_interfaces
  use astro_interfaces, except_this=>noema_draw_summary
  use astro_types
  !-----------------------------------------------------------------------
  ! @ private
  ! Draw coverage of spectral units independently of correlator units
  ! for summary purpose
  !
  !-----------------------------------------------------------------------
  type(spw_output_t), intent(in) :: spwout
  type(pfx_t), intent(in) :: pfx
  type(receiver_t), intent(in) :: rec
  type(current_boxes_t), intent(inout) :: cplot
  type(plot_molecules_t), intent(in) :: molecules
  logical, intent(inout) :: error
  ! local
  character(len=*), parameter :: rname='PLOT'
  integer(kind=4) :: ispw,ib,itune,ibb,overload,conflict
  integer(kind=4) ::iu,imode,it,mflex,nflex,maxbox
  real(kind=8)  :: srect,sbox,soff,spos,xt,yt,yh,yv,lowres,highres
  real(kind=4)  :: maxybox
  type(draw_rect_t) :: sprec
  type(draw_mark_t) :: mark
  logical :: dohighres,badsetup
  character(len=256) :: comm,molchar,defchar,commhigh
  !
  write (defchar,'(a,1x,f0.3)') 'SET CHARACTER',cplot%defchar
  write (molchar,'(a,1x,f0.3)') 'SET CHARACTER',cplot%molchar
  call gr_execl('CHANGE DIRECTORY')
  !
  ! check whether more resource than available are used or if chunks are used by more than 1 spw
  call noema_pfx_status(pfx,overload,conflict,error)
  if (error) return
  badsetup=(overload.gt.0).or.(conflict.gt.0)
  !
  ! Get flexible usage of correlator
  mflex=0
  nflex=0
  spos=0.
  soff=0.
  do iu=1,pfx%n_units
    imode=pfx%unit(iu)%imode
    do it=1,pfx%unit(iu)%mode(imode)%n_types
      if (.not.(pfx%unit(iu)%mode(imode)%chtype(it)%move_chunk)) cycle
      mflex=mflex+pfx%unit(iu)%mode(imode)%chtype(it)%use_chunks
      nflex=nflex+pfx%unit(iu)%mode(imode)%chtype(it)%usage
    enddo ! it
  enddo
  !
  dohighres=.false.
  lowres=spwout%win(1)%resol
  maxybox=-1.
  do ib=1,cplot%nbox
    if ((cplot%plotmode.eq.'SUMMARY_R').and.(ib.eq.1)) cycle
    ! In order to localise the higer box
    if (cplot%box(ib)%phys%ymax.gt.maxybox) then
      maxybox=cplot%box(ib)%phys%ymax
      maxbox=ib
    endif
    sbox=cplot%box(ib)%rest%ymax-cplot%box(ib)%rest%ymin
    write (comm,'(a,i0)') 'CHANGE DIRECTORY BOX',ib
    call gr_execl(comm)
    do ispw=1,spwout%n_spw
      if ((spwout%win(ispw)%restmin.ge.cplot%box(ib)%rest%xmin) &
        .and. (spwout%win(ispw)%restmax.le.cplot%box(ib)%rest%xmax)) then
        if (cplot%plotmode.eq.'SUMMARY_R') then
          spos = sbox/2d0
          if (spwout%win(ispw)%resol.eq.lowres) then
            sprec%col=alowrescol
            soff=sbox/25d0
          else
            sprec%col=ahighrescol
            soff=3*sbox/50d0
            dohighres=.true.
            highres=spwout%win(ispw)%resol
          endif
        else if (cplot%plotmode.eq.'SUMMARY_P') then
          spos=sbox
          if (spwout%win(ispw)%resol.eq.lowres) then
            sprec%col=alowrescol
            soff=sbox/10d0
          else
            sprec%col=ahighrescol
            soff=3d0*sbox/20d0
            dohighres=.true.
            highres=spwout%win(ispw)%resol
          endif
        endif
        srect=(spos-4d0*soff)/2d0
        if (spwout%win(ispw)%conflict.or.spwout%win(ispw)%overload) then
          sprec%col=aconflictcol
        endif
        sprec%xmin=spwout%win(ispw)%restmin
        sprec%xmax=spwout%win(ispw)%restmax
        if (spwout%win(ispw)%pol_code.eq.1) then
          sprec%ymax=spos-soff
          sprec%ymin=sprec%ymax-srect
          yh=(sprec%ymax+sprec%ymin)/2d0
        else if (spwout%win(ispw)%pol_code.eq.2) then
          sprec%ymin=cplot%box(ib)%rest%ymin+soff
          sprec%ymax=sprec%ymin+srect
          yv=(sprec%ymax+sprec%ymin)/2d0
        else
          call astro_message(seve%e,rname,'Problem with polarizations')
          error=.true.
          return
        endif
        call rec_draw_frect(sprec,cplot%box(ib)%rest,error)
        if (error) return
        sprec%col=achunkcol
        sprec%dash=1
        call rec_draw_rect(sprec,cplot%box(ib)%rest,error)
        if (error) return
        call gr_pen(colour=adefcol,error=error)
        if (error) return
        if (cplot%plotmode.eq.'SUMMARY_R') then
          xt=sprec%xmin
          yt=sprec%ymin
          write (comm,'(a,1x,f0.3,1x,f0.3,1x,a,i0,a)') 'DRAW TEXT',xt,yt,'"',ispw,'" 9 0 /USER'
          call gr_exec1(comm)
        endif
      endif
    enddo
    call rec_get_itune(rec,cplot%box(ib)%iband,itune,error)
    if (error) return
    if (cplot%box(ib)%sb_code.ne.0) then ! Plot confusion zone when possible
      do ibb=1,rec%desc%n_bbands
        call noema_draw_confusion(rec,itune,ibb,cplot%box(ib)%sb_code,cplot%box(ib)%rest,error)
        if (error) return
      enddo
      call gr_pen(colour=adefcol,error=error)
      if (error) return
    endif
    ! Draw molecules
    call gr_exec1(molchar)
    call rec_draw_molecules(molecules,cplot%box(ib)%rest,error)
    if (error) return
    call gr_exec1(defchar)
    ! Display tuning frequency
    if ((cplot%box(ib)%sb_code.eq.rec%tune(itune)%sb_code).or.(cplot%plotmode.eq.'SUMMARY_P')) then
      call rec_draw_linetune(rec%tune(itune),cplot%box(ib)%rest,error)
      if (error) return
    endif
    ! caption
    xt=cplot%box(ib)%rest%xmax
    write (comm,'(a,1x,f0.3,1x,f0.3,1x,a)') 'DRAW TEXT',xt,yh,'" H" 6 0 /USER'
    call gr_exec1(comm)
    write (comm,'(a,1x,f0.3,1x,f0.3,1x,a)') 'DRAW TEXT',xt,yv,'" V" 6 0 /USER'
    call gr_exec1(comm)
    xt=cplot%box(ib)%rest%xmin
    write (comm,'(a,1x,f0.3,1x,f0.3,1x,a)') 'DRAW TEXT',xt,yh,'"H " 4 0 /USER'
    call gr_exec1(comm)
    write (comm,'(a,1x,f0.3,1x,f0.3,1x,a)') 'DRAW TEXT',xt,yv,'"V " 4 0 /USER'
    call gr_exec1(comm)
    !redraw the box
    call rec_draw_fbox(cplot,ib,error)
    if (error) return
    call gr_execl('CHANGE DIRECTORY')
  enddo
  ! Caption above upper frame
  write (commhigh,'(a,i0)') 'CHANGE DIRECTORY BOX',maxbox
  call gr_execl(commhigh)
  write (comm,'(a)') 'DRAW TEXT 0 6 "Spectral windows:" 6 0 /CHARACTER 7'
  call gr_exec1(comm)
  mark%col=alowrescol
  mark%nside=4
  mark%style=3
  mark%s=cplot%defchar
  mark%x=0.5
  mark%y=5
  mark%ref='CHARACTER'
  mark%iref=7
  call rec_draw_mark(mark,error)
  if (error) return
  mark%col=adefcol
  mark%style=0
  call rec_draw_mark(mark,error)
  if (error) return
  call gr_pen(colour=adefcol,error=error)
  if (error) return
  write (comm,'(a,1x,f0.1,1x,a)') 'DRAW TEXT 1 5 "df =',lowres,'kHz" 6 0 /CHARACTER 7'
  call gr_exec1(comm)
  if (dohighres) then
    mark%col=ahighrescol
    mark%nside=4
    mark%style=3
    mark%s=cplot%defchar
    mark%x=0.5
    mark%y=4
    mark%ref='CHARACTER'
    mark%iref=7
    call rec_draw_mark(mark,error)
    if (error) return
    mark%col='BLACK'
    mark%style=0
    call rec_draw_mark(mark,error)
    if (error) return
    call gr_pen(colour=adefcol,error=error)
    if (error) return
    write (comm,'(a,1x,f0.1,1x,a)') 'DRAW TEXT 1 4 "df =',highres,'kHz" 6 0 /CHARACTER 7'
    call gr_exec1(comm)
  endif
  write (comm,'(a,1x,i0,a,i0,1x,a)') 'DRAW TEXT 0 3 "',nflex,'/',mflex,'flexible chunks used" 6 0 /CHARACTER 7'
  call gr_exec1(comm)
  call gr_execl('CHANGE DIRECTORY')
  call gr_pen(colour=adefcol,error=error)
  if (error) return
  ! Write warning message when setup is not feasible
  if (.not.(badsetup)) return
  xt=cplot%box(1)%phys%sx/4d0
  yt=5*cplot%defchar
  call gr_execl(commhigh)
  call gr_pen(colour=aconflictcol,error=error)
  if (error) return
  write (comm,'(a,1x,f0.3,1x,f0.3,1x,a)') &
  'DRAW TEXT',xt,yt,'"Setup not feasible:" 5 0 /BOX 7'
  call gr_exec1(comm)
  call gr_execl('CHANGE DIRECTORY')
  if (overload.gt.0) then
    xt=cplot%box(1)%phys%sx/4d0
    yt=3*cplot%defchar
    call gr_execl(commhigh)
    write (comm,'(a,1x,f0.3,1x,f0.3,1x,a)') &
    'DRAW TEXT',xt,yt,'"Too many chunks used" 5 0 /BOX 7'
    call gr_exec1(comm)
    call gr_execl('CHANGE DIRECTORY')
  endif
  if (conflict.gt.0) then
    xt=cplot%box(1)%phys%sx/4d0
    yt=4*cplot%defchar
    call gr_execl(commhigh)
    write (comm,'(a,1x,f0.3,1x,f0.3,1x,a,i0,1x,a)') &
    'DRAW TEXT',xt,yt,'"',conflict,'chunks in conflict" 5 0 /BOX 7'
    call gr_exec1(comm)
    call gr_execl('CHANGE DIRECTORY')
  endif
  call gr_execl('CHANGE DIRECTORY')
  call gr_pen(colour=adefcol,error=error)
  if (error) return
  !
end subroutine noema_draw_summary
!
subroutine noema_setup_file(line,error)
  use gbl_message
  use gkernel_interfaces
  use astro_interfaces, except_this=>noema_setup_file
  use astro_types
  use ast_line
  use my_receiver_globals
  use plot_molecules_globals
  !-----------------------------------------------------------------------
  ! @ private
  ! Create a sequence of commands to reach the present state based only on SPW /CH
  !
  !-----------------------------------------------------------------------
  character(len=*), intent(inout) :: line        ! command line
  logical, intent(inout) :: error
  ! local
  character(len=*), parameter :: rname='SETUP'
  integer(kind=4), parameter :: optfile=1 ! to write output in a file rather than on the screen
  logical :: dopchanged,badsetup,doobs
  integer(kind=4) :: nchar,olun,ier,iu,it,ispw,overload,conflict,ncm,im,kmol
  character(len=256) :: mess
  character(len=128) :: molfile,molfile2
  character(len=filename_length) :: file,argum
  character(len=32), allocatable  :: curr_mol(:,:)
  character(len=1)      :: dummy
  !
  !Preliminary checks
  if (rec%n_tunings.eq.0) then
    call astro_message(seve%e,rname,'No tuning found')
    error = .true.
    return
  endif
  if (.not.noema%ifproc%defined) then
    call astro_message(seve%e,rname,'Problem with IF Processor part')
    error = .true.
    return
  endif
  !
  !check that doppler did not changed since last tuning
  call rec_check_doppler(rec%source,dopchanged,error)
  if (dopchanged) then
    call astro_message(seve%e,rname,'Doppler changed since last tuning')
    call astro_message(seve%i,rname,'Nothing done')
    call rec_display_error('Doppler changed since last tuning',error)
    error = .true.
    return
  endif
  !
  ! Check status of pfx (overload or conflict)
  call noema_pfx_status(pfx,overload,conflict,error)
  if (error) return
  badsetup=.false.
  if (overload.gt.0) then
    badsetup=.true.
    call astro_message(seve%e,rname,'Setup requires more chunks than available')
  endif
  if (conflict.gt.0) then
    badsetup=.true.
    write (mess,'(i0,1x,a)') conflict,'Chunks used by more than one spectral window'
    call astro_message(seve%e,rname,mess)
  endif
  if (badsetup) then
    call astro_message(seve%e,rname,'Please solve conflicts before creating the procedure')
    error=.true.
    return
  endif
  !
  ! Output to STDOUT or file?
  molfile2='setup_molecules.lin'
  if (sic_present(optfile,0)) then  ! /FILE
    call sic_ch(line,optfile,1,argum,nchar,.true.,error)
    if (error)  return
    call sic_parse_file(argum,' ','.astro',file)
    !
    ier = sic_getlun(olun) !find a unit number
    if (ier.ne.1) then
      call astro_message(seve%e,rname,'No logical unit left')
      error = .true.
      return
    endif
    ier = sic_open(olun,file,'NEW',.false.)
    if (ier.ne.0) then
      call astro_message(seve%e,rname,'Cannot open file '//file)
      error = .true.
      call sic_frelun(olun)
      return
    endif
  else
    olun = 6  ! STDOUT
  endif
  !
  ! We are in observatory mode: no redshift for the source, all freq in LSR
  doobs=.true.
  !
  write (mess,'(a)') '! WARNING - Prototype version to set up the functionality'
  call noema_print(mess,olun)
  write (mess,'(a)') '!           Details of the syntax will be adapted to OBS'
  call noema_print(mess,olun)
  call noema_print('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!',olun)
  if (doobs.and.rec%source%z.ne.0) then
    call noema_print('SET FREQUENCY LSR',olun)
    call noema_print('! Source defined without z and use redshifted tuning frequency',olun)
  else
    call noema_print('SET FREQUENCY REST',olun)
  endif
  ! SOURCE command : mandatory to have exactly the same coverages
  call noema_setup_source(rec%source,doobs,mess,error)
  if (error) goto 10
  call noema_print(mess,olun)
  call noema_print('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!',olun)
  do it=1,rec%n_tunings
    ! TUNING COMMAND
    call noema_setup_rec(rec,it,doobs,mess,error)
    if (error) goto 10
    call noema_print(mess,olun)
    do iu=1,pfx%n_units
      !BASEBAND COMMAND
      if (pfx%unit(iu)%iband.ne.rec%tune(it)%iband) cycle
      ! Cycle if 0 spw are configure in this unit [should become a switch OFF at some point]
      if (.not.(any(spw%out%win(1:spw%out%n_spw)%label.eq.pfx%unit(iu)%label))) cycle
      write (mess,'(a,1x,a)') 'BASEBAND',pfx%unit(iu)%label
      call noema_print(mess,olun)
      do ispw=1,spw%out%n_spw
        !SPW COMMAND
        if (spw%out%win(ispw)%label.ne.pfx%unit(iu)%label) cycle
        if (.not.spw%out%win(ispw)%flexible) cycle
        write (mess,'(a,1x,i0,1x,a,1x,i0)') 'SPW /CHUNK',spw%out%win(ispw)%ich1,'TO',spw%out%win(ispw)%ich2
        call noema_print(mess,olun)
      enddo !spw
    enddo !basebands
  enddo ! tuning
  call noema_print('  ',olun)
  call noema_print('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!',olun)
  call noema_print('  ',olun)
  !
  ! Extract current lines from catalog
  ! Load molecular line [taken from older line versions]
  if (.not.sic_query_file(molecules%catalog,'data#dir:','.dat',molfile)) then
    call astro_message(seve%e,rname,'line catalog file not found')
    error=.true.
    return
  endif
  kmol = 0
  call read_lines(dummy,kmol,molfile)
  allocate(curr_mol(2,nmol))
  !
  call noema_setup_cata(spw%out,molecules,curr_mol,ncm,error)
  if (error) return
  !
  ! Write lines in data section of setup file
  write (mess,'(a,a)') 'BEGIN DATA GAG_SCRATCH:',trim(molfile2)
  call noema_print(mess,olun)
  call noema_print('! Known spectral lines in the covered frequency ranges',olun)
  do im=1,ncm
    write (mess,'(a,1x,a,a,a)') trim(curr_mol(1,im)),"'",trim(curr_mol(2,im)),"'"
    call noema_print(mess,olun)
  enddo
  write (mess,'(a,a)') 'END DATA GAG_SCRATCH:',trim(molfile2)
  call noema_print(mess,olun)
  !
  deallocate(curr_mol)
  !
10 continue
  if (olun.ne.6) then
    ier = sic_close(olun)
    call sic_frelun(olun)
  endif
  !
end subroutine noema_setup_file
!
subroutine noema_setup_cata(spwout,molecules,mlist,ncm,error)
  use gbl_message
  use gkernel_interfaces
  use astro_interfaces, except_this=>noema_setup_cata
  use astro_types
  use ast_line
  !-----------------------------------------------------------------------
  ! @ private
  ! extract from the line catalog the lines falling in the current spw
  !
  !-----------------------------------------------------------------------
  type(spw_output_t), intent(in)  :: spwout
  type(plot_molecules_t), intent(in)  :: molecules
  character(len=*), intent(inout) :: mlist(:,:)
  integer(kind=4), intent(inout) :: ncm
  logical,  intent(inout) :: error
  ! Local
  character(len=*), parameter :: rname='SETUP'
  integer(kind=4) :: im,iw,ic
  real(kind=8) :: mfreq
  !
  ic=0
  do im=1,nmol
    mfreq=molfreq(im)*mhzperghz
    do iw=1,spwout%n_spw
      if (mfreq.lt.spwout%win(iw)%restmin-molecules%width.or. &
          mfreq.gt.spwout%win(iw)%restmax+molecules%width) cycle
      ic=ic+1
      write (mlist(1,ic),'(f0.6)') molfreq(im)
      mlist(2,ic)=molname(im)
      exit ! molecule in list
    enddo ! spw
  enddo ! molecules
  ncm=ic
  !
end subroutine noema_setup_cata
!
subroutine noema_setup_source(recsou,doobs,mess,error)
  use gbl_message
  use gbl_constant
  use astro_interfaces, except_this=>noema_setup_source
  use astro_types
  !-----------------------------------------------------------------------
  ! @ private
  ! create the sourcecommand compatible with OBS [no redshift]
  !
  !-----------------------------------------------------------------------
  type(receiver_source_t), intent(in)  :: recsou
  logical,  intent(in) :: doobs
  character(len=*), intent(inout) :: mess
  character(len=128) :: mess1
  logical,  intent(inout) :: error
  !
  if (.not.recsou%nosource) then
    write (mess1,'(a,1x,a,1x,a)') 'SOURCE',trim(recsou%name),trim(recsou%coord)
    if (recsou%eq.ne.equinox_null) then
      write (mess1,'(a,1x,f0.3)') trim(mess1),recsou%eq
    endif
    write (mess1,'(a,1x,a,1x,a)') trim(mess1),trim(recsou%lambda),trim(recsou%beta)
    if (recsou%z.eq.0.or.doobs) then
      write (mess,'(a,1x,a,1x,f0.3)') trim(mess1),'LSR',recsou%vlsr
    else
      write (mess,'(a,1x,a,1x,f0.3)') trim(mess1),'RED',recsou%z
    endif
  else
    write (mess,'(a)') '! No source entered'
  endif
  !
end subroutine noema_setup_source
!
subroutine noema_setup_rec(rec,it,doobs,mess,error)
  use gbl_message
  use astro_interfaces, except_this=>noema_setup_rec
  use astro_types
  !-----------------------------------------------------------------------
  ! @ private
  ! create the tuning command compatible with OBS [no redshift]
  !
  !-----------------------------------------------------------------------
  type(receiver_t), intent(in)  :: rec
  integer(kind=4), intent(in) :: it
  logical,  intent(inout) :: doobs
  character(len=*), intent(inout) :: mess
  logical,  intent(inout) :: error
  ! Local
  real(kind=8) :: fcomm
  !
  if (doobs) then
    fcomm=rec%tune(it)%frest/(1+rec%source%z)/1d3
  else
    fcomm=rec%tune(it)%frest/1d3
  endif
  write (mess,'(a,1x,f0.6,1x,a,1x,f0.3)') &
      'TUNING',fcomm,trim(sideband(rec%tune(it)%sb_code)),rec%tune(it)%fcent
  if (rec%tune(it)%ongrid) return
  ! Add fixed freq option for tuning which are not on the grid
  write (mess,'(a,1x,a)') trim(mess),'/FIXEDFREQ'
  !
end subroutine noema_setup_rec
!
subroutine noema_print(message,olun)
  use gbl_message
  use astro_interfaces, except_this=>noema_print
  !-----------------------------------------------------------------------
  ! @ private
  ! send message to the given logical unit
  !
  !-----------------------------------------------------------------------
  character(len=*), intent(in) :: message
  integer(kind=4),  intent(in) :: olun
  ! Local
  character(len=*), parameter :: rname='SETUP'
  !
  if (olun.eq.6) then
    call astro_message(seve%r,rname,message)
  else
    write(olun,'(A)') trim(message)
  endif
  !
end subroutine noema_print
!
subroutine noema_lsrrange(spwout,pfx,rec,cplot,infopms,error)
  use gbl_message
  use gkernel_interfaces
  use astro_interfaces, except_this=>noema_lsrrange
  use astro_types
  use phys_const
  !-----------------------------------------------------------------------
  ! @ private
  ! Draw coverage of spectral units independently of correlator units
  ! for summary purpose
  !
  !-----------------------------------------------------------------------
  type(spw_output_t), intent(in) :: spwout
  type(pfx_t), intent(in) :: pfx
  type(receiver_t), intent(in) :: rec
  type(current_boxes_t), intent(in) :: cplot
  type(info_pms_t), intent(inout)   :: infopms
  logical, intent(inout) :: error
  ! local
  character(len=*), parameter :: rname='PLOT'
  !for chunk way
  integer(kind=4) :: iband,itune
  real(kind=8) :: hires,ds2,v2,z2
  character(len=5) :: spwlab
  character(len=200) :: mess,comm
  ! for spw way
  real(kind=8) :: wmin,ww
  integer(kind=4) :: iw,imin
  !
  hires=spwout%win(spwout%n_spw)%resol/1d3
  spwlab=spwout%win(spwout%n_spw)%label
  iband=spwout%win(spwout%n_spw)%iband
  call rec_get_itune(rec,iband,itune,error)
  if (error) return
  !
!   chwidth=0d0
!   do iu=1,pfx%n_units
!     if (pfx%unit(iu)%label.ne.spwlab) cycle
!     imode=pfx%unit(iu)%imode
!     do it=1,pfx%unit(iu)%mode(imode)%n_types
!       if (pfx%unit(iu)%mode(imode)%chtype(it)%df_chunks.ne.hires) cycle
!       itype=it
!       chwidth=pfx%unit(iu)%mode(imode)%chtype(it)%width_chunk
!     enddo
!   enddo
!   !
!   if (chwidth.eq.0) then
!     call astro_message(seve%e,rname,'Could not determine chunk width')
!     error=.true.
!     return
!   endif
!   !
!   lsrwidth=clight_kms*(chwidth/rec%source%lsrshift)/rec%tune(itune)%frest ! km/s
!   write (mess,'(a,1x,f0.3,1x,a)') 'Method A: 1 chunk is equivalent to',lsrwidth,'km/s in source frame'
!   call astro_message(seve%r,'rname',mess)
  !
!   !2nd method
!   ds2=(rec%tune(itune)%flsr+chwidth)/rec%tune(itune)%flsr*rec%source%lsrshift
!   if (rec%source%z.ne.0) then
!     z2=1d0/ds2-1
!     write (mess,'(a,1x,f0.6,1x,a)') 'Method B: 1 chunk is equivalent to',z2-rec%source%z,'redshift offset'
!   else
!     v2=clight_kms*(1-ds2)
!     write (mess,'(a,1x,f0.3,1x,a)') 'Method B: 1 chunk is equivalent to',v2-rec%source%vlsr,'km/s in source frame'
!   endif
!   call astro_message(seve%r,'rname',mess)
!   !
  ! Other way: width of the narrowest spw
  wmin=1e9
  do iw=1,spwout%n_spw
    ww=spwout%win(iw)%restmax-spwout%win(iw)%restmin
    if (ww.ge.wmin) cycle
    wmin=ww
    imin=iw
  enddo
  ds2=(rec%tune(itune)%flsr+wmin/2d0)/rec%tune(itune)%flsr*rec%source%lsrshift
  if (rec%source%z.ne.0) then
    z2=1d0/ds2-1
    write (mess,'(a,1x,f0.6,1x,a)') &
    'Half the most narrow SPW is equivalent toan offset of',abs(z2-rec%source%z),'in redshift'
    infopms%zrange=abs(z2-rec%source%z)
    infopms%redshift=.true.
  else
    v2=clight_kms*(1-ds2)
    write (mess,'(a,1x,f0.3,1x,a)') &
    'Half the most narrow SPW is equivalent to an offset of',abs(v2-rec%source%vlsr),'km/s in source LSR velocity'
    infopms%lsrrange=abs(v2-rec%source%vlsr)
    infopms%redshift=.false.
  endif
!   call astro_message(seve%r,'rname',mess)
  !
  if (cplot%nbox.ne.1) then
    call astro_message(seve%e,rname,'Could not determine the current plot nothing is drawn')
    error = .true.
    return
  endif
  !Write the information
  call gr_execl('CHANGE DIRECTORY BOX1')
  write (comm,'(a,a,a)') 'DRAW TEXT 0 -4 "',trim(mess),'" 6 0 /CHARACTER 1'
  call gr_pen(colour=adefcol,error=error)
  call gr_exec1(comm)
  call gr_execl('CHANGE DIRECTORY')
  !
end subroutine noema_lsrrange
