subroutine astro_observatory_command(line,error)
  use gbl_message
  use gkernel_interfaces
  use astro_interfaces, except_this=>astro_observatory_command
  use ast_line
  use ast_astro
  use my_receiver_globals
  !---------------------------------------------------------------------
  ! @ private
  ! Support routine for command
  ! OBSERVATORY [name|lon lat alt] [Sun-Avoidance]
  !---------------------------------------------------------------------
  character(len=*), intent(in)  :: line   !
  logical,          intent(out) :: error  !
  ! Local
  character(len=*), parameter :: rname='OBSERVATORY'
  character(len=12) :: argum
  integer(kind=4) :: nc,ier,nk
  character(len=24) :: clon,clat
  real(kind=8) :: alti,slim
  integer(kind=4), parameter :: mobs=33
  character(len=12) :: knownobs(mobs),previous
  ! Data
  data knownobs /  &
    'BURE','PDBI','NOEMA','PICOVELETA','VELETA',  &  ! IRAM ones first
    'ACA','ALMA','APEX','ATF','CARMA','CSO','EFFELSBERG','FCRAO','GBT','GI2T',  &
    'HHT','IOTA','JCMT','KITTPEAK','KOSMA','LASILLA','MAUNAKEA','MEDICINA',     &
    'NRO','NOBEYAMA','PARANAL','PTI','SEST','SMA','SMT','VLA','VLT','YEBES' /
  !
  previous=obsname
  error = .false.
  if (sic_narg(0).lt.2) then
    if (sic_narg(0).eq.0) then
      !
      ! No argument == decode GAG_ASTRO_OBS
      !
      ier = sic_getlog('GAG_ASTRO_OBS',argum)
      if (ier.ne.0) then
        call astro_message(seve%e,rname,  &
          'Failure in translation of GAG_ASTRO_OBS logical')
        error = .true.
        return
      endif
    else
      !
      ! Only one argument == name of observatory
      !
      call sic_ke (line,0,1,argum,nc,.true.,error)
      if (error) return
    endif
    if (argum.eq.'30M') argum='PICO'
    call sic_ambigs(rname,argum,obsname,nk,knownobs,mobs,error)
    if (error) return
    !
    call astro_observatory(obsname,error)
    if (error)  return
    if (.not.library_mode) then
      ! Astro was loaded in INTERACTIVE mode, including PDBI\ and ALMA\
      ! languages.
      select case (obsname)
      case ('NOEMA')
        ! NOEMA : language NOEMA
        call exec_string ('SIC  PICO\ OFF',error)
        call exec_string ('SIC  ALMA\ OFF',error)
        call exec_string ('SIC  PDBI\ OFF',error)
        call exec_string ('SIC  NOEMA\ ON',error)
        call exec_string ('SIC  PRIO 1 NOEMA',error)
        if (previous.ne.'NOEMA') then
          call noema_reset_setup(noema,error)
        endif
      case ('BURE','PDBI')
        call exec_string ('SIC  PICO\ OFF',error)
        call exec_string ('SIC  ALMA\ OFF',error)
        call exec_string ('SIC  NOEMA\ OFF',error)
        call exec_string ('SIC  PDBI\ ON',error)
        call exec_string ('SIC  PRIO 1 PDBI',error)
        obsname='BURE'
      case ('ALMA','ACA')
        call exec_string ('SIC  PDBI\ OFF',error)
        call exec_string ('SIC  PICO\ OFF',error)
        call exec_string ('SIC  NOEMA\ OFF',error)
        call exec_string ('SIC  ALMA\ ON',error)
        call exec_string ('SIC  PRIO 1 ALMA',error)
      case('VELETA','PICOVELETA')
        call exec_string ('SIC  PDBI\ OFF',error)
        call exec_string ('SIC  ALMA\ OFF',error)
        call exec_string ('SIC  NOEMA\ OFF',error)
        call exec_string ('SIC  PICO\ ON',error)
        call exec_string ('SIC  PRIO 1 PICO',error)
        if (previous.ne.'PICOVELETA'.and.previous.ne.'VELETA') then
          call pico_reset_emir(emir,error)
        endif
      end select
    endif
    call astro_message(seve%i,rname,'Selected '//trim(obsname)//' observatory')
  else
    !
    ! More than one argument == decode coordinates
    call sic_ch(line,0,1,clon,nc,.true.,error)
    if (error)  return
    call sic_ch(line,0,2,clat,nc,.true.,error)
    if (error)  return
    call sic_r8(line,0,3,alti,.true.,error)
    if (error) return
    slim = 30.d0
    call sic_r8(line,0,4,slim,.false.,error)
    if (error) return
    call astro_observatory(clon,clat,alti,slim,error)
    if (error)  return
  endif
  call astro_message(seve%i,rname,'Time needs to be reset')
end subroutine astro_observatory_command
!
subroutine astro_observatory_byname(arg,error)
  use gbl_message
  use gkernel_interfaces
  use astro_interfaces, except_this=>astro_observatory_byname
  use ast_astro
  !---------------------------------------------------------------------
  ! @ public-generic astro_observatory
  !  Set the Astro observatory given its name
  !---------------------------------------------------------------------
  character(len=*), intent(in)    :: arg    ! Telescope name
  logical,          intent(inout) :: error  ! Logical error flag
  ! Local
  character(len=*), parameter :: rname='OBSERVATORY'
  character(len=4) :: teles  ! Comparison is based on 4 characters
                             ! Hide this "detail" here
  real(4) :: diam
  !
  teles = arg  ! arg can be smaller or larger than 4 characters!
  freq = 100.d0
  !
  call gwcs_observatory_byname_astro(arg,lonlat,altitude,slimit,diam,error)
  select case (teles)
  case ('PARA','VLT ','VLTI')
    call astro_message(seve%i,rname,  &
      &  'Frequency set to 299.792458 THz = 1 micron')
    freq = 299792.458d0
  case ('IOTA')
    call astro_message(seve%i,rname,  &
      &  'Frequency set to 187.370286 THz = 1.6 micron')
    freq = 187370.286d0
  case ('PTI ')
    call astro_message(seve%i,rname,'Frequency set to 136.269299 THz = '//  &
       & '2.2 micron')
    freq = 136269.299d0
  case ('GI2T')
    call astro_message(seve%i,rname,'Frequency set to 500 THz = '//  &
       & '0.6 micron')
    freq = 499653.8397d0
  end select
end subroutine astro_observatory_byname
!
subroutine astro_observatory_bychcoords(clon,clat,alt,slim,error)
  use gbl_message
  use gkernel_interfaces
  use astro_interfaces, except_this=>astro_observatory_bychcoords
  !---------------------------------------------------------------------
  ! @ public-generic astro_observatory
  !  Set Astro observatory given its longitude/latitude (sexagesimal
  ! strings), altitude and avoidance
  !---------------------------------------------------------------------
  character(len=*), intent(in)    :: clon   ! Telescope longitude (sexagesimal string)
  character(len=*), intent(in)    :: clat   ! Telescope latitude (sexagesimal string)
  real(kind=8),     intent(in)    :: alt    ! Altitude [km]
  real(kind=8),     intent(in)    :: slim   ! Sun avoidance
  logical,          intent(inout) :: error  ! Logical error flag
  ! Local
  integer(kind=4) :: llon,llat
  real(kind=8) :: lon,lat
  !
  llon = lenc(clon)
  call sic_sexa(clon(1:llon),llon,lon,error)
  if (error) return
  !
  llat = lenc(clat)
  call sic_sexa(clat(1:llat),llat,lat,error)
  if (error) return
  !
  call astro_observatory_byr8coords(lon,lat,alt,slim,error)
  if (error)  return
  !
end subroutine astro_observatory_bychcoords
!
subroutine astro_observatory_byr8coords(lon,lat,alt,slim,error)
  use gbl_message
  use gkernel_interfaces
  use astro_interfaces, except_this=>astro_observatory_byr8coords
  use ast_astro
  !---------------------------------------------------------------------
  ! @ public-generic astro_observatory
  !  Set Astro observatory given its longitude/latitude (float values),
  ! altitude and avoidance
  !---------------------------------------------------------------------
  real(kind=8), intent(in)    :: lon    ! Telescope longitude [deg]
  real(kind=8), intent(in)    :: lat    ! Telescope latitude [deg]
  real(kind=8), intent(in)    :: alt    ! Altitude [km]
  real(kind=8), intent(in)    :: slim   ! Sun avoidance
  logical,      intent(inout) :: error  ! Logical error flag
  !
  real(8) :: xyz(3)
  character(len=message_length) :: mess
  !
  if (abs(lat).gt.90.d0) then
    call astro_message(seve%e,'OBSERVATORY','Latitude must be < 90 degrees')
    error = .true.
    return
  endif
  !
  lonlat(1) = lon
  lonlat(2) = lat
  altitude = alt
  slimit = slim
  obsname = "User defined"
  !
  call gwcs_lonlat2geo(lon,lat,alt,xyz)
  !
  write(mess,'(A,3(1X,F0.8))') 'XYZ',xyz
  call astro_message(seve%i,'OBSERVATORY',mess)
end subroutine astro_observatory_byr8coords
!
subroutine do_tele_beam(primbeam,beam,freq,arg)
  use gbl_message
  !---------------------------------------------------------------------
  ! @ public
  ! Default beam size according to Telescope
  ! Specific values for PICO and EFFELSBERG, otherwise just scale the
  ! BURE value.
  !---------------------------------------------------------------------
  real(kind=8),     intent(out) :: primbeam  ! Computed value
  real(kind=8),     intent(in)  :: beam      ! If not 0, return this value
  real(kind=8),     intent(in)  :: freq      ! Else, use freq+telescope
  character(len=*), intent(in)  :: arg       ! Telescope name
  !
  if (beam.eq.0.d0) then
    select case (arg)
    case ('BURE','PDBI','NOEM')
      primbeam = 56d0*(88d0/freq)
    case ('PICO','VELE')
      primbeam = 2398d0/freq   ! Value from C.Kramer
    case ('ALMA')
      primbeam = 15d0/12d0*56d0*(88d0/freq)
    case ('ACA')
      primbeam = 15d0/7d0*56d0*(88d0/freq)
    case ('ATF')
      primbeam = 15d0/12d0*56d0*(88d0/freq)
    case ('CARM')
      ! Consider here the 10.4m antennas
      primbeam = 15d0/10.4d0*56d0*(88d0/freq)
    case ('CSO ')
      primbeam = 15d0/10.4d0*56d0*(88d0/freq)
    case ('EFFE')
      primbeam = 43d0*(22d0/freq)
    case ('FCRA')
      primbeam = 15d0/14d0*56d0*(88d0/freq)
    case ('JCMT')
      primbeam = 56d0*(88d0/freq)
    case ('KITT')
      primbeam = 15d0/12d0*56d0*(88d0/freq)
    case ('SEST')
      primbeam = 56d0*(88d0/freq)
    case ('SMA ')
      primbeam = 15d0/6d0*56d0*(88d0/freq)
    case ('SMT ')
      primbeam = 15d0/10d0*56d0*(88d0/freq)
    case ('OVRO')    ! OBS NOT DEFINED !
      primbeam = 15d0/10.4d0*56d0*(88d0/freq)
    case ('45-M')    ! OBS NOT DEFINED !
      primbeam = 15d0/45d0*56d0*(88d0/freq)
    case ('NRO ')    ! OBS NOT DEFINED !
      primbeam = 15d0/10d0*56d0*(88d0/freq)
    case ('BIMA')    ! OBS NOT DEFINED !
      primbeam = 15d0/6.1d0*56d0*(88d0/freq)
    case default
      call astro_message(seve%w,'ASTRO','No default primary beam '//  &
      'available for observatory '//arg)
      primbeam = 0.d0
    end select
  else
    primbeam = beam
  endif
end subroutine do_tele_beam
