subroutine pdbi_select(line,error)
  use gkernel_interfaces
  use gbl_message
  use ast_line
  use ast_astro
  !---------------------------------------------------------------------
  ! @ private
  ! Command SELECT PDBI|ALMA epoch
  ! Used to define receiver/correlator capabilities
  !---------------------------------------------------------------------
  character(len=*), intent(in) :: line  !
  logical, intent(inout) :: error       !
  ! Local
  integer(kind=4) :: mkeys, nkey
  parameter(mkeys=5)
  character(len=12) :: arg, key, keys(mkeys)
  integer*4 :: nc,year, cycle
  character(len=*), parameter :: rname ='SELECT'
  ! Data
  data keys/'ALMA','CYCLE','PDBI','BURE','NOEMA'/
  !
  !------------------------------------------------------------------------
  !
  if (.not.sic_present(0,1)) then
     call pdbi_message(pdbi_year)
     call alma_message(alma_cycle)
     return
  endif
  !
  call sic_ke (line,0,1,arg,nc,.true.,error)
  if (error) return
  call sic_ambigs('SELECT',arg,key,nkey,keys,mkeys,error)
  !
  if (key.eq.'NOEMA') then
    !
    ! NOEMA = PDBI 2015 for the time being
    !
    if (sic_present(0,2)) then
      call sic_ch(line,0,2,arg,nc,.true.,error)
      if (error) return
      call sic_upper(arg)
      !
      if (arg.eq."NOW") then
        pdbi_year = 2015
      else
        call sic_i4(line,0,2,year,.false.,error)
        if (year.ne.2015) then
          call astro_message(seve%e,rname, &
               'Only possible year for NOEMA is 2015')
          error = .true.
          return
        endif
        pdbi_year = year
      endif
    else
      pdbi_year = 2015
    endif
    call noema_message(pdbi_year)
  elseif (key.eq.'PDBI' .or. key.eq.'BURE') then
    !
    ! BURE
    !
    call sic_ch(line,0,2,arg,nc,.true.,error)
    if (error) return
    call sic_upper(arg)
    !
    if (arg.eq."NOW") then
       pdbi_year = 2015
    else
       call sic_i4(line,0,2,year,.false.,error)
       if ((year.ne.1995).and.(year.ne.2000).and. &
            (year.ne.2006).and.(year.ne.2010).and. &
            (year.ne.2013).and.(year.ne.2015)) then
          call astro_message(seve%e,rname, &
               'Possible years are 1995 2000 2006 2010 2013 2015')
          error = .true.
          return
       endif
       pdbi_year = year
    endif
    !
    call pdbi_message(pdbi_year)
    !
  elseif (key.eq.'ALMA' .or. key.eq.'CYCLE') then
    !
    ! ALMA
    !
    call sic_ch(line,0,2,arg,nc,.true.,error)
    if (error) return
    call sic_upper(arg)
    if (arg.eq."FULL") then
       alma_cycle = 666 ! Hope full ALMA capabilities will be available in cycle 666
    else
       call sic_i4(line,0,2,cycle,.false.,error)
       if (error) return
       if (cycle.lt.0) then
          call astro_message(seve%e,rname,'ALMA Cycles start with 0')
          error = .true.
          return
       endif
    endif
    if (cycle.gt.1) cycle = 666
    alma_cycle = cycle
    !
    call alma_message(alma_cycle)
    !
  endif
end subroutine pdbi_select
!
subroutine noema_message(year)
  use gbl_message
  !---------------------------------------------------------------------
  ! @ private
  ! Describes the receivers/correlator capabilities for selected period
  !---------------------------------------------------------------------
  integer(kind=4), intent(in):: year  !
  ! Local
  character(len=64) :: ch
  character(len=*), parameter :: rname ='SELECT'
  !
  write(ch,'(A,I4)') 'NOEMA status is set to year ',year
  call astro_message(seve%i,rname,ch)
  !
  if (year.eq.2015) then
     call astro_message(seve%r,rname,'   >> NOEMA = PdBI 2015')
     call astro_message(seve%r,rname,'   >> This is the current system')
     call astro_message(seve%r,rname,'   >> New generation receivers')
     call astro_message(seve%r,rname,'   >> New LO system with 2013 update')
     call astro_message(seve%r,rname,'   >> Tuning optimized on a grid of LO frequencies')
  endif
end subroutine noema_message
!
subroutine pdbi_message(year)
  use gbl_message
  !---------------------------------------------------------------------
  ! @ private
  ! Describes the receivers/correlator capabilities for selected period
  !---------------------------------------------------------------------
  integer(kind=4), intent(in):: year  !
  ! Local
  character(len=64) :: ch
  character(len=*), parameter :: rname ='SELECT'
  !
  write(ch,'(A,I4)') 'PdBI status is set to year ',year
  call astro_message(seve%i,rname,ch)
  !
  if (year.eq.1995) then
     call astro_message(seve%r,rname,'   >> Old generation receivers')
     call astro_message(seve%r,rname,'   >> Usable IF = 100-600 MHz')
  elseif (year.eq.2000) then
     call astro_message(seve%r,rname,'   >> Old generation receivers')
     call astro_message(seve%r,rname,'   >> Usable IF = 100-680 MHz')
  elseif (year.eq.2006) then
     call astro_message(seve%r,rname,'   >> New generation receivers')
  elseif (year.eq.2010) then
     call astro_message(seve%r,rname,'   >> New generation receivers')
     call astro_message(seve%r,rname,'   >> New LO system')
  elseif (year.eq.2013) then
     call astro_message(seve%r,rname,'   >> New generation receivers')
     call astro_message(seve%r,rname,'   >> New LO system with 2013 update')
  elseif (year.eq.2015) then
     call astro_message(seve%r,rname,'   >> This is the current system')
     call astro_message(seve%r,rname,'   >> New generation receivers')
     call astro_message(seve%r,rname,'   >> New LO system with 2013 update')
     call astro_message(seve%r,rname,'   >> Tuning optimized on a grid of LO frequencies')
  endif
end subroutine pdbi_message
!
subroutine alma_message(cycle)
  use gbl_message
  !---------------------------------------------------------------------
  ! @ private
  ! Displays the selected ALMA cycle
  !---------------------------------------------------------------------
  integer(kind=4), intent(in):: cycle  !
  ! Local
  character(len=64) :: ch
  character(len=*), parameter :: rname ='SELECT'
  !
  if (cycle.eq.666) then
     write(ch,'(A)') 'Using FULL ALMA capabilities'
  else
     write(ch,'(A,i1,A)') 'Using ALMA Cycle ',cycle,' capabilities'
  endif
  call astro_message(seve%i,rname,ch)
end subroutine alma_message
