subroutine ctimen(ctime,ntime,error)
  use gbl_message
  !---------------------------------------------------------------------
  ! @ private
  ! ASTRO Utility routine
  ! Converts the TIME '12:34:56.1234' in internal code
  !  i.e.  Hours, Minutes, Seconds, and milliSeconds
  !---------------------------------------------------------------------
  character(len=*), intent(in) :: ctime          !
  integer(kind=4), intent(out) :: ntime(4)       !
  logical, intent(inout) :: error                !
  ! Local
  integer(kind=4) :: i, ndate(7), ltime
  real(kind=8) :: tt
  character(len=32) :: chain
  !------------------------------------------------------------------------
  ltime = len_trim(ctime)
  if (ctime(1:ltime).eq.'*') then
    call utc(ndate)            ! get current time
    do i = 1, 4
      ntime(i) = ndate(i+3)
    enddo
    return
  endif
  call sic_sexa (ctime,ltime,tt,error) ! TT in hours
  if (error) then
    call astro_message(seve%e,'CTIMEN','TIME conversion error')
    return
  endif
  if (tt.lt.0 .or. tt.gt.24d0) then
    write(chain,*) tt
    call astro_message(seve%e,'CTIMEN','Invalid time '//ctime(1:ltime)//' = '//chain)
    return
  endif
  ntime(1) = tt
  tt = (tt-ntime(1)) * 60.d0
  ntime(2) = tt
  tt = (tt-ntime(2)) * 60.d0
  ntime(3) = tt
  ntime(4) = (tt - ntime(3)) * 1000.d0
end subroutine ctimen
  !
subroutine ntimec(ntime,ctime,error)
  use gbl_message
  !---------------------------------------------------------------------
  ! @ private
  ! ASTRO Utility routine
  ! Converts the internal code (Hours, Minutes, Seconds, milliSeconds)
  ! to formatted TIME
  !---------------------------------------------------------------------
  character(len=*), intent(out) :: ctime        !
  integer(kind=4), intent(in) :: ntime(4)       !
  logical, intent(inout) :: error               !
  !------------------------------------------------------------------------
  integer :: ier
  !------------------------------------------------------------------------
  write(ctime,1001,iostat=ier) ntime
  if (ier.ne.0) then
    call astro_message(seve%e,'NTIMEC','TIME conversion error')
    error = .true.
    return
  endif
  1001  format(i2.2,':',i2.2,':',i2.2,'.',i3.3)
end subroutine ntimec
