subroutine result_comm(line,user_function,error)
  use gbl_message
  use gkernel_interfaces
  use class_interfaces, except_this=>result_comm
  use class_setup
  !---------------------------------------------------------------------
  ! @ private
  ! Support routine for command FIT\RESULT [N]
  !---------------------------------------------------------------------
  character(len=*), intent(in)    :: line           ! Input command line
  logical,          external      :: user_function  !
  logical,          intent(inout) :: error          ! Error flag
  ! Local
  character(len=*), parameter :: rname='RESULT'
  !
  call fit_res_comm(line,rname,.true.,user_function,error)
  if (error)  return
  !
end subroutine result_comm
!
subroutine residu_comm(line,user_function,error)
  use gbl_message
  use gkernel_interfaces
  use class_interfaces, except_this=>residu_comm
  use class_setup
  !---------------------------------------------------------------------
  ! @ private
  ! Support routine for command FIT\RESIDUALS [N]
  !---------------------------------------------------------------------
  character(len=*), intent(in)    :: line           ! Input command line
  logical,          external      :: user_function  !
  logical,          intent(inout) :: error          ! Error flag
  ! Local
  character(len=*), parameter :: rname='RESIDUAL'
  !
  call fit_res_comm(line,rname,.false.,user_function,error)
  if (error)  return
  !
end subroutine residu_comm
!
subroutine fit_res_comm(line,rname,doresu,user_function,error)
  use gbl_message
  use gkernel_interfaces
  use class_interfaces, except_this=>fit_res_comm
  use class_setup
  !---------------------------------------------------------------------
  ! @ private
  ! Support routine for commands:
  !   FIT\RESULT [N]
  !   FIT\RESIDUAL [N]
  ! Redirection relying on SET METHOD
  !---------------------------------------------------------------------
  character(len=*), intent(in)    :: line           ! Input command line
  character(len=*), intent(in)    :: rname          ! Calling routine name
  logical,          intent(in)    :: doresu         ! result or residual?
  logical,          external      :: user_function  !
  logical,          intent(inout) :: error          ! Error flag
  ! Local
  integer(kind=4) :: m
  !
  m = 0
  call sic_i4 (line,0,1,m,.false.,error)
  if (error) return
  m = max(m,0)
  !
  select case (set%method)
  case ('CONTINUUM')
    call fit_res(doresu,propoi,m,user_function,error)
  case ('GAUSS')
    call fit_res(doresu,progauss,m,user_function,error)
  case ('NH3')
    call fit_res(doresu,pronh3,m,user_function,error)
  case ('HFS')
    call fit_res(doresu,pronh3,m,user_function,error)
  case ('ABSORPTION')
    call fit_res(doresu,proabs,m,user_function,error)
  case ('SHELL')
    call fit_res(doresu,proshell,m,user_function,error)
  case default
    call class_message(seve%e,rname,'Not implemented for method '//set%method)
    error = .true.
    return
  end select
  !
end subroutine fit_res_comm
!
subroutine fit_res(doresu,fnc,m,user_function,error)
  use gildas_def
  use class_interfaces, except_this=>fit_res
  use class_data
  use class_setup
  !--------------------------------------------------------------------
  ! @ private
  ! Support routine for commands
  !  FIT\RESULT [N]
  !  FIT\RESIDUAL [N]
  !
  ! Compute the residuals from last fit
  !  M indicates which line must be subtracted. If M = 0 (default), all
  ! lines must be subtracted.
  !  Copy R into T
  !  Compute R = T - Fit(N)
  !--------------------------------------------------------------------
  logical,         intent(in)    :: doresu         ! result or residual?
  integer(kind=4), intent(in)    :: m              ! Line number
  logical,         external      :: user_function  !
  real(kind=4),    external      :: fnc            ! Function used for the fit
  logical,         intent(inout) :: error          ! Error flag
  ! Local
  real(kind=4) :: a,b,resul
  integer(kind=4) :: i
  logical :: dobase
  !
  call copyrt(user_function,'KEEP')
  !
  if (set%method.eq.'SHELL') then
    b = - r%head%spe%rchan * r%head%spe%fres
    a = r%head%spe%fres
    do i=1,r%cnchan
      if (r%spectre(i).ne.r%head%spe%bad) then
        resul = fnc(r,a*float(i)+b,m,dobase)
        if (doresu) then
          r%spectre(i) = resul
        else
          r%spectre(i) = r%spectre(i) - resul
        endif
      endif
    enddo
    !
  else
    do i=1,r%cnchan
      if (r%spectre(i).ne.r%head%spe%bad) then
        a = real(r%datax(i),kind=4)  ! data type conversion
        resul = fnc(r,a,m,dobase)
        if (doresu) then
          r%spectre(i) = resul
        else
          r%spectre(i) = r%spectre(i)-resul
        endif
      endif
    enddo
  endif
  !
  call newlimy(r,error)
  !
end subroutine fit_res
