subroutine run_sas(line,comm,error)
  use class_interfaces
  use class_addons
  !---------------------------------------------------------------------
  ! @ no-interface (because of module problems)
  ! Support routine for LAS\ language
  !---------------------------------------------------------------------
  character(len=*), intent(in)  :: line   ! Command line
  character(len=*), intent(in)  :: comm   ! Command name
  logical,          intent(out) :: error  ! Error status
  !
  call sub_sas(line,comm,error,mem_user(ip_user))
end subroutine run_sas
!
subroutine run_ana(line,comm,error)
  use class_addons
  !---------------------------------------------------------------------
  ! @ no-interface (because of module problems)
  ! Support routine for ANALYSE\ language
  !---------------------------------------------------------------------
  character(len=*), intent(inout) :: line   ! Command line
  character(len=*), intent(in)    :: comm   ! Command name
  logical,          intent(out)   :: error  ! Error status
  !
  call sub_ana(line,comm,error,mem_user(ip_user))
end subroutine run_ana
!
subroutine run_fit(line,comm,error)
  use class_addons
  !---------------------------------------------------------------------
  ! @ no-interface (because of module problems)
  ! Support routine for FIT\ language
  !---------------------------------------------------------------------
  character(len=*), intent(in)  :: line   ! Command line
  character(len=*), intent(in)  :: comm   ! Command name
  logical,          intent(out) :: error  ! Error status
  !
  call sub_fit(line,comm,error,mem_user(ip_user))
end subroutine run_fit
!
subroutine run_dec(line,comm,error)
  use class_addons
  !---------------------------------------------------------------------
  ! @ no-interface (because of module problems)
  ! Support routine for DSB2SSB\ language
  !---------------------------------------------------------------------
  character(len=*), intent(in)  :: line   ! Command line
  character(len=*), intent(in)  :: comm   ! Command name
  logical,          intent(out) :: error  ! Error status
  !
  call sub_dec(line,comm,error,mem_user(ip_user))
end subroutine run_dec
!
subroutine run_exp(line,comm,error)
  use class_addons
  !---------------------------------------------------------------------
  ! @ no-interface (because of module problems)
  ! Support routine for EXPERIMENTAL\ language
  !---------------------------------------------------------------------
  character(len=*), intent(in)  :: line   ! Command line
  character(len=*), intent(in)  :: comm   ! Command name
  logical,          intent(out) :: error  ! Error status
  !
  call sub_exp(line,comm,error,mem_user(ip_user))
end subroutine run_exp
!
subroutine sub_sas(line,comm,error,user_function)
  use gbl_constant
  use gbl_message
  use gkernel_interfaces
  use class_interfaces
  use class_data
  use class_setup
  use plot_formula
  !----------------------------------------------------------------------
  ! @ no-interface (because of argument type mismatch)
  ! CLASS Main routine for language LAS\
  ! Call appropriate subroutine according to COMM
  !----------------------------------------------------------------------
  character(len=*), intent(in)  :: line           ! Command line
  character(len=*), intent(in)  :: comm           ! Command name
  logical,          intent(out) :: error          ! Error status
  logical,          external    :: user_function  !
  ! Local
  character(len=*), parameter :: rname='LAS'
  integer(kind=4) :: i
  integer(kind=4), save :: icall=0
  !
  if (icall.ne.0) then
     Print *,'Rentrant call to RUN_LAS ',comm
     read(5,*) i
  endif
  icall = icall+1
  !
  call class_message(seve%c,rname,line)
  !
  error = .false.
  if (r%is_otf) then
     if (comm.eq.'COPY') then
        call class_copy(line,error,user_function)
     elseif (comm.eq.'FILE') then
        call class_file(line,error)
     elseif (comm.eq.'FIND') then
        call find(line,error)
     else if (comm.eq.'GET') then
        call get(line,error,user_function)
     elseif (comm.eq.'LIST') then
        call class_list(line,error)
     elseif (comm.eq.'SET') then
        call class_set_command(line,error)
     elseif (comm.eq.'SHOW') then
        call class_show_comm(line,error)
     else if (comm.eq.'WRITE') then
        call class_write_comm(line,error,user_function)
     else
        call class_message(seve%e,rname,'Command forbidden on OTF data')
        call class_message(seve%e,rname,'Use WRITE to convert this data into spectra')
        error = .true.
     endif
     icall = icall-1
     return
  endif
  !
  if (comm.eq.'FILE') then
     call class_file(line,error)
  elseif (comm.eq.'ACCUMULATE') then
     call class_accumulate(line,error,user_function)
  elseif (comm.eq.'ASSOCIATE') then
     call class_associate(line,error)
  elseif (comm.eq.'AVERAGE') then
     call class_average(line,error,user_function)
  elseif (comm.eq.'BASE') then
     call baseline (line,error,user_function)
  elseif (comm.eq.'BOX') then
     call class_box(line,error)
  elseif (comm.eq.'CATALOG') then
     call class_catalog(line,error)
  elseif (comm.eq.'CONSISTENCY') then
     call index_consistency_check(line,error,user_function)
  elseif (comm.eq.'COPY') then
     call class_copy(line,error,user_function)
  elseif (comm.eq.'DROP') then
     call class_drop(line,error)
  elseif (comm.eq.'DUMP') then
     call class_dump(line,error)
  elseif (comm.eq.'FIND') then
     call find(line,error)
  elseif (comm.eq.'EXTRACT') then
     call extract(line,error,user_function)
  elseif (comm.eq.'FOLD') then
     call class_fold(line,error,user_function)
  elseif (comm.eq.'GET') then
     call get(line,error,user_function)
  elseif (comm.eq.'HEADER') then
     call class_header(error)
  elseif (comm.eq.'LIST') then
     call class_list(line,error)
  elseif (comm.eq.'LOAD') then
     call load_2d(line,error,user_function)
  elseif (comm.eq.'MERGE') then
     call merge(error,user_function)
  elseif (comm.eq.'MULTIPLY') then
     call multi(line,error)
  elseif (comm.eq.'NEW_DATA') then
     call new_data(line,error)
  elseif (comm.eq.'PLOT') then
     call class_plot(line,error)
  elseif (comm.eq.'RECORD') then
     call class_message(seve%e,rname,'Command RECORD is obsolete in CLASS90')
     error = .true.
  elseif (comm.eq.'SAVE') then
     call class_save(line,error)
  elseif (comm.eq.'SET') then
     call class_set_command(line,error)
  elseif (comm.eq.'SHOW') then
     call class_show_comm(line,error)
  elseif (comm.eq.'SPECTRUM') then
     call class_spectrum(line,error)
  elseif (comm.eq.'STITCH') then
     call class_stitch(line,error,user_function)
  elseif (comm.eq.'SWAP') then
     if (t%head%xnum.eq.0) then
        call class_message(seve%e,rname,'No spectrum in T buffer.')
        error = .true.
        icall = icall-1
        return
     endif
     call swaprt(user_function)
  elseif (comm.eq.'TITLE') then
     call class_title(line,error)
  elseif (comm.eq.'WRITE') then
     call class_write_comm(line,error,user_function)
  elseif (comm.eq.'UPDATE') then
     ! Do not try to write it in case of error !...
     call class_update_comm(error,user_function)
  elseif (comm.eq.'MODIFY') then
     call modify(line,error)
  elseif (comm.eq.'TAG') then
     call tagout(line,error)
  elseif (comm.eq.'IGNORE') then
     call ignore(line,error)
  elseif (comm.eq.'FITS') then
     call fits_class(line,error,user_function)
  else
     call class_message(seve%e,rname,'You are using an undocumented, unsupported feature')
     call class_message(seve%e,rname,trim(comm)//' not yet implemented')
     call class_message(seve%e,rname,'Command line: '//line)
     error = .true.
  endif
  icall = icall-1
end subroutine sub_sas
!
subroutine sub_ana(line,comm,error,user_function)
  use gbl_message
  use class_interfaces
  use class_data
  !----------------------------------------------------------------------
  ! @ no-interface (because of argument type mismatch)
  ! CLASS Main routine for language ANALYSE\
  ! Call appropriate subroutine according to COMM
  !----------------------------------------------------------------------
  character(len=*), intent(inout) :: line           ! Command line (inout because of command DRAW)
  character(len=*), intent(in)    :: comm           ! Command name
  logical,          intent(out)   :: error          ! Error status
  logical,          external      :: user_function  !
  ! Local
  character(len=*), parameter :: rname='ANALYSE'
  character(len=message_length) :: mess
  integer(kind=4) :: i
  integer(kind=4), save :: icall=0
  !
  if (icall.ne.0) then
     Print *,'Rentrant call to RUN_ANA ',comm
     read(5,*) i
  endif
  !
  call class_message(seve%c,rname,line)
  !
  if (r%is_otf) then
     call class_message(seve%e,rname,'Command forbidden on OTF data')
     call class_message(seve%e,rname,'Use WRITE to convert this data into spectra')
     error = .true.
     return
  endif
  icall = icall+1
  !
  if (comm.eq.'DIVIDE') then
     call class_divide(line,error)
  elseif (comm.eq.'DRAW') then
     call class_draw(line,error)
  elseif (comm.eq.'FILL') then
     call class_fill(line,error,user_function)
  elseif (comm.eq.'FFT') then
     call class_fft(line,error,user_function)
  elseif (comm.eq.'GREG') then
     call class_greg(line,error,user_function)
  elseif (comm.eq.'NOISE') then
     call class_noise(line,error,user_function)
  elseif (comm.eq.'PRINT') then
     call class_print (line,error)
  elseif (comm.eq.'RESAMPLE') then
     call class_resample (line,error,user_function)
  elseif (comm.eq.'SMOOTH') then
     call smooth(line,error,user_function)
  elseif (comm.eq.'MAP') then
     call class_cells(line,error,user_function)
  elseif (comm.eq.'STRIP') then
     call class_strip(line,error,user_function)
  elseif (comm.eq.'TABLE') then
     call class_table(line,error,user_function)
  elseif (comm.eq.'CUBE') then
     ! Obsolete since 10-mar-2009
     call class_message(seve%e,rname,'Obsolete in CLASS90')
     call class_message(seve%e,rname,'Use TABLE and XY_MAP')
     error = .true.
     return
  elseif (comm.eq.'LMV') then
     call class_lmv(line,error,user_function)
  elseif (comm.eq.'GRID') then
     call class_message(seve%e,rname,'Obsolete in CLASS90')
     call class_message(seve%e,rname,'Use TABLE and XY_MAP')
     error = .true.
     return
  elseif (comm.eq.'STAMP') then
     call stamp(line,error,user_function)
  elseif (comm.eq.'REDUCE') then
     call class_message(seve%e,rname,'REDUCE not yet implemented')
     error = .true.
     ! if (r%head%gen%kind.eq.kind_sky) then
     !   call redsky(line,error)
     ! else
     !   call class_message(seve%w,'REDUCE','Only valid for skydips')
     ! endif
  elseif (comm.eq.'MEMORIZE') then
     call memorize(line,error)
  elseif (comm.eq.'RETRIEVE') then
     call retrieve(line,error,user_function)
  elseif (comm.eq.'POPUP') then
     call popup (line,error,user_function)
  elseif (comm.eq.'MODEL') then
     call model (line,error,user_function)
  elseif (comm.eq.'COMMENT') then
     call class_comment(line,error)
  else
     mess = 'ANALYSE'//char(92)//comm//' not yet implemented'
     call class_message(seve%i,rname,mess)
     error = .true.
  endif
  icall = icall-1
end subroutine sub_ana
!
subroutine sub_fit(line,comm,error,user_function)
  use gbl_constant
  use gbl_message
  use gkernel_interfaces
  use class_interfaces
  use class_data
  use class_setup
  use plot_formula
  !----------------------------------------------------------------------
  ! @ no-interface (because of argument type mismatch)
  ! CLASS Main routine for language FIT\
  !   Call appropriate subroutine according to COMM
  !----------------------------------------------------------------------
  character(len=*), intent(in)  :: line           ! Command line
  character(len=*), intent(in)  :: comm           ! Command name
  logical,          intent(out) :: error          ! Error status
  logical,          external    :: user_function  !
  ! Local
  character(len=*), parameter :: rname='FIT'
  character(len=message_length) :: mess
  integer(kind=4) :: i
  integer(kind=4), save :: icall=0
  !
  if (icall.ne.0) then
     Print *,'Rentrant call to RUN_FIT ',comm
     read(5,*) i
  endif
  !
  call class_message(seve%c,rname,line)
  !
  if (r%is_otf) then
     call class_message(seve%e,rname,'Command forbidden on OTF data')
     call class_message(seve%e,rname,'Use WRITE to convert this data into spectra')
     error = .true.
     return
  endif
  icall = icall+1
  !
  if (comm.eq.'DISPLAY') then
     call display(line,error)
  elseif (comm.eq.'VISUALIZE') then
     call plotfit(line,error)
  elseif (comm.eq.'MINIMIZE') then
     call minimize(line,error)
  elseif (comm.eq.'ITERATE') then
     call iterate_fit(line,error)
  elseif (comm.eq.'KEEP') then
     call keepfi(error)
  elseif (comm.eq.'LINES') then
     call fitlines(line,error)
  elseif (comm.eq.'METHOD') then
     call class_method(line,error)
  elseif (comm.eq.'RESIDUAL') then
     call residu_comm(line,user_function,error)
  elseif (comm.eq.'RESULT') then
     call result_comm(line,user_function,error)
  else
     mess = 'FIT'//char(92)//comm//' not yet implemented'
     call class_message(seve%i,rname,mess)
     error = .true.
  endif
  icall = icall-1
end subroutine sub_fit
!
subroutine sub_dec(line,comm,error,user_function)
  use gbl_constant
  use gbl_message
  use class_interfaces
  use class_data
  use class_setup
  use plot_formula
  !----------------------------------------------------------------------
  ! @ no-interface (because of argument type mismatch)
  ! CLASS Main routine for language DSB2SSB\
  !  Call appropriate subroutine according to COMM
  !----------------------------------------------------------------------
  character(len=*), intent(in)  :: line           ! Command line
  character(len=*), intent(in)  :: comm           ! Command name
  logical,          intent(out) :: error          ! Error status
  logical,          external    :: user_function  !
  ! Local
  character(len=*), parameter :: rname='DSB2SSB'
  character(len=message_length) :: mess
  integer(kind=4) :: i
  integer(kind=4), save :: icall=0
  !
  if (icall.ne.0) then
     Print *,'Rentrant call to RUN_DECONV ',comm
     read(5,*) i
  endif
  !
  call class_message(seve%c,rname,line)
  !
  if (r%is_otf) then
     call class_message(seve%e,rname,'Command forbidden on OTF data')
     call class_message(seve%e,rname,'Use WRITE to convert this data into spectra')
     error = .true.
     return
  endif
  icall = icall+1
  !
  if (comm.eq.'DECONVOLVE') then
     call deconv(line,error,user_function)
  else if (comm.eq.'INITIALIZE') then
     call deconv_init(line,error,user_function)
  else
     mess = rname//char(92)//comm//' not yet implemented'
     call class_message(seve%i,rname,mess)
     error = .true.
  endif
  icall = icall-1
end subroutine sub_dec
!
subroutine sub_exp(line,comm,error,user_function)
  use gbl_constant
  use gbl_message
  use class_interfaces
  use class_data
  use class_setup
  use plot_formula
  !----------------------------------------------------------------------
  ! @ no-interface (because of argument type mismatch)
  ! CLASS Main routine for language EXPERIMENTAL\
  !  Call appropriate subroutine according to COMM
  !----------------------------------------------------------------------
  character(len=*), intent(in)  :: line           ! Command line
  character(len=*), intent(in)  :: comm           ! Command name
  logical,          intent(out) :: error          ! Error status
  logical,          external    :: user_function  !
  ! Local
  character(len=*), parameter :: rname='EXPERIMENTAL'
  character(len=message_length) :: mess
  integer(kind=4) :: i
  integer(kind=4), save :: icall=0
  !
  if (icall.ne.0) then
     Print *,'Rentrant call to RUN_EXPERIMENTAL ',comm
     read(5,*) i
  endif
  !
  call class_message(seve%c,rname,line)
  !
  if (r%is_otf) then
     call class_message(seve%e,rname,'Command forbidden on OTF data')
     call class_message(seve%e,rname,'Use WRITE to convert this data into spectra')
     error = .true.
     return
  endif
  icall = icall+1
  !
  if (comm.eq.'DIFF') then
     call class_diff(line,error,user_function)
  elseif (comm.eq.'FILTER') then
     call class_filter(line,error)
  elseif (comm.eq.'MEDIAN') then
     call class_median(line,error)
  elseif (comm.eq.'RMS') then
     call class_rms(line,error,user_function)
  elseif (comm.eq.'SUBTRACT') then
     call class_subtract_comm(line,error,user_function)
  elseif (comm.eq.'UNBLANK') then
     call class_unblank(line,error)
  elseif (comm.eq.'UV_TABLE') then
     call class_uvt (line,error,user_function)
  elseif (comm.eq.'VARIABLE') then
     call class_variable(line,error,user_function)
  elseif (comm.eq.'WAVELET') then
     call class_wavelet(line,error,user_function)
  else
     mess = rname//char(92)//comm//' not yet implemented'
     call class_message(seve%i,rname,mess)
     error = .true.
  endif
  icall = icall-1
end subroutine sub_exp
!
function class_error()
  !---------------------------------------------------------------------
  ! @ private
  ! Support routine for the error status of the library. Does nothing
  ! here.
  !---------------------------------------------------------------------
  logical :: class_error  ! Function value on return
  !
  class_error = .false.
  !
end function class_error
