module class_index
  use classic_params
  use class_types 
  !
  type (indx_t) :: smin    ! Valeurs minimales pour FIND
  type (indx_t) :: smax    ! Valeurs maximales pour FIND
  !
  type (optimize), target, save :: ix  ! Index du fichier d'entree
  type (optimize), target, save :: ox  ! Index du fichier de sortie
  type (optimize), target, save :: cx  ! Index de la selection courante
  !
  type (flag_t), save :: flg     ! Flag for selecting data
  !
  ! Index numbers of observation in memory. If 0 : no obs in memory,
  ! if -1 : an obs in memory that  was not read in a file.
  ! knext : pointer to the next index entry to consider
  integer(kind=entry_length) :: knext          ! Pointer to next observation
  integer(kind=entry_length) :: nindex         ! Index size
  integer(kind=entry_length) :: last_xnum = 0  ! Last observation obtained by GET
end module class_index
!
module class_data
  use class_types
  use class_index
  !
  type(observation), target  :: RandT(2)  ! Target for R and T
  !
  type(observation), pointer :: R  ! R buffer (current observation)
  type(observation), pointer :: T  ! T buffer (previous observation)
  type(observation)          :: P  ! Plotted observation (from LOAD)
  !
  real(kind=xdata_kind), allocatable, target :: pdatas(:), pdatai(:), pdatav(:), pdatax(:)
  real, allocatable, target :: pdataw(:)
  real(kind=xdata_kind), allocatable, target :: qdatas(:), qdatai(:), qdatav(:), qdatax(:)
  real, allocatable, target :: qdataw(:)
  real, allocatable, target :: pdata2(:,:), qdata2(:,:)
end module class_data
!
module class_setup
  use class_types
  type(class_setup_t) :: set
  type(class_title_t) :: titre
  real(kind=4), allocatable :: window_polygon(:,:)
  !
  ! Support for SET VARIABLE
  integer(kind=4), parameter :: setvar_off=0
  integer(kind=4), parameter :: setvar_read=1
  integer(kind=4), parameter :: setvar_write=2
  integer(kind=4) :: setvar_presec(-mx_sec:0) = setvar_off  ! Activation mode for each section
end module class_setup
!
module class_virtual
  use gildas_def
  use classic_params
  use class_types
  !------------------------------------------------------------------------
  ! Support variables for the virtual buffer (SET VIRTUAL).
  !------------------------------------------------------------------------
  !
  logical,                      allocatable :: got_header(:)  ! Is the header in memory?
  integer(kind=address_length), allocatable :: v_header(:)    ! Header position in buffer
  logical,                      allocatable :: got_data(:)    ! Is the data section in memory?
  integer(kind=address_length), allocatable :: v_data(:)      ! Memory address of the data section
  !
  ! Header memory buffer
  integer(kind=4) :: header_length
  integer(kind=4) :: number_of_headers
  integer(kind=4) :: next_header
  type(header),               allocatable, save :: header_content(:)
  integer(kind=entry_length), allocatable, save :: header_xnum(:)
  type(class_user_t),         allocatable, save :: user_content(:)
  type(class_assoc_t),        allocatable, save :: assoc_content(:)
  !
  ! Data memory buffer
  integer(kind=address_length) :: data_memory_address   ! Memory address of the data buffer
  integer(kind=8)              :: data_memory_length    ! [words] Memory length of the data buffer
  integer(kind=8)              :: data_memory_offset    ! [words] Offset of the next available memory address in the data buffer
  integer(kind=4), parameter   :: n_regions=64          ! Number of regions
  integer(kind=4), parameter   :: smallest_length=128   ! [words] Min size of region
  integer(kind=4), save        :: current_region        ! Active region
  integer(kind=8), save        :: region_length         ! [words] Region length
  integer(kind=4), save        :: region_size           ! Max Number of observations per region
  integer(kind=4), allocatable, save :: number_of_obs(:) ! Cur Number of obs per region
  integer(kind=entry_length), allocatable :: data_xnum(:,:)
  !
  ! Support of GET bug corrected on 2008-11-06
  logical :: virtual_bug_20081106=.false.
  !
end module class_virtual
