subroutine class_uvt(line,error,user_function)
  use gildas_def
  use image_def
  use gbl_constant
  use gbl_message
  use class_dependencies_interfaces
  use class_interfaces, except_this=>class_uvt
  use class_data
  use class_setup
  !----------------------------------------------------------------------
  ! @ private
  ! CLASS Support routine for command
  !  UV_TABLE Name [/VERBOSE]
  ! Create a Zero Spacing UV Table from the current R spectrum
  !----------------------------------------------------------------------
  character(len=*), intent(in)  :: line           ! Input command line
  logical,          intent(out) :: error          ! Logical flag
  logical,          external    :: user_function
  ! Local
  character(len=*), parameter :: rname='UV_TABLE'
  character(len=filename_length) :: chain
  integer(kind=4) :: i,j,ier,nc
  integer(kind=4) :: chfirs, chlast, ltypem
  type(gildas) :: tab
  real(kind=4), allocatable :: dtab(:)
  real(kind=4) :: xoff,yoff,weight
  type(indx_t) :: ind
  integer(kind=4), parameter :: optverb=1  ! /VERBOSE
  !
  if (r%head%xnum.eq.0) then
    call class_message(seve%e,rname,'No R spectrum in memory')
    error = .true.
    return
  endif
  !
  call sic_ch (line,0,1,chain,nc,.true.,error)
  if (error) return
  call gildas_null(tab, type = 'UVT')
  !
  call sic_parse_file(chain,' ','.uvt',tab%file)
  !
  ! Feedback
  if (sic_present(optverb,0)) then
    call class_message(seve%i,rname,  &
      'Creating UV table '//trim(tab%file)//' from spectrum:')
    !
    call index_fromobs(r%head,ind,error)
    if (error)  return
    call out0('Terminal',0.,0.,error)
    if (error)  return
    call class_list_default(ind)
  endif
  !
  chfirs = r%cimin   ! 1
  chlast = r%cimax   ! r%cnchan
  !
  ! Prepare header
  tab%gil%ndim   = 2
  !
  tab%gil%dim(1) = 7+3*(chlast-chfirs+1)
  tab%gil%dim(2) = 1
  !
  tab%gil%ref(1) = r%head%spe%rchan-chfirs+1.d0
  tab%gil%val(1) = r%head%spe%restf ! Sky frequency
  tab%gil%inc(1) = r%head%spe%fres  ! Frequency resolution ...
  !
  tab%gil%ref(2) = 0.d0
  tab%gil%val(2) = 0.d0
  tab%gil%inc(2) = 1.d0
  !
  ! Projection information
  ltypem = r%head%pos%system
  if (ltypem.eq.type_eq) then
    tab%gil%epoc = r%head%pos%equinox
    tab%gil%ra = r%head%pos%lam
    tab%gil%dec = r%head%pos%bet
    call equ_to_gal(tab%gil%ra,tab%gil%dec,0.0,0.0,tab%gil%epoc,   &
                    tab%gil%lii,tab%gil%bii,xoff,yoff,error)
    if (error)  return
    tab%gil%a0 = tab%gil%ra
    tab%gil%d0 = tab%gil%dec
    tab%char%syst = 'EQUATORIAL'
    tab%gil%xaxi = 2
    tab%gil%yaxi = 3
  elseif (ltypem.eq.type_ga) then
    tab%gil%epoc = set%equinox
    tab%gil%lii = r%head%pos%lam
    tab%gil%bii = r%head%pos%bet
    call gal_to_equ(tab%gil%lii,tab%gil%bii,0.0,0.0,   &
                    tab%gil%ra,tab%gil%dec,xoff,yoff,tab%gil%epoc,error)
    if (error)  return
    tab%gil%a0 = tab%gil%lii
    tab%gil%d0 = tab%gil%bii
    tab%char%syst = 'GALACTIC'
    tab%gil%xaxi = 2
    tab%gil%yaxi = 3
  endif
  tab%gil%bval = r%head%spe%bad
  tab%gil%eval = -1.0
  !
  tab%char%name = r%head%pos%sourc
  !
  ! Projection
  tab%gil%ptyp = r%head%pos%proj
  tab%gil%pang = r%head%pos%projang
  tab%gil%xaxi = 2
  tab%gil%yaxi = 3
  !
  ! Spectroscopy
  tab%char%line = r%head%spe%line
  tab%gil%freq  = r%head%spe%restf
  tab%gil%voff  = r%head%spe%voff
  tab%gil%vres  = r%head%spe%vres
  tab%gil%fima  = r%head%spe%image
  tab%gil%fres  = r%head%spe%fres
  tab%gil%faxi  = 1
  !
  ! Here define the order of the basic "columns".
  ! No "extra" ones
  tab%gil%column_size = 0
  tab%gil%column_pointer(code_uvt_u) = 1
  tab%gil%column_pointer(code_uvt_v) = 2
  tab%gil%column_pointer(code_uvt_w) = 3
  tab%gil%column_pointer(code_uvt_date) = 4
  tab%gil%column_pointer(code_uvt_time) = 5
  tab%gil%column_pointer(code_uvt_anti) = 6
  tab%gil%column_pointer(code_uvt_antj) = 7
  do i=1,code_uvt_last
    if (tab%gil%column_pointer(i).ne.0) tab%gil%column_size(i) = 1
  enddo
  !
  ! Finalize the UV Table header
  tab%gil%natom = 3
  tab%gil%nstokes = 1
  tab%gil%nvisi = 1
  tab%gil%nchan = (chlast-chfirs+1)
  tab%gil%type_gdf = code_gdf_uvt
  call gdf_setuv (tab,error)
  if (error) return
  !
  ! call gdf_print_header(tab)
  ! Prepare data
  !
  allocate(dtab(tab%gil%dim(1)),stat=ier)
  if (failed_allocate(rname,'dtab',ier,error))  return
  tab%loca%size = tab%gil%dim(1)*tab%gil%dim(2)
  dtab(:) = 0.
  !
  select case (set%weigh)
  case ('S')
    call obs_weight_sigma(rname,r,weight,error)
  case ('T')
    call obs_weight_time(rname,r,weight,error)
  case ('E')
    weight = 1.
  case default
    call class_message(seve%e,rname,'SET WEIGHT '//set%weigh//' is not supported')
    error = .true.
  end select
  if (error)  return
  !
  j = 0
  do i=chfirs,chlast
    dtab(8+j) = r%spectre(i)
    if (r%spectre(i).eq.r%head%spe%bad) then
      dtab(10+j) = 0.
    else
      dtab(10+j) = weight
    endif
    j = j+3
  enddo
  call gdf_write_image(tab,dtab,error)
  if (error) then
    call class_message(seve%f,rname,'Cannot create output table')
  endif
  deallocate(dtab)
  !
end subroutine class_uvt
