module mapping_interfaces_none
  interface
    subroutine cossin(phase,rcos,rsin)
      !-------------------------------------------------------
      ! Semi-Fast,  Semi-accurate Sin/Cos pair computation
      ! using (not yet clever) interpolation from a precise
      ! loop-up table
      !
      ! A solution using Taylor expansion and symmetries
      ! would be faster and more accurate
      !-------------------------------------------------------
      real(8), intent(inout) :: phase
      real(8), intent(out) :: rcos
      real(8), intent(out) :: rsin
    end subroutine cossin
  end interface
  !
  interface
    subroutine uv_free_buffers
      use clean_def
      use clean_arrays
      !---------------------------------------------------------------------
      ! MAPPING
      !   Deallocate all UV buffers.
      !---------------------------------------------------------------------
    end subroutine uv_free_buffers
  end interface
  !
  interface
    subroutine map_message_set_id(id)
      use gbl_message
      !---------------------------------------------------------------------
      ! Alter library id into input id. Should be called by the library
      ! which wants to share its id with the current one.
      !---------------------------------------------------------------------
      integer, intent(in) :: id
    end subroutine map_message_set_id
  end interface
  !
  interface
    subroutine compact_data(error)
      use clean_def
      use clean_arrays
      use gbl_message
      logical error
    end subroutine compact_data
  end interface
  !
  interface
    subroutine clean_beam (line,error)
      use clean_def
      use clean_arrays
      use gbl_message
      character(len=*) line
      logical error
    end subroutine clean_beam
  end interface
  !
  interface
    subroutine new_dirty_beam
      use clean_def
      use clean_arrays
      !
      ! needed when a new dirty map is computed by command uv_map:
    end subroutine new_dirty_beam
  end interface
  !
  interface
    subroutine save_result(error)
      use gildas_def
      use clean_types
      use gbl_message
      logical error
    end subroutine save_result
  end interface
  !
  interface
    subroutine define_var (error)
      use gildas_def
      use clean_def
      use clean_arrays
      use clean_types
      use clean_default
      use clean_support
      use code_names
      !
      logical :: error  ! Logical error flag
    end subroutine define_var
  end interface
  !
  interface
    subroutine shift_uvdata(hx,nu,nv,visi,cs,nc,xy)
      use image_def
      use gkernel_types
      use phys_const
      !
      type(gildas), intent(inout) :: hx
      integer, intent(in) :: nu        ! Size of a visibility
      integer, intent(in) :: nv        ! Number of visibilities
      real, intent(inout) :: visi(nu,nv)  !
      real, intent(in) :: cs(2)        ! Cos/Sin of Rotation
      integer, intent(in) :: nc
      real(8), intent(in) :: xy(2,nc)  ! Position Shift
    end subroutine shift_uvdata
  end interface
  !
  interface
    subroutine maxcmp(nx,ny,wl,a)
      integer, intent(in) :: nx,ny
      real, intent(in) :: wl(2,nx,ny)
      real, intent(out) :: a
    end subroutine maxcmp
  end interface
  !
  interface
    subroutine userc (nv,coords,xref,xval,xinc,yref,yval,yinc)
      integer, intent(in) :: nv                     !
      real(4), intent(out) :: coords(2,nv)          !
      real(8), intent(in) :: xref                   !
      real(8), intent(in) :: xval                   !
      real(8), intent(in) :: xinc                   !
      real(8), intent(in) :: yref                   !
      real(8), intent(in) :: yval                   !
      real(8), intent(in) :: yinc                   !
    end subroutine userc
  end interface
  !
  interface
    subroutine mos_primary (prim,primary,bsize)
      !
      ! Mosaic and Primary beam support
      ! Compute a Gaussian primary beam
      !
      use image_def
      use gkernel_types
      type(gildas), intent(in) :: prim         ! Primary beam header
      real, intent(inout) :: primary(prim%gil%dim(1),prim%gil%dim(2)) ! Values
      real, intent(in) :: bsize                ! Primary beam size in radian
    end subroutine mos_primary
  end interface
  !
  interface
    subroutine mos_inverse (n,weight,thre)
      integer, intent(in) :: n
      real, intent(inout) :: weight(n)
      real, intent(in) :: thre
    end subroutine mos_inverse
  end interface
  !
  interface
    subroutine mos_addsq (n,nf,weight,lobe)
      integer, intent(in) :: n                      ! Size of arrays
      integer, intent(in) :: nf                     ! Number of fields
      real, intent(inout) :: weight(n)              ! Total weight
      real, intent(in) :: lobe(nf,n)                ! Primary beam array
    end subroutine mos_addsq
  end interface
  !
  interface
    subroutine connect_flux
      !
    end subroutine connect_flux
  end interface
  !
  interface
    subroutine mrc_clear
    end subroutine mrc_clear
  end interface
  !
  interface
    function minimum(n,a)
      integer n,i
      real a(*)
      real minimum
    end function minimum
  end interface
  !
  interface
    subroutine map_beams(task,map_beam,huv,nx,ny,nb,nc)
      use gbl_message
      use image_def
      !
      character(len=*), intent(in) :: task
      integer, intent(inout) :: map_beam
      type(gildas), intent(in) :: huv
      integer, intent(in) :: nx ! X size
      integer, intent(in) :: ny ! Y size
      integer, intent(in) :: nc ! Number of channels
      integer, intent(out) :: nb ! Number of beams
    end subroutine map_beams
  end interface
  !
  interface
    subroutine wipe_aper(line,error)
      use gildas_def
      use gkernel_types
      use clean_def
      use clean_arrays
      use gbl_message
      !----------------------------------------------------------------------
      ! GILDAS:  DECONV Internal routine
      !          Implementation of WIPE deconvolution algorithm,
      !          developed by Lannes et al. at OMP (Toulouse-France)
      !----------------------------------------------------------------------
      character(len=*) line
      logical error
    end subroutine wipe_aper
  end interface
  !
  interface
    subroutine wipe_beam(line,error)
      use gildas_def
      use gkernel_types
      use clean_def
      use clean_arrays
      use clean_types
      use gbl_format
      use gbl_message
      !----------------------------------------------------------------------
      ! GILDAS:  DECONV Internal routine
      !          Implementation of WIPE deconvolution algorithm,
      !          developed by Lannes et al. at OMP (Toulouse-France)
      !----------------------------------------------------------------------
      character(len=*) line
      logical error
    end subroutine wipe_beam
  end interface
  !
  interface
    subroutine wipe_error(line,error)
      use gildas_def
      use gkernel_types
      use clean_def
      use clean_arrays
      use clean_types
      use gbl_format
      use gbl_message
      !----------------------------------------------------------------------
      ! GILDAS:  DECONV Internal routine
      !          Implementation of WIPE deconvolution algorithm,
      !          developed by Lannes et al. at OMP (Toulouse-France)
      !----------------------------------------------------------------------
      character(len=*) line
      logical error
    end subroutine wipe_error
  end interface
  !
  interface
    subroutine wipe(line,error)
      !----------------------------------------------------------------------
      ! MAPPING Support routine for command
      !	WIPE [INFO | APER | BEAM | SOLVE [/INTERACTIVE] | ERROR]
      !----------------------------------------------------------------------
      character(len=*), intent(in)  :: line  ! Command line
      logical,          intent(out) :: error ! Returned status
    end subroutine wipe
  end interface
  !
  interface
    subroutine wipe_info(line,error)
      use clean_def
      use clean_arrays
      use gbl_message
      !----------------------------------------------------------------------
      ! GILDAS:  DECONV Internal routine
      !          Implementation of WIPE deconvolution algorithm,
      !          developed by Lannes et al. at OMP (Toulouse-France)
      !----------------------------------------------------------------------
      character(len=*) line
      logical error
    end subroutine wipe_info
  end interface
  !
  interface
    subroutine wipe_solve(line,error)
      use gildas_def
      use gkernel_types
      use clean_def
      use clean_arrays
      use clean_support
      use clean_types
      use gbl_format
      use gbl_message
      !----------------------------------------------------------------------
      ! GILDAS:  DECONV Internal routine
      !          Implementation of WIPE deconvolution algorithm,
      !          developed by Lannes et al. at OMP (Toulouse-France)
      !----------------------------------------------------------------------
      character(len=*) line
      logical error
    end subroutine wipe_solve
  end interface
  !
  interface
    function tab_tfbssf(f)
      real(kind=4) :: tab_tfbssf  ! Function value on return
      real(kind=4), intent(out) :: f(0:2047)
    end function tab_tfbssf
  end interface
  !
  interface
    function lenb(a)
      use gbl_message
      integer(kind=1) a(*)
      integer lenb,i
    end function lenb
  end interface
  !
  interface
    subroutine gdf_c2f_plot(action)
      use gildas_def
      !---------------------------------------------------------------------
      !
      !---------------------------------------------------------------------
      integer(kind=1) action(*)
    end subroutine gdf_c2f_plot
  end interface
  !
  interface
    subroutine get_scale (beam,nx,ny,nscale)
      integer nx,ny
      real beam(nx,ny)
      !
      real nscale
    end subroutine get_scale
  end interface
  !
  interface
    subroutine copy_map(in,out,n,nscale)
      integer n
      real out(n)
      real in(n),nscale
    end subroutine copy_map
  end interface
  !
  interface
    subroutine flux_prep(area,ratio,major,minor)
      real area,ratio,major,minor
    end subroutine flux_prep
  end interface
  !
  interface
    subroutine sg_debug(line,error)
      use clean_def
      use clean_arrays
      use clean_types
      use clean_default
      use gbl_message
      !
      character(len=*), intent(in) :: line
      logical, intent(inout) :: error
    end subroutine sg_debug
  end interface
  !
  interface
    subroutine mapping_pack_set(pack)
      use gpack_def
      !
      type(gpack_info_t), intent(out) :: pack
    end subroutine mapping_pack_set
  end interface
  !
  interface
    subroutine mapping_pack_init(gpack_id,error)
      use sic_def
      !----------------------------------------------------------------------
      !
      !----------------------------------------------------------------------
      integer :: gpack_id
      logical :: error
    end subroutine mapping_pack_init
  end interface
  !
  interface
    subroutine mapping_pack_on_exit(error)
      !----------------------------------------------------------------------
      !
      !----------------------------------------------------------------------
      logical :: error
    end subroutine mapping_pack_on_exit
  end interface
  !
  interface
    subroutine big_wait(n,b)
      real, intent(inout) :: b
      integer, intent(in) :: n
    end subroutine big_wait
  end interface
  !
end module mapping_interfaces_none
