subroutine uv_shift (new,ra,dec,ang,off,doit)
  use gkernel_interfaces
  use gkernel_types
  !-------------------------------------------------
  ! @ private
  !
  ! MAPPING
  !   Compute shift parameters
  !-------------------------------------------------
  real(kind=8), intent(inout) :: new(3)   ! New phase center and angle
  real(kind=8), intent(in) :: ra,dec,ang  ! Old center and angle
  real(kind=8), intent(out) :: off(3)     ! Required Offsets and Angle
  logical, intent(inout) :: doit          ! Do shift
  !
  real(8), parameter :: pi=3.14159265358979323846d0
  real(8), parameter :: sec=(pi/180d0/3600d0)
  type(projection_t) :: proj
  logical :: error
  !
  if (.not.doit) return
  !
  ! Check phase center agreement to 0.001" and angle to 1"
  ! These should be customizable in MAPPING
  !
  if ( abs(new(1)-ra).lt.1d-3*sec .and.   &
     &    abs(new(2)-dec).lt.1d-3*sec .and.   &
     &    abs(new(3)-ang).lt.sec ) then
    new(1) = ra
    new(2) = dec
    new(3) = ang
    doit  = .false.
    return
  endif
  !
  off(1) = ra
  off(2) = dec
  off(3) = -ang                ! GreG convention
  error = .false.
  call gwcs_projec(ra,dec,-ang,3,proj,error)  ! Previous projection system
  call abs_to_rel (proj,new(1),new(2),off(1),off(2),1)  ! Offsets of new center
  off(3) = new(3)-ang          ! Additional rotation required
  if (abs(off(3)).lt.sec) off(3) = 0.d0
end subroutine uv_shift
!
subroutine uv_shift_mosaic(line,error)
  use clean_def
  use clean_default
  use clean_arrays
  use phys_const
  use gkernel_types
  use gkernel_interfaces
  use mapping_interfaces, except_this => uv_shift_mosaic
  !-------------------------------------------------------------------
  ! @ private
  !  MAPPING
  !   Internal routine to phase shift a Mosaic
  !-------------------------------------------------------------------
  character(len=*), intent(in) :: line
  logical error
  !
  type(projection_t) :: proj
  real(8) :: pos(2)
  real(4) :: cs(2)
  real(8), allocatable :: rpos(:,:)
  real(8) :: freq
  logical :: precise=.true.
  integer :: nc,nu,nv,i,ier,loff,moff
  !
  !
  ! Nothing to do is UV data is already prepared
  if (themap%nfields.ge.0) return
  !
  ! Compute observing frequency, and new phase center in wavelengths
  pos = 0
  !
  ! Compute the mean offset as phase center position
  pos(1) = minval(themap%offxy(1,:)) + maxval(themap%offxy(1,:))
  pos(2) = minval(themap%offxy(2,:)) + maxval(themap%offxy(2,:))
  pos = pos/2
  !
  if (any(pos.ne.0d0)) then
    call gwcs_projec(huv%gil%a0,huv%gil%d0,-huv%gil%pang,huv%gil%ptyp,proj,error)
    call rel_to_abs (proj,pos(1),pos(2),huv%gil%a0,huv%gil%d0,1)
  endif
  !
  cs  = [1.0,0.0]
  !
  ! Should we play with UV buffers ?
  !
  ! o_pang = huv%gil%pang
  ! huv%gil%pang = ang*pi/180.0d0
  !
  if (precise) then
    nc = huv%gil%nchan
    allocate(rpos(2,nc),stat=ier)
    do i=1,huv%gil%nchan
      freq = gdf_uv_frequency(huv,dble(i))
      rpos(1:2,i) = - freq * f_to_k * pos(1:2)
    enddo
  else
    nc = 1
    allocate(rpos(2,1),stat=ier)
    freq = gdf_uv_frequency(huv)
    rpos(1:2,1) = - freq * f_to_k * pos(1:2)
  endif
  !
  !cs(1)  =  cos(huv%gil%pang-o_pang)
  !cs(2)  = -sin(huv%gil%pang-o_pang)
  !
  ! Recenter all channels, Loop over line table
  nu = huv%gil%dim(1)
  nv = huv%gil%nvisi
  call shift_uvdata (huv,nu,nv,duv,cs,nc,rpos)
  !
  ! PATCH: Convert the LOFF - MOFF columns into Pointing columns...
  loff = huv%gil%column_pointer(code_uvt_loff)
  moff = huv%gil%column_pointer(code_uvt_moff)
  !
  if (loff.ne.0 .or. moff.ne.0) then
    huv%gil%column_pointer(code_uvt_xoff) = loff
    huv%gil%column_pointer(code_uvt_yoff) = moff
    huv%gil%column_size(code_uvt_xoff) = 1
    huv%gil%column_size(code_uvt_yoff) = 1
    !
    huv%gil%column_pointer(code_uvt_loff) = 0
    huv%gil%column_pointer(code_uvt_moff) = 0
    huv%gil%column_size(code_uvt_loff) = 0
    huv%gil%column_size(code_uvt_moff) = 0
  endif
  !
  themap%nfields = abs(themap%nfields)
  !
end subroutine uv_shift_mosaic
!
subroutine shift_uvdata(hx,nu,nv,visi,cs,nc,xy)
  use image_def
  use gkernel_interfaces
  use gkernel_types
  use phys_const
  !
  type(gildas), intent(inout) :: hx
  integer, intent(in) :: nu        ! Size of a visibility
  integer, intent(in) :: nv        ! Number of visibilities
  real, intent(inout) :: visi(nu,nv)  !
  real, intent(in) :: cs(2)        ! Cos/Sin of Rotation
  integer, intent(in) :: nc
  real(8), intent(in) :: xy(2,nc)  ! Position Shift
  !
  integer(kind=index_length) :: i
  integer :: ix, iu, iv, jc, loff, moff
  real(8) :: phi, sphi, cphi, freq, x, y
  real :: u, v, reel, imag
  real(8), allocatable :: lm_uv(:)
  !
  iu = hx%gil%column_pointer(code_uvt_u)
  iv = hx%gil%column_pointer(code_uvt_v)
  !
  ! If there is a Phase offset column, use it
  !  integer(kind=4), parameter :: code_uvt_loff = 10 ! Phase center Offset
  !  integer(kind=4), parameter :: code_uvt_moff = 11 ! Phase center Offset
  loff = hx%gil%column_pointer(code_uvt_loff)
  moff = hx%gil%column_pointer(code_uvt_moff)
  !
  if (loff.ne.0 .or. moff.ne.0) then
    allocate(lm_uv(nc))
    if (nc.eq.1) then
      freq = gdf_uv_frequency(hx)
      lm_uv(1) = freq * f_to_k
    else
      do i=1,hx%gil%nchan
        freq = gdf_uv_frequency(hx,dble(i))
        lm_uv(i) = freq * f_to_k
      enddo
    endif
    !
    do i = 1,nv
      u = visi(iu,i)
      v = visi(iv,i)
      visi(iu,i) = cs(1)*u - cs(2)*v
      visi(iv,i) = cs(2)*u + cs(1)*v
      !
      if (nc.eq.1) then
        x = xy(1,1) + lm_uv(1) * visi(loff,i)
        y = xy(2,1) + lm_uv(1) * visi(moff,i)
        phi = x*u + y*v
        cphi = cos(phi)
        sphi = sin(phi)
      endif
      !
      do jc=1,hx%gil%nchan
        ix = hx%gil%fcol+(jc-1)*hx%gil%natom
        if (nc.gt.1) then
          x = xy(1,jc) + lm_uv(jc) * visi(loff,i)
          y = xy(2,jc) + lm_uv(jc) * visi(moff,i)
          phi = x*u + y*v
          cphi = cos(phi)
          sphi = sin(phi)
        endif
        reel = visi(ix,i) * cphi - visi(ix+1,i) * sphi
        imag = visi(ix,i) * sphi + visi(ix+1,i) * cphi
        visi(ix,i) = reel
        visi(ix+1,i) = imag
      enddo
    enddo
    !
  else if (xy(1,1).eq.0. .and. xy(2,1).eq.0.) then
    do i = 1,nv
      u = visi(iu,i)
      v = visi(iv,i)
      visi(iu,i) = cs(1)*u - cs(2)*v
      visi(iv,i) = cs(2)*u + cs(1)*v
    enddo
  else
    do i = 1,nv
      u = visi(iu,i)
      v = visi(iv,i)
      visi(iu,i) = cs(1)*u - cs(2)*v
      visi(iv,i) = cs(2)*u + cs(1)*v
      !
      if (nc.eq.1) then
        phi = xy(1,1)*u + xy(2,1)*v
        cphi = cos(phi)
        sphi = sin(phi)
      endif
      !
      do jc=1,hx%gil%nchan
        ix = hx%gil%fcol+(jc-1)*hx%gil%natom
        if (nc.gt.1) then
          phi = xy(1,jc)*u + xy(2,jc)*v
          cphi = cos(phi)
          sphi = sin(phi)
        endif
        reel = visi(ix,i) * cphi - visi(ix+1,i) * sphi
        imag = visi(ix,i) * sphi + visi(ix+1,i) * cphi
        visi(ix,i) = reel
        visi(ix+1,i) = imag
      enddo
    enddo
  endif
end subroutine shift_uvdata
