subroutine com_support(line,error)
  use gildas_def
  use gkernel_interfaces
  use mapping_interfaces, only : map_message
  use clean_arrays
  use clean_support
  use gbl_message
  !----------------------------------------------------------------------
  ! @ private
  !
  ! MAPPING Support routine for command
  !	  SUPPORT
  ! 1         [/PLOT]
  ! 2         [/CURSOR]
  ! 3         [/RESET]
  ! 4         [/MASK]
  ! 5         [/VARIABLE]
  !----------------------------------------------------------------------
  character(len=*), intent(in)  :: line  ! Commandand line
  logical,          intent(out) :: error ! Logical error flag
  !
  ! Local variables
  character(len=*), parameter :: rname='SUPPORT'
  integer(kind=4), parameter :: optname=0
  integer(kind=4), parameter :: optplot=1
  integer(kind=4), parameter :: optcurs=2
  integer(kind=4), parameter :: optrese=3
  integer(kind=4), parameter :: optmask=4
  integer(kind=4), parameter :: optvari=5
  logical :: fromfile
  character(len=filename_length) :: file
  integer(kind=4) :: ier
  real(kind=4), allocatable, target :: imask(:,:)
  !
  call sic_delvariable('CLEAN_MASK',.false.,error)
  !
  if (sic_present(optcurs,0)) then  ! /CURSOR
     call map_message(seve%i,rname,'Use cursor to define polygon')
     user_method%do_mask = .true.
     file = ''
     fromfile = .false.
     !
  else if (sic_present(optrese,0)) then  ! /RESET
     if (allocated(dmask)) then
        deallocate(dmask,stat=ier)
     endif
     call greg_poly_reset(supportpol,supportvar,error)
     if (error)  return
     user_method%do_mask = .true.  ! The mask HAS changed...
     return
     !
  else if (sic_present(optmask,0)) then  ! /MASK
     if (allocated(dmask)) then
        call sic_def_real ('CLEAN_MASK',dmask,hmask%gil%ndim,   &
             &        hmask%gil%dim,.true.,error)
        user_method%do_mask = .true.
        !
        ! Show it if asked for by /PLOT option
        if (sic_present(1,0)) then
           allocate (imask(hmask%gil%dim(1),hmask%gil%dim(2)),stat=ier)
           imask = 0
           where (dmask.ne.0) imask = 1
           call sic_def_real ('BITMAP_MASK',imask,hmask%gil%ndim,   &
             &        hmask%gil%dim,.true.,error)
           call gr_exec2('PLOT BITMAP_MASK /SCALING LIN -0.5 1.5')
           error = gr_error()
           call sic_delvariable('BITMAP_MASK',.false.,error)
           deallocate (imask,stat=ier)
        endif
     endif
     return
  else
     !
     ! This also re-instate the Polygon as the Clean support
     user_method%do_mask = .true.
     if (sic_narg(optname).ge.1) then
        call greg_poly_parsename(line,optname,optvari,fromfile,file,error)
        if (error) return
     else
        fromfile = .false.
        file = ''  ! Not from file, not from var, i.e. from cursor
     endif
  endif
  !
  ! Define the polygon from file or cursor:
  call greg_poly_define(rname,file,fromfile,supportpol,supportvar,error)
  if (error)  return
  !
  if (sic_present(optplot,0)) then  ! /PLOT
    call greg_poly_plot(supportpol,error)
    if (error)  return
  endif
  !
  ! One would need to transfer the polygon to the Mapping support area
  ! here...
end subroutine com_support
