!
subroutine no_check_mask(method,head)
  use clean_def
  use image_def
  !--------------------------------------------------------------
  ! @ private
  !
  ! MAPPING   Clean/Mosaic
  !     Check that the search mask is defined
  !--------------------------------------------------------------
  type(gildas),    intent(in)    :: head   !
  type(clean_par), intent(inout) :: method !
  !
  Print *,'Called check_mask'
end subroutine no_check_mask
!
subroutine check_mask(method,head)
  use clean_def
  use image_def
  use gbl_message
  use gkernel_types
  use gkernel_interfaces, no_interface=>gr4_slmsk
  use mapping_interfaces, except_this=>check_mask
  use clean_support
  !--------------------------------------------------------------
  ! @ private
  !
  ! MAPPING   Clean/Mosaic
  !     Check that the search mask is defined
  !--------------------------------------------------------------
  type(gildas),    intent(in)    :: head   !
  type(clean_par), intent(inout) :: method !
  !
  ! Local variables
  integer, save :: last_box(4)
  integer nx,ny
  !
  logical found
  integer :: memory(1)
  integer(kind=address_length) :: ip
  type(sic_descriptor_t) :: desc
  character(len=10) :: rname = 'CHECK_MASK'
  !
  ! Mask
  nx = head%gil%dim(1)
  ny = head%gil%dim(2)
  if (method%do_mask) then
    if (.not.associated(method%mask)) then
      call map_message(seve%f,rname,'Association problem')
      method%box = last_box
      return
    endif
    call sic_descriptor('CLEAN_MASK',desc,found)
    if (found) then
      if (desc%dims(1).ne.nx .or. desc%dims(2).ne.ny) then
        call map_message(seve%e,rname,'Mask has wrong sizes')
        method%box = last_box
        return
      endif
      !
      ! Setup the Logical mask, and its bounding box
      !
      ip = gag_pointer(desc%addr,memory)
      call gr4_slmsk (memory(ip),method%mask,nx,ny,method%box)
    else
      !
      ! Compute the logical mask from the bounding polygon
      !
      call gr8_glmsk(supportpol,method%mask,nx,ny,   &
                     head%gil%convert(1,1),head%gil%convert(1,2),method%box)
    endif
    method%do_mask = .false.
    call get_mask(method%mask,nx*ny,method%nlist)
    call get_list(method%mask,nx*ny,method%list,method%nlist)
    last_box = method%box
  else
    method%box = last_box
  endif
end subroutine check_mask
