subroutine uv_reweight_comm(line,error)
  use gkernel_interfaces
  use mapping_interfaces, except_this=>uv_reweight_comm
  use clean_def
  use clean_arrays
  use gbl_message
  use phys_const
  !------------------------------------------------------------------------
  ! @ private
  !
  ! MAPPING
  !   Reweight the data
  !------------------------------------------------------------------------
  character(len=*), intent(in) :: line  ! Command line
  logical, intent(out) :: error         ! Error flag
  !
  character(len=11) :: rname = 'UV_REWEIGHT'
  character(len=80) :: chain
  integer(kind=index_length) :: iv
  integer :: ic
  real(4) :: scale
  !
  if (huv%loca%size.eq.0) then
    call map_message(seve%e,rname,'No UV data loaded')
    error = .true.
    return
  endif
  !
  error = .false.
  call sic_r4(line,0,1,scale,.true.,error)
  if (error) return
  !
  do iv=1,huv%gil%nvisi
    do ic=1,huv%gil%nchan
      duv(3*ic+huv%gil%fcol-1,iv) = scale * duv(3*ic+huv%gil%fcol-1,iv)
    enddo
  enddo
  do_weig = .true.
end subroutine uv_reweight_comm
!
subroutine uv_resample_comm(line,comm,error)
  use gkernel_interfaces
  use mapping_interfaces, except_this=>uv_resample_comm
  use clean_def
  use clean_arrays
  use gbl_message
  use phys_const
  !---------------------------------------------------------------------
  ! @ private
  !
  ! MAPPING
  !   Resample in velocity the UV Table or compress it
  !   Support for commands
  !     UV_RESAMPLE NC [Ref Val Inc]
  !     UV_COMPRESS NC
  !---------------------------------------------------------------------
  character(len=*), intent(in) :: line  ! Command line
  character(len=*), intent(in) :: comm  ! Calling command
  logical, intent(out) :: error         ! Error flag
  !
  character(len=11) :: rname = 'UV_RESAMPLE'
  character(len=80) :: chain
  real(8) :: convert(3)
  integer :: nt, nc, nchan
  type (gildas) :: uvout
  integer :: nu, nv, ni, i
  !
  if (huv%loca%size.eq.0) then
    call map_message(seve%e,'UV_MAP','No UV data loaded')
    error = .true.
    return
  endif
  !
  error = .false.
  ! No argument: get back to the original UV data
  if (sic_narg(0).eq.0) then
    call uv_reset_buffer(rname)
    do_weig = .true.
    return
  endif
  !
  call sic_i4(line,0,1,nc,.true.,error)
  if (error) return
  !
  ! Compute the Frequency Conversion from the Velocity Conversion
  call gildas_null(uvout, type = 'UVT')
  call gdf_copy_header(huvi,uvout,error)
  !
  if (comm.eq.'UV_RESAMPLE') then
    if (nc.gt.huvi%gil%nchan) nc = huvi%gil%nchan
    ! Set default resampling
    convert = huvi%gil%convert(:,1)
    !
    call sic_r8(line,0,2,convert(1),.false.,error)
    if (error) return
    call sic_r8(line,0,3,convert(2),.false.,error)
    if (error) return
    call sic_r8(line,0,4,convert(3),.false.,error)
    if (error) return
    ! Convert(3) is initially on a Velocity Scale.
    uvout%gil%voff = convert(2)
    uvout%gil%vres = convert(3)
    uvout%gil%fres = - uvout%gil%vres * 1d3 / clight * uvout%gil%freq
    uvout%gil%convert(1,1) = convert(1)
    uvout%gil%convert(3,1) = uvout%gil%fres  ! For convenience, not needed
    uvout%gil%nchan = nc
    !
  else if (comm.eq.'UV_COMPRESS') then
    if (nc.gt.huvi%gil%nchan) nc = huvi%gil%nchan
    write(chain,'(A,I6,A)') 'Averaging by chunks of ',nc,' channels'
    call map_message(seve%i,comm,chain)
    uvout%gil%inc(1) = uvout%gil%inc(1)*nc
    uvout%gil%ref(1) = (2.0*uvout%gil%ref(1)+nc-1.0)/2/nc
    uvout%gil%vres = nc*uvout%gil%vres
    uvout%gil%fres = nc*uvout%gil%fres
    ! Change the number of channels
    uvout%gil%nchan = uvout%gil%nchan/nc
  else
    call map_message(seve%e,rname,'Unrecognized choice '//comm)
  endif
  !
  ! Work with complete structure
  uvout%gil%dim(1) = 3*uvout%gil%nchan + 7 + uvout%gil%ntrail ! and nothing more ?...
  if (uvout%gil%ntrail.ne.0) then
    do i=1,code_uvt_last
      if (uvout%gil%column_pointer(i).gt.uvout%gil%lcol) uvout%gil%column_pointer(i) =  &
        & uvout%gil%column_pointer(i) + uvout%gil%dim(1) - huvi%gil%dim(1)
    enddo
  endif
  !
  ni = huvi%gil%dim(1)
  nu = uvout%gil%dim(1)
  nv = uvout%gil%dim(2)
  nt = uvout%gil%ntrail
  nchan = uvout%gil%nchan
  !
  ! Prepare appropriate array...
  call uv_select_buffer(rname,nu,nv,error)
  if (error) return
  !
  ! Now resample
  if (comm.eq.'UV_RESAMPLE') then
    call resample_uv (huvi,uvout,duvi,duv,nt)
  else if (comm.eq.'UV_COMPRESS') then
    call compress_uv (duv,nu,nv,nchan,duvi,ni,nc,nt)
  endif
  !
  ! Set header
  !
  call gdf_copy_header(uvout,huv,error)
  !
  if (allocated(duvt)) deallocate (duvt)            ! UV data not plotted
  uv_plotted = .false.
  do_weig = .true.  ! Recompute weight
  !
  ! Redefine SIC variables ?
  call sic_delvariable ('UV',.false.,error)
  call sic_def_real ('UV',duv,huv%gil%ndim,   &
                    huv%gil%dim,.true.,error)
end subroutine uv_resample_comm
!
