#include <stdio.h>
#include <math.h>
#include "wipe.h"

#ifndef IABS
#define IABS(x) (((x) >= 0) ? (x) : -(x))
#endif
#ifndef  M_PI
#define  M_PI   3.14159265358979323846
#endif

/*****************************************************************************/
/*****************************************************************************/
void    wipe_analysis(
int      *P_nx,        /* number  of pixels in the X direction               */
int      *P_ny,        /* number  of pixels in the Y direction               */
int      *P_NT,        /* maximum number of eigenvectors to compute          */
int      *P_M,         /* oversampling factor of the object workspace (input)*/
float    *P_eps_mu,    /* stop test value for relative variation of mu (input)*/
float    *P_eps_xn,    /* stop test value for relative variation of xn (input)*/
int      *Hr,          /* synthetic aperture (nx*ny)                          */
int      *D,           /* effective support constraint (ny*ny)                */
float    *DM,          /* Dusty Map (nx*ny)                                   */
float    *NM,          /* Neat Map (nx*ny)                                    */
float    *DB,          /* Dirty BEAM (4*nx*ny)                                */
float    *theta,       /* angles between the solution & eigenvectors (NT)     */
float    *lambda,      /* eigenvalues (NT)                                    */
float    *mode,        /* eigenmodes (NT*nx*ny)                               */
char     *work)        /* work space: 11*NX*NY+NT*NX*NY+NT*NT+4*NT            */
/******************************************************************************/
/******************************************************************************/
{
extern   void     dir_fft2D_real();
extern   void     inv_fft2D_real();
extern   int      ql_tridiag();
/**/
register int      i, j, k, l, is2, js2;
int      nx=*P_nx, ny=*P_ny, nxy=nx*ny;
int      n1x=2*nx, n1y=2*ny, n1xy=n1x*n1y;
int      n1xs2=n1x/2, n1ys2=n1y/2, n1xs4=n1x/4, n1ys4=n1y/4;
int      NT=*P_NT, un=1, dim_krylov=0;
int      r1, rn1, q1, qq1, r2, rn2, q2, qq2, one_x, one_y;
int      M = *P_M;
float    xn_xn, dn_zn, rn_rn, rn_rn_sav, tmp1, tmp2;
float    omega, omega_sav = 1.0, gamma, gamma_sav = 0.0;
float    mu = 1.0, kappa;
float    tab_step_x, tab_step_y, FUNCT_x[2049], FUNCT_y[2049];
float    *TMP, *rn, *dn, *zn, *Rn, *VP, *B0, *B1, *C0, *C1, *DTP;
float    eps_mu = *P_eps_mu, eps_xn = *P_eps_xn;
/******************************************************************************/

eps_mu *= 0.01;
eps_xn *= 0.01;

/******* computing addresses of the different internal arrays ***
*******/
k = 0;
TMP = (float*) &work[k]; k += n1x*n1y*sizeof(float);
rn  = (float*) &work[k]; k += nx*ny*sizeof(float);
dn  = (float*) &work[k]; k += nx*ny*sizeof(float);
zn  = (float*) &work[k]; k += nx*ny*sizeof(float);
DTP = (float*) &work[k]; k += n1x*n1y*sizeof(float);
Rn  = (float*) &work[k]; k += NT*nx*ny*sizeof(float);
VP  = (float*) &work[k]; k += NT*NT*sizeof(float);
B0  = (float*) &work[k]; k += NT*sizeof(float);
B1  = (float*) &work[k]; k += NT*sizeof(float);
C0  = (float*) &work[k]; k += NT*sizeof(float);
C1  = (float*) &work[k]; k += NT*sizeof(float);

/******* compute DTP (Dirty Transfer Pattern) using FFT from the received DB
*******/
for (i=0; i<4*nxy; i++) DTP[i] = DB[i];
dir_fft2D_real(DTP,n1x,n1y);

tab_step_x = tab_tfbssf(FUNCT_x);
tab_step_y = tab_tfbssf(FUNCT_y);
one_x = ((int) (tab_step_x));
one_y = ((int) (tab_step_y));
for (k=0; k<2048; k++) 
    {
    FUNCT_x[k] *= FUNCT_x[k];
    FUNCT_y[k] *= FUNCT_y[k];
    }

k = 0;
for (i=0; i<n1y; i++)
  {
  is2 = i/2;
  q2 = i - n1ys2;
  r2 = q2 % M;
  q2 = IABS(q2);
  rn2 = (q2 - n1y) % M;
  qq2 = ((int) ((tab_step_y*q2)/n1y));
  for (j=0; j<n1x; j++)
    {
    js2 = j/2;
    q1 = j - n1xs2;
    r1 = q1 % M;
    q1 = IABS(q1);
    rn1 = (q1 - n1x) % M;
    qq1 = ((int) ((tab_step_x*q1)/n1x));
    if (((float)Hr[nx*is2+js2] < 0.5) && (r1 == 0) && (r2 == 0))
      {
      tmp1 = FUNCT_x[qq1]*FUNCT_y[qq2];
      k++;
      }
    else tmp1 = 0.0;
    if ((q1 < n1xs4) && (q2 >= n1ys4))
      {
      if ((r1 == 0) && (rn2 == 0)) 
         tmp1 += (FUNCT_x[qq1]*FUNCT_y[one_y-qq2]);
      }
    else if ((q1 >= n1xs4) && (q2 < n1ys4))
      {
      if ((rn1 == 0) && (r2 == 0)) 
         tmp1 += (FUNCT_y[qq2]*FUNCT_x[one_x-qq1]);
      }
    else if ((q1 >= n1xs4) && (q2 >= n1ys4))
      {
      if ((r1  == 0) && (rn2 == 0)) 
         tmp1 += (FUNCT_x[qq1]*FUNCT_y[one_y-qq2]);
      if ((rn1 == 0) && (r2  == 0)) 
         tmp1 += (FUNCT_y[qq2]*FUNCT_x[one_x-qq1]);
      if ((rn1 == 0) && (rn2 == 0)) 
         tmp1 += (FUNCT_x[one_x-qq1]*FUNCT_y[one_y-qq2]);
      }
    if (j <= n1xs2)
      {
      if ((0 <= i) && (i <= n1ys2)) 
         DTP[n1x*(n1ys2-i)+n1xs2-j] += tmp1*(M*M);
      }
    else
      {
      if ((0 < i) && (i < n1ys2)) 
         DTP[n1x*(n1ys2-i)+3*n1xs2-j] += tmp1*(M*M);
      }
    }
  }

/******* initializing Conjugate Gradients: NM = 0 ************************
*******/
for (i=0; i<nxy; i++)
  {
  NM[i] = 0.0;
  dn[i] = rn[i] = (float)D[i]*DM[i];
  }

/******* Conjugate Gradients iterations ************************************
*******/
l = 0;
while (l < NT) 
  {
  /***** zn = A*A.dn ***********************************************************
  *****/
  /* dn mis dans TMP */
  for (i=0; i<n1xy; i++)  TMP[i] = 0.0;
  for (i=0; i<nx; i++)   
  for (j=0; j<ny; j++) TMP[n1x*(i+n1ys4)+j+n1xs4] = dn[nx*i+j];

  /* FFT of TMP */
  dir_fft2D_real(TMP,n1x,n1y);  

  /* product of TMP and DTP */
  TMP[0] = TMP[0]*DTP[0];
  TMP[n1x*n1ys2] = TMP[n1x*n1ys2]*DTP[n1x*n1ys2];
  TMP[n1xs2] = TMP[n1xs2]*DTP[n1xs2];
  TMP[n1x*n1ys2+n1xs2] = TMP[n1x*n1ys2+n1xs2]*DTP[n1x*n1ys2+n1xs2];
  for (j=1; j<n1xs2; j++)
    {
    tmp1 = TMP[j]; tmp2 = TMP[n1x-j];
    TMP[j] = tmp1*DTP[j] - tmp2*DTP[n1x-j];
    TMP[n1x-j] = tmp1*DTP[n1x-j] + tmp2*DTP[j];
    tmp1 = TMP[n1x*n1ys2+j]; tmp2 = TMP[n1x*n1ys2+n1x-j];
    TMP[n1x*n1ys2+j] = tmp1*DTP[n1x*n1ys2+j]-tmp2*DTP[n1x*n1ys2+n1x-j];
    TMP[n1x*n1ys2+n1x-j] = tmp1*DTP[n1x*n1ys2+n1x-j]+tmp2*DTP[n1x*n1ys2+j];
    }
  for (i=1; i<n1ys2; i++)
  for (j=0; j<n1x; j++)
    {
    tmp1 = TMP[n1x*i+j]; tmp2 = TMP[n1x*(n1y-i)+j];
    TMP[n1x*i+j] = tmp1*DTP[n1x*i+j] - tmp2*DTP[n1x*(n1y-i)+j];
    TMP[n1x*(n1y-i)+j] = tmp1*DTP[n1x*(n1y-i)+j] + tmp2*DTP[n1x*i+j];
    }

  /* inverse FFT of TMP */
  inv_fft2D_real(TMP,n1x,n1y);  

  /* storing TMP in zn (truncated by the support D) */
  for (i=0; i<ny; i++)
  for (j=0; j<nx; j++) 
      zn[nx*i+j] = (float)D[nx*i+j]*TMP[n1x*(i+n1ys4)+j+n1xs4];
      
  /***** (rn|rn), (dn|zn) ******************************************************
  *****/
  rn_rn = 0.0; /* (rn|rn) */
  dn_zn = 0.0; /* (dn|zn) */
  for (i=0; i<nxy; i++)
    {
    rn_rn += rn[i]*rn[i];
    dn_zn += dn[i]*zn[i];
    }

  /***** omega *****************************************************************
  *****/
  omega = rn_rn/dn_zn;
  rn_rn_sav = rn_rn;

  /***** normalizing residu rn **************************************************
  *****/
  rn_rn = sqrt(rn_rn);
  for (i=0; i<ny; i++)
  for (j=0; j<nx; j++) Rn[nxy*l+nx*i+j] = rn[nx*i+j]/rn_rn;

  /***** building Krylov tridiagonal matrix ***************************
  *****/
  B0[l] = (1.0/omega) + (gamma_sav/omega_sav);
  B1[l] = -sqrt(gamma_sav)/omega_sav;
  dim_krylov = l+1;

  /***** updating NM & rn, (rn|rn) & (xn|xn) ****************************
  *****/
  rn_rn = 0.0; /* (rn|rn) */
  xn_xn = 0.0; /* (xn|xn) */
  for (i=0; i<nxy; i++)
    {
    NM[i] += omega*dn[i];
    rn[i] -= omega*zn[i];
    rn_rn += rn[i]*rn[i];
    xn_xn += NM[i]*NM[i];
    }
    
  /***** gamma *****************************************************************
  *****/
  gamma = rn_rn/rn_rn_sav;
   
  /***** mis a jour de dn ******************************************************
  *****/
  for (i=0; i<nxy; i++)  dn[i] = rn[i] + gamma*dn[i];

  /***** saving omega & gamma *********************************************
  *****/
  omega_sav = omega;
  gamma_sav = gamma;

  /***** exit tests *********************************************************
  *****/
  if (dim_krylov > 1)
    {
    /* computing eigenvalues of A*A
    */
    for (i=0; i<dim_krylov; i++) 
        {C0[i] = B0[i]; C1[i] = B1[i];}
    ql_tridiag(C0,C1,NULL,dim_krylov,NT);

    /* condition number: kappa
    */
    kappa = sqrt(C0[dim_krylov-1] / C0[0]);
    printf("\nKrylov %d: mu = %e, mu' = %e, kappa = %e", 
            dim_krylov,C0[0],C0[dim_krylov-1],kappa);

    /* test on mu precision; stop test for CG
    */
    if (((mu-C0[0]) < eps_mu) && ((sqrt(rn_rn/xn_xn)/mu) < eps_xn)) 
       break;

    /* new estimate of the smallest eigenvalue: mu
    */
    mu = C0[0];
    }
        
  /***** next iteration  *************************************************
  *****/
  l++;
  }

/******* computing eigenvalues & eigenvectors of A*A *********************
*******/
ql_tridiag(B0,B1,VP,dim_krylov,NT);
for (l=0; l<dim_krylov; l++) lambda[l] = B0[l];

/******* computing eigenvalues *******************************************
*******/
dim_krylov--;
for (l=0; l<dim_krylov; l++)
  {
  for (i=0; i<nxy; i++)  
      mode[nxy*l+i] = 0.0;
  for (k=0; k<dim_krylov; k++)
    {
    for (i=0; i<nxy; i++)
      mode[nxy*l+i] += VP[NT*k+l]*Rn[nxy*k+i];
    }
  }
/******* computing angle between solution and eigenmodes **************
*******/
tmp1 = 0.0; /* (xn|xn) */
for (i=0; i<nxy; i++)
    tmp1 += NM[i]*NM[i];
for (l=0; l<dim_krylov; l++) 
  {
  tmp2 = 0.0; /* (xn|mode) */
  for (i=0; i<nxy; i++)
      tmp2 += NM[i]*mode[nxy*l+i];
  theta[l] = fabs(tmp2)/sqrt(tmp1); /* |(xn|mode)|/||xn|| */
  }

printf("\n");
for (i=0; i<dim_krylov; i++)
  {
  theta[i] = (acos(theta[i])*180./M_PI);
  printf("\nlambda[%d] = %f\ttheta[%d] = %f",i,lambda[i],i,theta[i]);
  }
printf("\n");
for (i=0; i<3; i++)
  {
if (i == 0)
  gdf_c2f_plot("_mp0");
else if (i == 1)
  gdf_c2f_plot("_mp1");
else if (i == 2)
  gdf_c2f_plot("_mp2");
  }
gdf_c2f_plot("_nm");
 printf("\n\n");
}
