subroutine wipe_aper(line,error)
  use gildas_def
  use gkernel_types
  use gkernel_interfaces
  use clean_def
  use clean_arrays
  use gbl_message
  !----------------------------------------------------------------------
  ! GILDAS:  DECONV Internal routine
  !          Implementation of WIPE deconvolution algorithm,
  !          developed by Lannes et al. at OMP (Toulouse-France)
  !----------------------------------------------------------------------
  character(len=*) line
  logical error
  !
  real(8), parameter :: pi=3.14159265358979323846d0
  real(8), parameter :: clight=299792458d-6
  include 'gbl_memory.inc'
  !
  ! Local variables:
  logical refresh,found
  integer map_size(2),ndim
  integer(kind=index_length) :: dim(4)
  integer nx,ny,nc,nv,nplanes,proc,size
  integer size_vu,size_he
  real    scale_x,scale_y,wipe_eta
  real(8) freq
  real    aperture(2),map_cell(2),ax_scale,ay_scale
  real    wcol,major,minor,angle,theta
  real(8) :: rproc
  character(len=message_length) :: mess
  character(len=4) :: rname = 'WIPE'
  !
  integer(kind=address_length) he,hew,hr,dr,vu,vv,vn,addr
  type(sic_descriptor_t) :: desc
  !
  call sic_get_inte('WIPE%PROC',proc,error)
  if (error) goto 102
  call sic_get_inte('MAP_SIZE[1]',map_size(1),error)
  if (error) goto 102
  call sic_get_inte('MAP_SIZE[2]',map_size(2),error)
  if (error) goto 102
  call sic_get_real('MAP_CELL[1]',map_cell(1),error)
  if (error) goto 102
  call sic_get_real('MAP_CELL[2]',map_cell(2),error)
  if (error) goto 102
  call sic_get_real('WIPE%ETA',wipe_eta,error)
  if (error) goto 102
  call sic_get_real('WIPE%THETA',theta,error)
  if (error) goto 102
  !
  ! Testing sic variables
  !
  if ((map_cell(1).eq.0).or.(map_cell(2).eq.0)) then
    call map_message(seve%e,rname,'MAP_CELL undefined')
    error = .true.
    return
  endif
  if ((map_size(1).eq.0).or.(map_size(2).eq.0)) then
    call map_message(seve%e,rname,'MAP_SIZE undefined')
    error = .true.
    return
  endif
  if (wipe_eta.eq.0) then
    call map_message(seve%e,rname,'WIPE%ETA undefined')
    error = .true.
    return
  endif
  call sic_get_real('WCOL',wcol,error)
  if (theta.lt.-90 .or. theta.gt.90) then
    call map_message(seve%e,rname,'Rotation angle out of range')
    error = .true.
    return
  endif
  !
  ! Redefine some parameters
  map_cell(1) = map_cell(1)*pi/180.0/3600.0   ! In radians
  map_cell(2) = map_cell(2)*pi/180.0/3600.0   ! In radians
  nx = map_size(1)
  ny = map_size(2)
  !
  nc = huv%gil%nchan
  nplanes = huv%gil%dim(1)
  nv = huv%gil%dim(2)
  if ((proc.lt.0).or.(proc.gt.nc)) then
    call map_message(seve%e,rname,'WIPE%PROC out of expected range')
    error = .true.
    return
  elseif  (proc.eq.0) then
    proc = 1
  endif
  if (wcol.lt.0.or.wcol.gt.nc) then
    call map_message(seve%e,rname,'Invalid weight channel WCOL')
    error = .true.
    return
  elseif  (wcol.eq.0) then
    wcol = proc
  endif
  !
  ! Compute observing sky frequency for U,V cell size
  rproc = proc
  freq = gdf_uv_frequency(huv,rproc)
  scale_x = clight/freq/map_cell(1)
  scale_y = clight/freq/map_cell(2)
  ! Set scale of uv coverage in meters for ticks labeLIng
  if (.not.sic_varexist('WIPE%SX')) then
    call map_message(seve%e,rname,'WIPE%SX undefined')
    error = .true.
    return
  endif
  call sic_let_real('WIPE%SX',scale_x,error)
  call sic_let_real('WIPE%SY',scale_y,error)
  !
  if (.not.sic_varexist('WIPE%MAJOR')) then
    call map_message(seve%e,rname,'WIPE%MAJOR undefined')
    error = .true.
    return
  endif
  if (.not.sic_varexist('WIPE%MINOR')) then
    call map_message(seve%e,rname,'WIPE%MINOR undefined')
    error = .true.
    return
  endif
  if (.not.sic_varexist('WIPE%ANGLE')) then
    call map_message(seve%e,rname,'WIPE%ANGLE undefined')
    error = .true.
    return
  endif
  !
  !  Refreshing or not?
  refresh = sic_present(0,1)
  !
  !     Memory allocation
  !
  !  allocating first time and reusing 2nd time if size identical
  !  otherwise freeing memory and reallocating for VU,VV,VN
  size_vu = 5*nv
  call sic_descriptor('WIPE%VU',desc,found)
  if (.not.found) then
    if (sic_getvm(size_vu,addr).ne.1) goto 99
    vu = gag_pointer(addr,memory)
    vv = vu+2*nv
    vn = vv+2*nv
    ndim = 1
    dim(1) = 2*nv
    call sic_def_real_addr('WIPE%VU',memory(vu),ndim,dim,.true.,error)
    call sic_def_real_addr('WIPE%VV',memory(vv),ndim,dim,.true.,error)
    dim(1) = nv
    call sic_def_real_addr('WIPE%VN',memory(vn),ndim,dim,.true.,error)
  else
    call sic_descriptor('WIPE%VU',desc,found)
    size = desc%dims(1)
    ! total size since we have 2 vectors
    size = 2.5*size
    if (size.eq.size_vu) then
      vu = gag_pointer(desc%addr,memory)
      call sic_descriptor ('WIPE%VV',desc,found)
      vv = gag_pointer(desc%addr,memory)
      call sic_descriptor ('WIPE%VN',desc,found)
      vn = gag_pointer(desc%addr,memory)
    else
      call sic_descriptor('WIPE%VU',desc,found)
      vu = desc%addr
      size = 2.5*desc%dims(1)
      call sic_delvariable('WIPE%VU',.false.,error)
      call sic_delvariable('WIPE%VV',.false.,error)
      call sic_delvariable('WIPE%VN',.false.,error)
      call free_vm(size,vu)
      if (sic_getvm(size_vu,addr).ne.1) goto 99
      vu = gag_pointer(addr,memory)
      vv = vu+2*nv
      vn = vv+2*nv
      ndim = 1
      dim(1) = 2*nv
      call sic_def_real_addr('WIPE%VU',memory(vu),ndim,dim,.true.,error)
      call sic_def_real_addr('WIPE%VV',memory(vv),ndim,dim,.true.,error)
      dim(1) = nv
      call sic_def_real_addr('WIPE%VN',memory(vn),ndim,dim,.true.,error)
    endif
  endif
  !
  !  allocating first time and reusing 2nd time if size identical
  !  otherwise freeing memory and reallocating for HE,HEW,HR,DR
  size_he = 4*nx*ny
  call sic_descriptor('WIPE%HE',desc,found)
  if (.not.found) then
    if (sic_getvm(size_he,addr).ne.1) goto 99
    he = gag_pointer(addr,memory)
    hew = he+nx*ny
    hr = hew+nx*ny
    dr = hr+nx*ny
    ndim = 2
    dim(1) = nx
    dim(2) = ny
    call sic_def_inte_addr('WIPE%HE', memory(he), ndim,dim,.true.,error)
    call sic_def_real_addr('WIPE%HEW',memory(hew),ndim,dim,.true.,error)
    call sic_def_inte_addr('WIPE%HR', memory(hr), ndim,dim,.true.,error)
    call sic_def_inte_addr('WIPE%DR', memory(dr), ndim,dim,.true.,error)
  else
    size = desc%dims(1)*desc%dims(2)
    ! Taking into account the fact that we have 4 arrays
    size = 4*size
    if (size.eq.size_he) then
      he = gag_pointer(desc%addr,memory)
      call sic_descriptor ('WIPE%HEW',desc,found)
      hew = gag_pointer(desc%addr,memory)
      call sic_descriptor ('WIPE%HR',desc,found)
      hr = gag_pointer(desc%addr,memory)
      call sic_descriptor ('WIPE%DR',desc,found)
      dr = gag_pointer(desc%addr,memory)
    else
      call sic_descriptor('WIPE%HE',desc,found)
      he = gag_pointer(desc%addr,memory)
      call sic_delvariable('WIPE%HE',.false.,error)
      call sic_delvariable('WIPE%HEW',.false.,error)
      call sic_delvariable('WIPE%HR',.false.,error)
      call sic_delvariable('WIPE%DR',.false.,error)
      call free_vm(size,desc%addr)
      if (sic_getvm(size_he,addr).ne.1) goto 99
      he = gag_pointer(addr,memory)
      hew = he+nx*ny
      hr = hew+nx*ny
      dr = hr+nx*ny
      ndim = 2
      dim(1) = nx
      dim(2) = ny
      call sic_def_inte_addr('WIPE%HE', memory(he), ndim,dim,.true.,error)
      call sic_def_real_addr('WIPE%HEW',memory(hew),ndim,dim,.true.,error)
      call sic_def_inte_addr('WIPE%HR', memory(hr), ndim,dim,.true.,error)
      call sic_def_inte_addr('WIPE%DR', memory(dr), ndim,dim,.true.,error)
    endif
  endif
  !
  ! Load UV data
  !
  write(mess,101) 'Processing plane number ',proc
  call map_message(seve%i,rname,mess)
  call get_uv(nv,memory(vu),memory(vv),memory(vn),duv,   &
     &    nplanes,wcol,scale_x,scale_y)
  !
  !  Initialisations before calling WIPE_APERTURE
  !
  call sic_get_real('WIPE%APER[1]',aperture(1),error)
  call sic_get_real('WIPE%APER[2]',aperture(2),error)
  ax_scale = aperture(1)
  ay_scale = aperture(2)
  !
  ! computes dusty map, neat beam, dusty beam, synthetic aperture
  !
  call wipe_aperture (   &     !
     &    nx,   &              ! Number of pixels in X (input)
     &    ny,   &              ! Number of pixels in X (input)
     &    nv,   &              ! number of visibilities (input)
     &    ax_scale,   &        ! expansion parameters of the axis of the
     &    ay_scale,   &        ! ellipsoide corresponding to HR (input)
     &    major,   &           ! major axis of the ellipse HR (output)
     &    minor,   &           ! minor axis of the ellipse HR (output)
     &    angle,   &           ! angle of the ellipse HR (output)
     &    theta,   &           ! Rotation angle to apply to default aperture
     &    wipe_eta,   &        ! mesure of interpolation to perform (input)
     &    memory(vu),   &      ! U coordinate of the uv plane (input)
     &    memory(vv),   &      ! V coordinate of the uv plane (input)
     &    memory(vn),   &      ! Sigma on visibility (1/sqrt(2) (input)
     &    memory(he),   &      ! Experimental aperture (output)
     &    memory(hew),   &     ! Weighted experimental aperture (output)
     &    memory(hr),   &      ! Synthesized aperture (output)
     &    memory(dr))
  !
  call sic_let_real('WIPE%MAJOR',major,error)
  call sic_let_real('WIPE%MINOR',minor,error)
  call sic_let_real('WIPE%ANGLE',angle,error)
  !
  if (.not.gtexist('<APER')) then
    call exec_program('CREATE DIREC <APER /GEOMETRY 768 384 /PLOT_PAGE 20 10')
  endif
  call exec_program('@ wipe_aper.map')
  if (.not.refresh) then
    call exec_program('@ x_wipe_aper.map')
  endif
  return
  !
  99    call map_message(seve%e,rname,'Memory allocation failure')
  error = .true.
  return
  !
  101   format(a,i10)
  102   call map_message(seve%e,rname,'SIC variable not defined')
  return
end subroutine wipe_aper
!
