subroutine wipe_beam(line,error)
  use gildas_def
  use gkernel_types
  use gkernel_interfaces
  use clean_def
  use clean_arrays
  use clean_types
  use gbl_format
  use gbl_message
  !----------------------------------------------------------------------
  ! GILDAS:  DECONV Internal routine
  !          Implementation of WIPE deconvolution algorithm,
  !          developed by Lannes et al. at OMP (Toulouse-France)
  !----------------------------------------------------------------------
  character(len=*) line
  logical error
  !
  real(8), parameter :: pi=3.14159265358979323846d0
  include 'gbl_memory.inc'
  !
  ! Local variables:
  logical found
  character weight_mode*2
  character(len=message_length) :: chain
  integer map_size(2),wcol,proc,ier,ndim,overs,size_ipw
  integer(kind=index_length) :: dim(4)
  integer nx,ny,nc,nv,nplanes,size,npm,word,one
  parameter(word=1)
  real vref,voff,vinc,map_cell(2),thresh,dynamic
  real nscale,ratio,f_major,f_minor,major,minor,angle
  real cpu1,cpu0
  !
  real scale_x,scale_y,eps
  real(kind=8), parameter :: clight=299792458d-6  ! Frequency in MHz
  real(kind=8) freq
  integer(kind=address_length) hr,dr,nb,db,vu,vv,vn,vs,vk
  integer(kind=address_length) addr,addr_ipw
  type(sic_descriptor_t) :: desc
  integer(kind=address_length) ip_w,ipw
  character(len=4) :: rname = 'WIPE'
  !
  ! Have we passed wipe_aper already?
  !
  if (.not.sic_varexist('WIPE%HR')) then
    call map_message(seve%e,rname,'You need to run WIPE_APER first.')
    error = .true.
    return
  endif
  !
  !  Reading and testing SIC variables
  !
  call sic_get_inte('MAP_SIZE[1]',map_size(1),error)
  if (error) then
    call map_message(seve%e,rname,'MAP_SIZE[1] undefined')
    return
  endif
  call sic_get_inte('MAP_SIZE[2]',map_size(2),error)
  if (error) then
    call map_message(seve%e,rname,'MAP_SIZE[2] undefined')
    return
  endif
  call sic_get_real('MAP_CELL[1]',map_cell(1),error)
  if ((map_cell(1).eq.0).or.error) then
    call map_message(seve%e,rname,'MAP_CELL undefined')
    return
  endif
  call sic_get_real('MAP_CELL[2]',map_cell(2),error)
  if ((map_cell(2).eq.0).or.error) then
    call map_message(seve%e,rname,'MAP_CELL undefined')
    return
  endif
  call sic_get_char('WEIGHT_MODE',weight_mode,ier,error)
  if (error) then
    call map_message(seve%e,rname,'WEIGHT_MODE undefined')
    return
  endif
  call sic_get_inte('WIPE%OVERS',overs,error)
  if (error.or.(overs.lt.0)) then
    call map_message(seve%e,rname,'WIPE%OVERS undefined')
    return
  endif
  call sic_get_real('WIPE%EPS',eps,error)
  if ((eps.lt.0).or.error) then
    call map_message(seve%e,rname,'WIPE%EPS undefined')
    return
  endif
  call sic_get_inte('WIPE%NPM',npm,error)
  if (error.or.(npm.lt.0)) then
    call map_message(seve%e,rname,'WIPE%NPM undefined')
    return
  endif
  call sic_get_real('WIPE%THRESH',thresh,error)
  if ((thresh.lt.0).or.error) then
    call map_message(seve%e,rname,'WIPE%THRESH undefined')
    return
  endif
  call sic_get_real('WIPE%DYNAMIC',dynamic,error)
  if ((dynamic.lt.0).or.error) then
    call map_message(seve%e,rname,'WIPE%DYNAMIC undefined')
    return
  endif
  call sic_get_real('WIPE%MAJOR',major,error)
  if ((major.le.0).or.error) then
    call map_message(seve%e,rname,'WIPE%MAJOR undefined')
    return
  endif
  call sic_get_real('WIPE%MINOR',minor,error)
  if ((minor.le.0).or.error) then
    call map_message(seve%e,rname,'WIPE%MINOR undefined')
    return
  endif
  call sic_get_real('WIPE%ANGLE',angle,error)
  if (error) then
    call map_message(seve%e,rname,'WIPE%ANGLE undefined')
    return
  endif
  !
  ! Redefine some parameters
  map_cell(1) = map_cell(1)*pi/180.0/3600.0   ! In radians
  map_cell(2) = map_cell(2)*pi/180.0/3600.0   ! In radians
  nx = map_size(1)
  ny = map_size(2)
  addr = locstr(weight_mode)
  ip_w = bytpnt(addr,membyt)
  !
  nc = (huv%gil%dim(1)-7)/3
  nplanes = huv%gil%dim(1)
  nv = huv%gil%dim(2)
  !
  call sic_get_inte('WCOL',wcol,error)
  if ((wcol.lt.0).or.(wcol.gt.nc)) then
    call map_message(seve%e,rname,'Current Working Plane: WCOL undefined')
    return
  endif
  call sic_get_inte('WIPE%PROC',proc,error)
  if ((proc.lt.0).or.(proc.gt.nc)) then
    call map_message(seve%e,rname,'Current Working Plane: PROC undefined')
    return
  elseif  (proc.eq.0) then
    proc = 1
  endif
  if (wcol.eq.0) wcol = 1
  !
  ! grabbing and allocating additional SIC  variables
  call sic_descriptor ('WIPE%HR',desc,found)
  if (desc%dims(1)*desc%dims(2).ne.nx*ny) then
    call map_message(seve%e,rname,'Sizes have changed!')
    error = .true.
    return
  endif
  hr = gag_pointer(desc%addr,memory)
  call sic_descriptor ('WIPE%DR',desc,found)
  if (.not.found) goto 102
  dr = gag_pointer(desc%addr,memory)
  call sic_descriptor ('WIPE%VU',desc,found)
  if (.not.found) goto 102
  vu = gag_pointer(desc%addr,memory)
  call sic_descriptor ('WIPE%VV',desc,found)
  if (.not.found) goto 102
  vv = gag_pointer(desc%addr,memory)
  call sic_descriptor ('WIPE%VN',desc,found)
  if (.not.found) goto 102
  vn = gag_pointer(desc%addr,memory)
  !
  ! Compute observing sky frequency for U,V cell size
  freq = gdf_uv_frequency(huv, dble(proc))
  scale_x = clight/freq/map_cell(1)
  scale_y = clight/freq/map_cell(2)
  !
  ! Compute image attributes
  !
  ! Beam allocation
  call gdf_copy_header(huv,hbeam,error)
  !
  hbeam%gil%ndim = 2
  hbeam%gil%dim(1) = nx
  hbeam%gil%dim(2) = ny
  hbeam%gil%dim(3) = 1
  hbeam%gil%dim(4) = 1
  vref = huv%gil%convert(1,1)
  voff = huv%gil%convert(2,1)
  vinc = huv%gil%convert(3,1)
  hbeam%gil%convert(1,1) = nx/2+1
  hbeam%gil%convert(1,2) = ny/2+1
  hbeam%gil%convert(2,1) = 0
  hbeam%gil%convert(2,2) = 0
  hbeam%gil%convert(3,1) = -map_cell(1)    ! Assume EQUATORIAL system
  hbeam%gil%convert(3,2) = map_cell(2)
  hbeam%gil%proj_words = 0
  hbeam%gil%extr_words = 0
  hbeam%char%code(1) = 'ANGLE'
  hbeam%char%code(2) = 'ANGLE'
  hbeam%loca%size = nx*ny
  hbeam%char%type = 'GILDAS_IMAGE'
  !
  ! New Beam place
  if (allocated(dbeam)) then
    call sic_delvariable ('BEAM',.false.,error)
    deallocate(dbeam)
  endif
  allocate(dbeam(nx,ny,1,1),stat=ier)
  call sic_def_real ('BEAM',dbeam,hbeam%gil%ndim,   &
     &    hbeam%gil%dim,.true.,error)
  hbeam%r3d => dbeam(:,:,:,1)
  !
  call map_message(seve%i,rname,'Creating beam ')
  !
  ! allocating mandatory SIC variable
  call sic_descriptor ('WIPE%NB',desc,found)
  size = desc%dims(1)*desc%dims(2)
  size = 5*size
  call sic_descriptor ('WIPE%VS',desc,found)
  size = size+2*desc%dims(1)
  call sic_descriptor ('WIPE%NB',desc,found)
  if (found) then
    if (size.ne.5*nx*ny+2*nv) then
      call sic_delvariable('WIPE%DB',.false.,error)
      call sic_delvariable('WIPE%VS',.false.,error)
      call sic_delvariable('WIPE%VK',.false.,error)
      !
      call sic_descriptor ('WIPE%NB',desc,found)
      call free_vm(size,desc%addr)
      call sic_delvariable('WIPE%NB',.false.,error)
      !
      size = 5*nx*ny+2*nv
      ier = sic_getvm(size,addr)
      if (ier.ne.1) goto 99
      nb = gag_pointer(addr,memory)
      db = nb+nx*ny
      vs = db+4*nx*ny
      vk = vs+nv
      !
      ndim = 2
      dim(1) = nx
      dim(2) = ny
      call sic_def_real_addr ('WIPE%NB',memory(nb),ndim,dim,.true.,error)
      dim(1) = 2*nx
      dim(2) = 2*ny
      call sic_def_real_addr ('WIPE%DB',memory(db),ndim,dim,.true.,error)
      ndim = 1
      dim(1) = nv
      call sic_def_real_addr ('WIPE%VS',memory(vs),ndim,dim,.true.,error)
      call sic_def_real_addr ('WIPE%VK',memory(vk),ndim,dim,.true.,error)
    else
      call sic_descriptor ('WIPE%NB',desc,found)
      nb = gag_pointer(desc%addr,memory)
      db = nb+nx*ny
      vs = db+4*nx*ny
      vk = vs+nv
    endif
  else
    size = 5*nx*ny+2*nv
    ier = sic_getvm(size,addr)
    if (ier.ne.1) goto 99
    nb = gag_pointer(addr,memory)
    db = nb+nx*ny
    vs = db+4*nx*ny
    vk = vs+nv
    !
    ndim = 2
    dim(1) = nx
    dim(2) = ny
    call sic_def_real_addr ('WIPE%NB',memory(nb),ndim,dim,.true.,error)
    dim(1) = 2*nx
    dim(2) = 2*ny
    call sic_def_real_addr ('WIPE%DB',memory(db),ndim,dim,.true.,error)
    ndim = 1
    dim(1) = nv
    call sic_def_real_addr ('WIPE%VS',memory(vs),ndim,dim,.true.,error)
    call sic_def_real_addr ('WIPE%VK',memory(vk),ndim,dim,.true.,error)
  endif
  !
  ! Allocating Work Space and pointers
  !
  size_ipw = (18*nx*ny+2*nx+2*ny+9*nv)*word
  if (sic_getvm(size_ipw,addr_ipw).ne.1) goto 99
  ipw = gag_pointer(addr_ipw,memory)
  !
  write(chain,101) 'Using weight plane number     ',wcol
  call map_message(seve%i,rname,chain)
  !
  ! computes dusty map, neat beam, dusty beam
  !
  call map_message(seve%i,rname,'Computing Dusty Map, Dirty Beam, Neat Beam.')
  call map_message(seve%i,rname,'Please wait, this may take some time...')
  call gag_cpu(cpu0)
  call wipe_beams (   &        !
     &    nx,   &              ! Number of pixels in X (input)
     &    ny,   &              ! Number of pixels in X (input)
     &    nv,   &              ! number of visibilities (input)
     &    overs,   &           ! oversampling factor of the object workspace
     &    eps,   &             ! test value for power method stop (input)
     &    npm,   &             ! maximum number of iterations of power method (input)
     &    thresh,   &          ! threshold for computation of FT of NB on Le+
     &    dynamic,   &         ! maximum dynamic allowed for weight function
     &    memory(hr),   &      ! Synthesized aperture (output)
     &    memory(dr),   &      ! Synthesized aperture (output)
     &    memory(db),   &      ! Dusty beam (output)
     &    memory(nb),   &      ! Neat beam (output)
     &    memory(vu),   &      ! U coordinate (input/output)
     &    memory(vv),   &      ! V coordinate (input/output)
     &    memory(vn),   &      ! Sigma on visibility (1|sqrt(2) (input/output)
     &    memory(vs),   &      ! regularization function on visibilities
     &    memory(vk),   &      ! weight function (output)
     &    membyt(ip_w),   &    ! weight mode: natural or uniform (input)
     &    memory(ipw))
  !
  ! Displaying Neat Beam, Dusty Beam and Dusty Map
  !
  call gag_cpu(cpu1)
  write(chain,'(A,F9.2)') 'Finished Dusty',cpu1-cpu0
  call map_message(seve%i,rname,chain)
  !
  call free_vm(size_ipw,addr_ipw)
  one = 1
  call gdf_f2f_plot('_db')
  call gdf_f2f_plot('_nb')
  !
  ! Get scale (map unit to Jy/Beam) from Neat Beam
  call get_scale(memory(nb),nx,ny,nscale)
  !
  size = nx*ny
  call copy_map(memory(nb),dbeam,size,nscale)
  !
  ! computing right parameters for flux calculation
  ratio = major/minor
  nscale = nscale*abs(hbeam%gil%convert(3,1)*hbeam%gil%convert(3,2))
  call flux_prep(nscale,ratio,f_major,f_minor)
  !
  !?      BEAM_MAJO = F_MAJOR
  !?      BEAM_MINO = F_MINOR
  !?      BEAM_POSA = PI*ANGLE/180.0
  !
  save_data(code_save_beam) = .true.
  return
  !
  99    call map_message(seve%e,rname,'Memory allocation failure')
  error = .true.
  return
  101   format(a,i10)
  102   call map_message(seve%e,rname,'SIC variable not defined')
  error = .true.
  return
end subroutine wipe_beam
