#include <stdio.h>
#include <math.h>
#include <string.h>
#include "wipe.h"
#ifndef SWAP
#define SWAP(x,y,tmp) {tmp = x; x = y; y = tmp;}
#endif
#ifndef POW2
#define POW2(x) ((x)*(x))
#endif
#ifndef FNINT
#define FNINT(x) nnearest(x)
#endif
#ifndef MAXIMUM
#define MAXIMUM(x,y)  (((x) > (y)) ? (x) : (y))
#endif
#ifndef MINIMUM
#define MINIMUM(x,y)  (((x) < (y)) ? (x) : (y))
#endif
#ifndef  M_PI
#define  M_PI   3.14159265358979323846
#endif

int nnearest( float);

/******************************************************************************/
/******************************************************************************/
void     wipe_beams(
int      *P_nx,     /* number of pixels in the x direction                    */
int      *P_ny,     /* number of pixels in the y direction                    */
int      *P_nf,     /* number of visibilities in Le+ (DC included)            */
int      *P_M,      /* oversampling factor of the object workspace            */
float    *P_eps,    /* measure of the crossinterpolation to perform           */
int      *P_it_max, /* maximum number of iterations for the power method      */
float    *P_thresh, /* threshold for computation of FT of NB on Le+           */
float    *P_dynamic,/* maximum dynamic allowed for weight function            */
int      *Hr,       /* aperture to be synthesized                             */
int      *Dr,       /* effective support of the synthesized beam in sky plane */
float    *DB,       /* Dirty Beam (n1*n1)                                     */
float    *NB,       /* Neat Beam (nx*ny)                                      */
float    *Vu,       /* points of the frequency list Le (2*nf) in the uv plane */
float    *Vv,       /*        in the u and v directions                       */
float    *Vn,       /* variance of the noise on Le+ (nf)                      */
float    *Vs,       /* regularisation function on the visibilities (nf)       */
float    *Vk,       /* weighting function on Le+ (nf)                         */
char     *W,        /* weighting mode: natural 'n' or uniform 'u'             */
char     *work)     /* workspace: 18*NX*NY+2*NX+2*NY+9*NV                     */
/******************************************************************************/
/******************************************************************************/
{
extern   void     inv_fft2D_real(), ffs1D_taper();
extern   int      inv_ffs2D_real();
extern   void     dir_fft2D_real();
/**/             
register int      k, kk, m, n, r;
int      i, j;
int      nx=*P_nx, ny=*P_ny, nxy=nx*ny;
int      nf=*P_nf,M=*P_M;
float    eps=*P_eps, it_max=*P_it_max;
int      nxs2=nx/2, nys2=ny/2, count;
int      n1x=2*nx, n1y=2*ny, n1xs2=n1x/2, n1ys2=n1y/2;
int      n2x=2*n1x, n2y=2*n1y, delta, it;
int      uu, vv, imin, jmin, imax, jmax, sum, flag;
int      *cell_I, *cell_J, *cell_K, *cell_N, *cell_R;
float    dux0, duy0, deltaf;
float    thresh=*P_thresh, dynamic=*P_dynamic, w_min, w_max, moy=0.0;
float    x, y, arg, dux, duy, ui, vi, uj, vj; 
float    tab_step_x, tab_step_y;
float    khi0, khi2, khi4;
float    FUNCTX[2048],FUNCTY[2048];
float    *tape_x, *tape_y, *Vw, *TMP;

/******* computing sizes and addresses of the different internal arrays ***
*******/
k = 0;
tape_x =  (float*) &work[k]; k +=     n1x*sizeof(float);
tape_y =  (float*) &work[k]; k +=     n1y*sizeof(float);
TMP    =  (float*) &work[k]; k += n2x*n2y*sizeof(float);
Vw     =  (float*) &work[k]; k +=    2*nf*sizeof(float);
cell_N =    (int*) &work[k]; k +=     nxy*sizeof(int);
cell_R =    (int*) &work[k]; k +=     nxy*sizeof(int);
cell_I =    (int*) &work[k]; k +=    2*nf*sizeof(int);
cell_J =    (int*) &work[k]; k +=    2*nf*sizeof(int);
cell_K =    (int*) &work[k]; k +=    2*nf*sizeof(int);

/*** tabulation of the square of the Fourier Transform of B-spline scaling function ***
*******/
dux0 = 1.0/((float) nx); dux = M*dux0/2.0;
duy0 = 1.0/((float) ny); duy = M*duy0/2.0;
tab_step_x = tab_tfbssf(FUNCTX)/dux;
tab_step_y = tab_tfbssf(FUNCTY)/duy;
for (k=0; k<2048; k++) {FUNCTX[k] *= FUNCTX[k]; FUNCTY[k] *= FUNCTY[k];}


/******* moves Dr to the 4 corners *********************************************
*******/
for (i=0; i<nys2; i++)
for (j=0; j<nxs2; j++)
  {
  SWAP(Dr[i*nx+j],Dr[nx*(i+nys2)+(j+nxs2)],delta);
  SWAP(Dr[nx*(i+nys2)+j],Dr[nx*i+(j+nxs2)],delta);
  }

/******* put Hr in the real part of a complex FFT matrix (Im = 0) ***************
*******/
SWAP(Hr[nxs2],Hr[nx*nys2],delta);
SWAP(Hr[0],Hr[nx*nys2+nxs2],delta);
for (j=1; j<nxs2; j++)
  {
  Hr[j] = Hr[nx*nys2+(nxs2+j)];
  Hr[nx*nys2+j] = Hr[nxs2+j];
  }
for (i=1; i<nys2; i++)
for (j=0; j<nxs2; j++)
  {
  Hr[nx*i+j] = Hr[nx*(nys2+i)+(nxs2+j)];
  Hr[nx*i+(nys2+j)] = Hr[nx*(nys2+i)+j];
  }
for (j=(nxs2+1); j<nx; j++) Hr[j] = Hr[nx*nys2+j] = 0;
for (i=(nys2+1); i<ny; i++)
for (j=0; j<nx; j++) Hr[nx*i+j] = 0;

/******* Computing the Neat Beam NB ******************************************
*******/
for (i=0; i<nxy; i++) NB[i] = (float)Hr[i];

inv_fft2D_real(NB,nx,ny);

for (i=0; i<nxy; i++) NB[i] *= (float)Dr[i];

khi0 = 0.0;
for (i=0; i<nxy; i++) khi0 += NB[i]*NB[i];

khi2 = (float)sqrt((double)khi0);
for (i=0; i<nxy; i++) NB[i] /= khi2;

it = 0;
while (1)
  {
  dir_fft2D_real(NB,nx,ny);
  for (i=0; i<nxy; i++) NB[i] *= (float)Hr[i];

  inv_fft2D_real(NB,nx,ny);
  for (i=0; i<nxy; i++) NB[i] *= (float)Dr[i];

  khi4 = 0.0;
  for (i=0; i<nxy; i++) khi4 += NB[i]*NB[i];
  khi2 = (float)sqrt((double)khi4);
  for (i=0; i<nxy; i++) NB[i] /= khi2;

  if ((khi0/khi4) > (1.0-eps)) break; khi0 = khi4;
  if (it > it_max) break; it++;
  khi2 = (float)sqrt((double)khi4);
  }
  if (it >= it_max)
     {
     printf(" \nW-WIPE, you have reached the maximum number of allowed ");
     printf("Power Method iterations.");
     printf(" \nW-WIPE, you may have not converged to Neat Beam solution.");
     printf(" \nW-WIPE, we suggest you increase WIPE_NPM.\n");  
     }

/******* Neat Beam normalization *****************************************
*******/
dir_fft2D_real(NB,nx,ny);
for (i=0; i<nxy; i++) NB[i] *= (float)Hr[i];

khi2 = NB[0];
for (i=0; i<nxy; i++) NB[i] /= khi2;

inv_fft2D_real(NB,nx,ny);
for (i=0; i<nxy; i++) NB[i] *= (float)Dr[i];

/******* stores NB and Dr with the origin at the center **********
*******/
for (i=0; i<nys2; i++)
for (j=0; j<nxs2; j++)
  {
  SWAP(Dr[nx*i+j],Dr[nx*(i+nys2)+(j+nxs2)],delta);
  SWAP(Dr[nx*(i+nys2)+j],Dr[nx*i+(j+nxs2)],delta);

  SWAP(NB[nx*i+j],NB[nx*(i+nys2)+(j+nxs2)],deltaf);
  SWAP(NB[nx*(i+nys2)+j],NB[nx*i+(j+nxs2)],deltaf);
  }

/******* restoring initial storage of Hr ***********************
*******/
SWAP(Hr[0],Hr[nx*nys2+nxs2],delta);
SWAP(Hr[nxs2],Hr[nx*nys2],delta);
for (i=1; i<nys2; i++)
for (j=0; j<nxs2; j++)
  {
  Hr[nx*(nys2+i)+(nxs2+j)] = Hr[nx*i+j];
  Hr[nx*(nys2+i)+j] = Hr[nx*i+(nys2+j)];
  }
for (j=1; j<nxs2; j++)
  {
  Hr[nx*nys2+(nxs2+j)] = Hr[j];
  Hr[nxs2+j] = Hr[nx*nys2+j];
  }
for (i=1; i<nys2; i++)
  {
  Hr[nx*i] = Hr[nx*(ny-i)];
  Hr[nx*i+nys2] = Hr[nx*(ny-i)+nys2];
  }
for (j=1; j<nxs2; j++)
  {
  Hr[j] = Hr[nx-j];
  Hr[nx*nys2+j] = Hr[nx*nys2+(nx-j)];
  }
for (i=1; i<nxs2; i++)
for (j=1; j<nys2; j++)
  {
  Hr[nx*i+j] = Hr[nx*(ny-i)+(nx-j)];
  Hr[nx*i+(ny-j)] = Hr[nx*(ny-i)+j];
  }

/******* Initialization of arrays **********************
*******/
for (i=0; i<nxy; i++)  cell_N[i] = 0;
for (i=0; i<2*nf; i++) cell_K[i] = 0;

/******* computing points of Le-, to get Le ***************
*******/
for (k=0; k<nf; k++)
  {
  if (Vv[k] < 0.0) {Vv[k] = -Vv[k]; Vu[k] = -Vu[k];}
  if ((Vv[k] == 0.0) && (Vu[k] < 0.0)) {Vu[k] = -Vu[k];}
  Vu[k+nf] = -Vu[k];
  Vv[k+nf] = -Vv[k];
  }

/******* computing cells where the points (u,v) fall  **********
*******/
for (k=0; k<2*nf; k++)
  {
  i = nys2 + FNINT(Vv[k]/duy0);
  j = nxs2 + FNINT(Vu[k]/dux0);
  cell_I[k] = i;       /* line of the cell (i,j) for the point (u,v)   */
  cell_J[k] = j;       /* column of the cell (i,j) for the point (u,v) */
  cell_N[nx*i+j] += 1; /* number of points in the cell (i,j)           */
  }

/******* number of points (u,v) encoutered till cell (i,j) *******
*******/
sum = 0;
for (i=0; i<ny; i++)
for (j=0; j<nx; j++)
  {
  cell_R[nx*i+j] = sum;  /* number of points (u,v) encoutered till cell  */
  sum += cell_N[nx*i+j]; /* (i,j) not included when reaching the end!    */
  }

/******* affecting each (u,v) point in a cell an address **********
*******/
for (k=0; k<2*nf; k++)
  {
  i = cell_I[k];
  j = cell_J[k];
  cell_K[cell_R[nx*i+j]] = k;
  cell_R[nx*i+j] += 1;
  }

/***** re-computing the number of points (u,v) encoutered till actual cell ****
*****/
sum = 0;
for (i=0; i<ny; i++)
for (j=0; j<nx; j++)
  {
  cell_R[nx*i+j] = sum;  /* number of points (u,v) encoutered till cell  */
  sum += cell_N[nx*i+j]; /* (i,j) not included when reaching the end!    */
  }

/***** computing Fourier Transform of the Neat Beam S(u,v) on Le+ ***************
*****/
for (k=0; k<nf; k++) Vs[k] = 0.0;
thresh *= NB[nx*nys2+nxs2];
for (i=0; i<ny; i++)
  {
  y = 2.0*M_PI*((float) (i - nys2));
  for (j=0; j<nx; j++)
    {
    x = 2.0*M_PI*((float) (j - nxs2));
    if (NB[nx*i+j] > thresh)
      {
      for (k=0; k<nf; k++)
        {
        arg = Vu[k]*x + Vv[k]*y;
        Vs[k] += NB[nx*i+j]*cos(arg) ;
        }
      }
    }
  }

/***** computing weights W(u,v) on Le+ ***************************************
*****/
flag = 0;
for (k=0; k<nf; k++)
 {
 if (Vn[k]*Vs[k] != 0)
   {
   if((flag == 1) && (w_min > 1.0/POW2(Vn[k]*Vs[k])))
     w_min = 1.0/POW2(Vn[k]*Vs[k]);
   if(flag == 0)
     w_min = 1.0/POW2(Vn[k]*Vs[k]);
   flag = 1;
   }
 }
 
if (flag == 0)
   {
   printf("\n E-WIPE, Can't continue with this weight channel");
   printf("\n E-WIPE,  All the weights are null !\n");
   return;
   }

/***** troncating weights W(u,v) on Le+ and extension to Le *****************
****** Taking into account null weights in Vw final weight function   *******
*****/
w_max = dynamic*w_min;
count = 0;
for (k=0; k<nf; k++)
    {
    if (Vn[k]*Vs[k] != 0)
       {
       if (1.0/POW2(Vn[k]*Vs[k]) <= w_max)
          Vw[k] = 1.0/POW2(Vn[k]*Vs[k]);
       else 
          Vw[k] = w_max ;
       }
    else 
       {
       Vw[k] = 0;
       count++;
       }

    Vw[k+nf] = Vw[k];
    }

/******* computing regularized data R(u,v), I(u,v) et N(u,v) on Le+ ******
*******/
for (k=0; k<nf; k++)
  Vn[k] *= Vs[k];

/******* computing K(u,v) function ******************************************
*******/
moy = 0.0;
for (k=0; k<nf; k++)
  {
  Vk[k] = 0.0;
  /* Coordinates of frequency point k */
  ui = Vu[k];
  vi = Vv[k];
  /* Cell in which this point lies */
  i = cell_I[k];
  j = cell_J[k];
  /* Nearest cells */
  imax = MINIMUM(ny-1,i+1+M/2);
  jmax = MINIMUM(nx-1,j+1+M/2);
  imin = MAXIMUM(0,i-1-M/2);
  jmin = MAXIMUM(0,j-1-M/2);
  for (i=imin; i<=imax; i++)
  for (j=jmin; j<=jmax; j++)
    {
    n = cell_N[nx*i+j]; /* Number of points in the Fourier cell (i,j) */
    r = cell_R[nx*i+j]; /* Rank of the first point of this cell       */
    for (m=0; m<n; m++)
      {
      kk = cell_K[r + m];
      uj = Vu[kk] - ui; uj = fabs(uj);
      vj = Vv[kk] - vi; vj = fabs(vj);
      /* Test on the distance between the two points */
      if ((uj < duy) && (vj < dux))
        {
        uu = ((int) (tab_step_x*uj));
        vv = ((int) (tab_step_y*vj));
        Vk[k] += FUNCTX[uu]*FUNCTY[vv]*Vw[kk];
        }
      }
    }
  /******* uniform weighting **************************************************
  *******/
  moy += Vk[k];
  if (Vk[k] != 0)
     Vk[k] = Vw[k]/Vk[k];
  }
/******* natural weighting **************************************************
*******/
if ((W[0] != 'u') && (W[0] != 'U'))
  {
  moy /= ((float) (nf-count));
  for (k=0; k<nf; k++) Vk[k] = Vw[k]/moy;
  }

/******* computing Dirty Beam **************************************************
*******/
ffs1D_taper(tape_x,n1x,4,1.505064,80);
for (i=0; i<n1x; i++) tape_x[i] = ((float) n2x)*dux/tape_x[i];
ffs1D_taper(tape_y,n1y,4,1.505064,80);
for (i=0; i<n1y; i++) tape_y[i] = ((float) n2y)*duy/tape_y[i];
Vk[0] *= 0.5; /* DC component */
inv_ffs2D_real(TMP,n2x,n2y,Vk,NULL,Vu,Vv,nf,4,1.505064);
Vk[0] *= 2.0; /* DC component */
inv_fft2D_real(TMP,n2x,n2y);

for (i=0; i<n1y; i++)
for (j=0; j<n1x; j++) 
    DB[n1x*i+j] = TMP[n2x*(n1ys2+i)+n1xs2+j]*(tape_x[i]*tape_y[j]);
}


/******************************************************************************/
/******************************************************************************/
int      nnearest(float x)
/******************************************************************************/
/******************************************************************************/
{
int    i;

i = ((int) x);
if (i < 0)
  {
  x = ((float) i) - x;
  if (x > (1.0-x)) i--;
  }
else
  {
  x = x - ((float) i);
  if (x > (1.0-x)) i++;
  }

return(i);
}
