#include <stdio.h>
#include <math.h>
#include "wipe.h"

#define POW2(x) ((x)*(x))
#define IABS(x) (((x) >= 0) ? (x) : -(x))
#ifndef  M_PI
#define  M_PI   3.14159265358979323846
#endif

/*****************************************************************************/
/*****************************************************************************/
void    wipe_cw(
int     *P_nx,    /* Number of pixels in X (input)                           */
int     *P_ny,    /* Number of pixels in Y (input)                           */
int     *P_nf,    /* number of visibilities (input)                          */
int     *P_NT,    /* Maximum dimension of the krylov space (input)           */
int     *P_M,     /* oversampling factor of the object workspace (input)     */
int     *P_MNWI,  /* maximum number of wipe iterations (input)               */
int     *P_MNCI,  /* maximum number of clean iterations (input)              */
int     *P_MNCG,  /* maximum number of conjugate gradients iterations (input)*/
int     *P_MNPOS, /* max number of positivity constraint iterations (input)  */
float   *P_EPS_POS, /* Precision on the positivity constraint stop (input)   */
float   *P_SEUIL, /* threshold on the residu for support expansion (input)   */
float   *P_PMU,   /* stop test value for relative variation of mu (input)    */
float   *P_PPHI,  /* stop test value for relative variation of phi (input)   */
float   *P_lmnb,  /* lower margin set on NB value (input)                    */
float   *P_umnb,  /* upper margin set on NB value (input)                    */
int     *Hr,      /* Synthesized aperture (input)                            */
float   *DB,      /* Dirty Beam (input)                                      */
float   *NB,      /* Neat Beam (input)                                       */
float   *DM,      /* Dusty Map (input)                                       */
float   *Vr,      /* Real part of the experimental visibilities (input)      */
float   *Vi,      /* imaginary part of the experimental visibilities (input) */
float   *Vn,      /* Sigma on visibility 1|sqrt(2)  (input)                  */
float   *Vk,      /* weight function (output)                                */
int     *CB,      /* initial support ifo clean (input)                       */
float   *NM,      /* Neat Map (output)                                       */
float   *dn,      /* Residu (output)                                         */
float   *NMR,     /* Neat Map+residu (output)                                */
int     *D,       /* upport of Neat Map (output)                             */
char    *work)    /* Working space: (27*NX*NY+4*NT)*WORD                     */
/*****************************************************************************/
/*
   REMARKS:
     To avoid using an additional array the dirty beam is used in the beginning
   of the program to compute the Dirty Transfer Pattern (its FFT) and restored
   at the end to the original value.
     Moreover, DB, DM and Vk are internally normalized by "arg" and restored
   at the end.   
*/
/*****************************************************************************/
{
extern   void     inv_fft2D_real();
extern   void     dir_fft2D_real();
extern   void     mul_fft2D_real();
extern   void     zero_padding();
extern   int      ql_tridiag();
extern   int      connex(), cell();
/**/
register int      i, j, k, kk;
int     nx=*P_nx, ny=*P_ny, nxy=nx*ny, nf=*P_nf;
int     n1x=2*nx, n1y=2*ny, n1xy = n1x*n1y;
int     nxs2=nx/2, nys2=ny/2, n1xs2=n1x/2, n1ys2=n1y/2;
int     n1xs4=n1x/4, n1ys4=n1y/4;
int     it_clean, n_clean, it_wipe, n_wipe;
int     itmax_wipe=*P_MNWI, itmax_clean=*P_MNCI;
int     NT = *P_NT, M = *P_M, itmax_gc=*P_MNCG, itmax_posgc=*P_MNPOS;
int     it_gc, it_posgc;
int     elimine, i_conv, nbre;
int     i00, i11, j00, j11, r, rr, nr;
int     m1, m2, ic, jc, Nr, Ns, Ms, i0, i1, i2, j0, j1, j2;
int     w_switch, flag, wipe, un=1, ni0, ni1;
int     r1, rn1, q1, qq1, r2, rn2, q2, qq2, one_x, one_y, is2, js2;
int     *ID, *classe_i, *classe_j, *Dsav, *sav3;
char    *mask, *motif, reponse[2];
float   tab_step_x, tab_step_y, FUNCT_x[2049], FUNCT_y[2049];
float   Vis_RI2, Vis_N2, sigma_re, dux0, duy0, dux, duy, Q, XX, YY;
float   rn_rn, rn_rn_sav, xn_xn, dn_zn, omega_clean, omega;
float   gamma, omega_sav, gamma_sav, sup, size, tmp1, tmp2;
float   mu, kappa, eps, eta, flux0;
float   c, Cr, Ce, Ce_sav=0, Ce_inf, Ce_inf_sav=0;
float   *rn, *zn, *NTP, *DTP, *TMP1, *TMP2, *NMsav;
float   *classe_r, *neat, *sav1, *sav2;
float   arg, *K1, *K2, *KK1, *KK2;
float   EPS_POS=*P_EPS_POS, SEUIL=*P_SEUIL;
float   eps_mu=*P_PMU, eps_phi=*P_PPHI, lmnb=*P_lmnb, umnb=*P_umnb;


/******* computing sizes and addresses of internal arrays ***
*******/
/* Initializing working space */
for (i=0; i<(27*nxy+4*NT)*sizeof(int); i++)
    work[i] = 0;

dux0 = 1.0/((float) nx); duy0 = 1.0/((float) ny); 
dux = M*dux0/2.0;        duy = M*duy0/2.0;

k = 0;
Dsav     = (int*)   &work[k];  k += nx*ny*sizeof(int);
NMsav    = (float*) &work[k];  k += nx*ny*sizeof(float);
sav1     = (float*) &work[k];  k += nx*ny*sizeof(float);
sav2     = (float*) &work[k];  k += nx*ny*sizeof(float);
sav3     = (int*)   &work[k];  k += nx*ny*sizeof(int);
rn       = (float*) &work[k];  k += nx*ny*sizeof(float);
zn       = (float*) &work[k];  k += nx*ny*sizeof(float);
NTP      = (float*) &work[k];  k += n1x*n1y*sizeof(float);
DTP      = (float*) &work[k];  k += n1x*n1y*sizeof(float);
TMP1     = (float*) &work[k];  k += n1x*n1y*sizeof(float);
TMP2     = (float*) &work[k];  k += n1x*n1y*sizeof(float);
neat     = (float*) &work[k];  k += nxs2*nys2*sizeof(float);
classe_r = (float*) &work[k];  k += 2*nxs2*nys2*sizeof(float);
classe_i = (int*)   &work[k];  k += 2*nxs2*nys2*sizeof(int);
classe_j = (int*)   &work[k];  k += 2*nxs2*nys2*sizeof(int);
ID       = (int*)   &work[k];  k += nx*ny*sizeof(int);
mask     = (char*)  &work[k];  k += nx*ny*sizeof(char);
motif    = (char*)  &work[k];  k += nxs2*nys2*sizeof(char);
K1       = (float*) &work[k];  k += NT*sizeof(float);
K2       = (float*) &work[k];  k += NT*sizeof(float);
KK1      = (float*) &work[k];  k += NT*sizeof(float);
KK2      = (float*) &work[k];  k += NT*sizeof(float);

/* initial flux
*/
flux0 = 0.0;

/******* Dirty Transfer Pattern ***********************************************
*******/
dir_fft2D_real(DB,n1x,n1y);

/******* Dusty Transfer Pattern ***********************************************
*******/
tab_step_x = tab_tfbssf(FUNCT_x);
tab_step_y = tab_tfbssf(FUNCT_y);
one_x = ((int) (tab_step_x));
one_y = ((int) (tab_step_y));
for (k=0; k<2048; k++) FUNCT_x[k] *= FUNCT_x[k];
for (k=0; k<2048; k++) FUNCT_y[k] *= FUNCT_y[k];

for (i=0; i<n1xy; i++)  DTP[i] = DB[i];
k = 0;
for (i=0; i<n1y; i++)
  {
  is2 = i/2;
  q2 = i - n1ys2;
  r2 = q2 % M;
  q2 = IABS(q2);
  rn2 = (q2 - n1y) % M;
  qq2 = ((int) ((tab_step_y*q2)/n1y));
  for (j=0; j<n1x; j++)
    {
    js2 = j/2;
    q1 = j - n1xs2;
    r1 = q1 % M;
    q1 = IABS(q1);
    rn1 = (q1 - n1x) % M;
    qq1 = ((int) ((tab_step_x*q1)/n1x));
    if ((Hr[nx*is2+js2] < 0.5) && (r1 == 0) && (r2 == 0))
      {
      tmp1 = FUNCT_x[qq1]*FUNCT_y[qq2];
      k++;
      }
    else tmp1 = 0.0;
    if ((q1 < n1xs4) && (q2 >= n1ys4))
      {
      if ((r1 == 0) && (rn2 == 0)) 
         tmp1 += (FUNCT_x[qq1]*FUNCT_y[one_y-qq2]);
      }
    else if ((q1 >= n1xs4) && (q2 < n1ys4))
      {
      if ((rn1 == 0) && (r2 == 0)) 
         tmp1 += (FUNCT_y[qq2]*FUNCT_x[one_x-qq1]);
      }
    else if ((q1 >= n1xs4) && (q2 >= n1ys4))
      {
      if ((r1  == 0) && (rn2 == 0)) 
         tmp1 += (FUNCT_x[qq1]*FUNCT_y[one_y-qq2]);
      if ((rn1 == 0) && (r2  == 0)) 
         tmp1 += (FUNCT_y[qq2]*FUNCT_x[one_x-qq1]);
      if ((rn1 == 0) && (rn2 == 0)) 
         tmp1 += (FUNCT_x[one_x-qq1]*FUNCT_y[one_y-qq2]);
      }
    if (j <= n1xs2)
      {
      if ((0 <= i) && (i <= n1ys2)) 
         DTP[n1x*(n1ys2-i)+n1xs2-j] += tmp1*(M*M) ;
      }
    else
      {
      if ((0 < i) && (i < n1ys2)) 
         DTP[n1x*(n1ys2-i)+3*n1xs2-j] += tmp1*(M*M) ;
      }
    }
  }

/******* Neat Transfer Pattern *************************************************
*******/
for (i=0; i<ny; i++)
for (j=0; j<nx; j++) NTP[n1x*(n1ys4+i)+n1xs4+j] = NB[nx*i+j];
dir_fft2D_real(NTP,n1x,n1y);

/******* Dirty Beam convolved with the Neat Beam *************************************
*******/
mul_fft2D_real(NTP,DB,TMP1,n1x,n1y);
inv_fft2D_real(TMP1,n1x,n1y);

/*   Normalisation coefficient :  arg = s(0) / (h \star s)(0)
*/
arg =  NB[nx*nys2 + nxs2] / TMP1[n1x*n1ys2 + n1xs2];  


/* Normalisation
*/
for (i=0; i<n1xy; i++)
  {
  DB[i] *= arg;
  DTP[i] *= arg;
  TMP1[i] *= arg;
  }

for (i=0; i<nxy; i++)
  {
  DM[i] *= arg;
/*
  CB[i] = 1;
*/
  }

for (k=0; k<nf; k++) Vk[k] *= arg;

/******* eta *******************************************************************
*******/
eta = 0.0;
for (i=0; i<ny; i++)
for (j=0; j<nx; j++) eta += NB[nx*i+j]*TMP1[n1x*(n1ys4+i)+n1xs4+j];
eta = (float)sqrt((double)eta);

/****** C_e ******************************************************************
*******/
Vis_RI2 = Vis_N2 = sigma_re = 0.0;
for (k=0; k<nf; k++)
  {
  Vis_RI2 += (POW2(Vr[k]) + POW2(Vi[k])) * Vk[k];
  Vis_N2  += POW2(Vn[k]) * Vk[k];
  sigma_re += POW2(Vn[k] * Vk[k]); 
  }
Vis_RI2 *= 2.0;
Vis_N2  *= 2.0;
sigma_re *= 2.0;
Vis_RI2 += POW2(Vr[0]) * Vk[0];
Vis_N2  += POW2(Vn[0]) * Vk[0];
sigma_re += POW2(Vn[0] * Vk[0]);
Vis_RI2 *= dux*duy;
Vis_N2  *= dux*duy;

Ce = sqrt(Vis_RI2/Vis_N2);
sigma_re = sqrt(sigma_re) * (dux*duy);

Q = Vis_RI2;

fflush(stdout);
printf("clean_nbr_iter......: "); fflush(stdout);
fscanf(stdin,"%d",&n_clean);

if (n_clean < 1)
   {
   n_clean = 1;
   omega_clean = 0.2;
   }
else
   {
   printf("clean_gain..........: "); fflush(stdout);
   fscanf(stdin,"%f",&omega_clean);
   }

/******* margins ****************************************************************
*******/
tmp1 = lmnb*NB[nx*nys2+nxs2];
m1 = 0; i = nys2+1; while (NB[nx*i+nxs2] > tmp1) {m1++; i++;}
m2 = 0; j = nxs2+1; while (NB[nx*nys2+j] > tmp1) {m2++; j++;}

XX = lmnb*NB[nx*nys2+nxs2];   /* for the support of s */
YY = umnb*NB[nx*nys2+nxs2];   /* for its interior */
Ms = Ns = 0;
i1 = nys2 - m1; i2 = nys2 + m1;
j1 = nxs2 - m2; j2 = nxs2 + m2;
for (i=i1; i<=i2; i++)
  {
  i0 = i-i1;
  for (j=j1; j<=j2; j++)
    {
    j0 = j-j1;
    motif[nxs2*i0+j0] = 0;
    neat[nxs2*i0+j0] = NB[nx*i+j];
    if (NB[nx*i+j] > XX)
      {
      Ns++;
      if (NB[nx*i+j] < YY) motif[nxs2*i0+j0] = 1;
      else {Ms++; motif[nxs2*i0+j0] = 2;}
      }
    }
  }

/******* CLEAN & WIPE ITERATIONS ********************************************
*******/
w_switch = flag = wipe = 0;
while (flag == 0)
  {
  /******* INITIAL RESIDU: NM=0, dn=DM *****************************************
  *******/
  for (i=0; i<nxy; i++)
    {
    NM[i] = 0.0;
    dn[i] = DM[i];
    NMR[i] = NM[i]+dn[i];
    D[i] = 0;
    }
  Q = Vis_RI2;

  /******* CLEAN ITERATIONS *************************************************
  *******/
  it_clean = 0;
  while (it_clean < itmax_clean)
    {
    printf("CLEAN (%d) ....: ",it_clean);

    /***** searching sup of |dn|
    *****/
    sup = 0.0;
    for (i=0; i<ny; i++)
    for (j=0; j<nx; j++)
      {
      tmp1 = fabs(dn[nx*i+j]*(float)CB[nx*i+j]);
      if (tmp1 > sup) {ic = i; jc = j; sup = tmp1;}
      }

    /***** computing (NB|dn) for NB centered in (ic,jc)
    *****/
    i0 = ic - m1 ; i00 = (i0 < 0  ?  0 : i0);
    i1 = ic + m1 ; i11 = (i1 < ny ? i1 : ny-1);
    j0 = jc - m2 ; j00 = (j0 < 0  ?  0 : j0);
    j1 = jc + m2 ; j11 = (j1 < nx ? j1 : nx-1);
    tmp1 = 0.0;
    for (i=i00; i<=i11 ;i++)
    for (j=j00; j<=j11 ;j++) 
        tmp1 += neat[nxs2*(i-i0)+(j-j0)]*dn[nx*i+j];
    gamma = omega_clean*tmp1/POW2(eta);

    /***** updating NM and its support D
    *****/
    for (i=i00; i<=i11; i++)
    for (j=j00; j<=j11; j++)
      {
      if (motif[nxs2*(i-i0)+(j-j0)] != 0) D[nx*i+j] = 1;
      NM[nx*i+j] += gamma*neat[nxs2*(i-i0)+(j-j0)];
      }

    /* Flux
    */
    XX =  0.0;
    for (i=0; i<nxy; i++)
      XX += (NM[i]+rn[i]*(float)D[i]);
/*
      XX += NM[i];
*/
    YY = (XX - flux0)/XX;
    flux0 = XX;
    printf("component:[%d,%d], FLUX:%E, RELATIVE_VARIATION:%E\n",ic,jc,XX,YY); 

    /***** updating the quadratic error term Q
    *****/
    Q -= omega_clean*(2.0 - omega_clean)*POW2(tmp1/eta);
    Ce = sqrt(Q/Vis_N2);

    /***** updating rn
    *****/
    ic -= n1ys2; /* n1ys2 + (i-ic) = i - (ic-n1ys2) */
    jc -= n1xs2; /* n1xs2 + (j-ic) = j - (jc-n1xs2) */
    for (i=0; i<ny; i++)
    for (j=0; j<nx; j++) dn[nx*i+j] -= gamma*TMP1[n1x*(i-ic)+j-jc];

    /***** stop test
    *****/
    if (it_clean >= n_clean)
      {
      gdf_c2f_plot("_clean_nm");
      gdf_c2f_plot("_clean_res");

      /* SAVE NM + Res
      */
      for (i=0; i<nxy; i++)
        NMR[i] = NM[i] + dn[i];
      gdf_c2f_plot("_clean_nmres");

      printf("\nQ-WIPE,  LEAVE CLEAN & WIPE ? [y,n] "); 
      fflush(stdout); 
      fscanf(stdin,"%s",reponse);
      if (reponse[0] == 'y') {flag = 1; break;}
      else
        {
        printf("\nQ-WIPE,  CLEAN or WIPE ? [c,w] "); 
	fflush(stdout); 
        fscanf(stdin,"%s",reponse);
        printf("\n");
        if (reponse[0] == 'c')
          {
          k = n_clean;
          printf("CLEAN.....: clean_iter = "); fflush(stdout);
          fscanf(stdin,"%d",&n_clean);
          if (n_clean <= k) {wipe = 0; break;}
          }
        else {w_switch = wipe = 1; break;}
        }
      }

    it_clean++;
    }

for (i=0; i<nxy; i++)   if ((float)D[i] > 0.5) CB[i] = 1;

  if (wipe == 0) break;

  /******* WIPE ITERATIONS **************************************************
  *******/
  Ce_inf = Ce;
  it_wipe = 0;
  while (it_wipe < itmax_wipe)
    {
    printf("WIPE.......: wipe_iter=%d\n",it_wipe);

    /*** recherche du sup de rn a l'exterieur de D
    ***/
    if (it_wipe > 0) 
      {
      sup = 0.0;
      for (i=(m1+1); i<(ny-m1-1); i++)
      for (j=(m2+1); j<(nx-m2-1); j++)
        {
        if (rn[nx*i+j]*(float)CB[nx*i+j] > sup)
          {ic = i; jc = j; sup = rn[nx*i+j];}
        }
      }

    /***** rn = dn
    *****/
    for (i=0; i<nxy; i++) rn[i] = dn[i];

    /***** SUPPORT EXPANSION ************************************************
    *****/
    if ((flag == 0) && (it_wipe != 0))
      {
      /*** searching for sup of rn outside D
      ***/
      sup = 0.0;
      for (i=0; i<ny; i++)
      for (j=0; j<nx; j++)
        {
        if (((float)D[nx*i+j] < 0.5) && 
                    (rn[nx*i+j]*(float)CB[nx*i+j] > sup))
          {ic = i; jc = j; sup = rn[nx*i+j];}
        }
      /*** thresholding rn
      ***/
      XX = sup*SEUIL;
      for (i=0; i<nxy; i++)
        {
	if (rn[i]*(float)CB[i] > XX) 
           mask[i] = 1;
        else
           mask[i] = 0;
        }
      /*** ordering Nr first values of rn decreasingly
      ***/
      mask[nx*ic+jc] = 0;
      Nr = 1;
      classe_r[Nr] = sup; classe_i[Nr] = ic; classe_j[Nr] = jc;
      for (i=0; i<ny;i++)
      for (j=0; j<nx;j++)
        {
        if (mask[nx*i+j] == 1)
          {
          XX = rn[nx*i+j];
          if (XX <= classe_r[Nr])
            {
            if (Nr < Ns) 
	       {Nr++; classe_r[Nr] = XX; classe_i[Nr] = i; classe_j[Nr] = j;}
            }
          else
            {
            nr=Nr;
            for (r=2; r<=nr; r++)
              {
              YY = classe_r[r];
              if (XX >= YY)
                {
                Nr++;
                if (Nr > Ns) Nr=Ns;
                for (rr=Nr; rr>r; rr--)
                  {
                  classe_r[rr] = classe_r[rr-1];
                  classe_i[rr] = classe_i[rr-1];
                  classe_j[rr] = classe_j[rr-1];
                  }
                classe_r[r] = XX; classe_i[r] = i; classe_j[r] = j;
                break;
                }
              }
            }
          }
        }
      /*** mask of Nr retained pixels
      ***/
      for (i=0; i<nxy;i++)  mask[i] = 0;
      for (r=1; r<=Nr; r++) mask[nx*classe_i[r]+classe_j[r]] = 1;
      /*** modifying D 
      ***/
      for (i=0; i<nxy;i++) if (mask[i] == 1) D[i] = 1;
      }

    /***** CONJUGATE GRADIENTS INITIALISATION ******************************
    *****/
    if ((flag == 0) && (it_wipe != 0))
      {
      /*** rn = D.dn
      ***/
      for (i=0; i<nxy; i++) rn[i] = dn[i]*(float)D[i];
      }
    else
      {
      /*** NM = D.NM
      ***/
      for (i=0; i<nxy; i++)  NM[i] = NM[i]*D[i];

      /*** zn = A*A.NM 
      ***/
      zero_padding(NM,nx,ny,TMP1,n1x,n1y);
      dir_fft2D_real(TMP1,n1x,n1y);
      mul_fft2D_real(TMP1,DTP,TMP1,n1x,n1y);
      inv_fft2D_real(TMP1,n1x,n1y);
      zero_padding(TMP1,n1x,n1y,zn,nx,ny);

      /*** dn = DM - A*A.NM
      ***/
      for (i=0; i<nxy; i++)  dn[i] = DM[i]-zn[i];

      /*** rn = D.(DM - A*A.NM)
      ***/
      for (i=0; i<nxy; i++)  rn[i] = dn[i]*(float)D[i];
      }

    for (i=0; i<nxy; i++)  dn[i] = rn[i];
    omega_sav = 1.0;
    gamma_sav = 0.0;

    /***** CONJUGATE GRADIENTS ITERATIONS **********************************
    *****/
    it_gc = 0;
    while (it_gc < itmax_gc)
      {
      /*** zn = D.A*A.dn
      ***/
      zero_padding(dn,nx,ny,TMP1,n1x,n1y);
      dir_fft2D_real(TMP1,n1x,n1y);
      mul_fft2D_real(TMP1,DTP,TMP1,n1x,n1y);
      inv_fft2D_real(TMP1,n1x,n1y);
      zero_padding(TMP1,n1x,n1y,zn,nx,ny);
      for (i=0; i<nxy; i++)  zn[i] *= (float)D[i];
      /*** omega
      ***/
      rn_rn = dn_zn = 0.0;
      for (i=0; i<nxy; i++)
        {
        rn_rn += (rn[i]*rn[i]);
        dn_zn += (dn[i]*zn[i]);
        }
      rn_rn_sav = rn_rn;
      omega = rn_rn/dn_zn;
      /*** building the tridiagonal Krylov matrix (K1,K2)
      ***/
      K1[it_gc] = (1.0/omega) + (gamma_sav/omega_sav);
      K2[it_gc] = -sqrt(gamma_sav) / omega_sav;
      /*** updating NM and rn
      ***/
      rn_rn = xn_xn = 0.0;
      for (i=0; i<nxy; i++)
        {
        NM[i] += (omega*dn[i]);
        rn[i] -= (omega*zn[i]);
        rn_rn += (rn[i]*rn[i]);
        xn_xn += (NM[i]*NM[i]);
        }
      /*** gamma
      ***/
      gamma = rn_rn/rn_rn_sav;
      /*** updating dn
      ***/
      for (i=0; i<nxy; i++)  dn[i] = rn[i] + gamma*dn[i];
      /*** omega & gamma for next line of Krylov  matrix
      ***/
      omega_sav = omega;
      gamma_sav = gamma;
      /*** computing eigenvalues of kappa, stop test
      ***/
      if (it_gc > 0)
        {
        printf("WIPE(%d)....: iter_gc=%d ",it_wipe,it_gc);
        for (i=0; i<=it_gc; i++) {KK1[i] = K1[i]; KK2[i] = K2[i];}
        ql_tridiag(KK1,KK2,NULL,it_gc+1,itmax_gc);
        kappa = sqrt(KK1[it_gc]/KK1[0]);
        printf("mu=%E, mu'=%E, kappa=%E\n",KK1[0],KK1[it_gc],kappa);
/*
        if ((it_gc > 1) && ((1. - KK1[0]/mu) < eps_mu))
*/
        if ((it_gc > 1) && ((mu - KK1[0]) < eps_mu))
          {
          eps = sqrt(rn_rn/xn_xn)/mu;
          if (eps < eps_phi) break;
          }
        mu = KK1[0];
        }
      else printf("WIPE(%d)....: iter_gc=%d\n",it_wipe,it_gc);
      /*** next iteration
      ***/
      it_gc++;
      }

    /***** NUMBER OF FORBIDDEN POINTS ******************************************
    *****/
    ni0 = 0;  /* Number of pixels in D                    */
    ni1 = 0;  /* Number of forbidden negative pixels in D */
    for (i=0; i<nxy; i++)
      {
      ID[i] = 0;
      if ((float)D[i] > 0.5)
        { 
        ID[i] = 1;
        ni0++;
        if (NM[i] < 0.0) 
           {D[i] = 0; NM[i] = 0.0; ni1++;}
        }
      }

    /***** NEW RESIDU ******************************************************
    *****/
    /* zn = A*A.NM
    */
    zero_padding(NM,nx,ny,TMP1,n1x,n1y);
    dir_fft2D_real(TMP1,n1x,n1y);
    for (i=0; i<n1xy; i++)  TMP2[i] = TMP1[i];
    mul_fft2D_real(TMP1,DTP,TMP1,n1x,n1y);
    inv_fft2D_real(TMP1,n1x,n1y);
    zero_padding(TMP1,n1x,n1y,zn,nx,ny);
    /* dn = DM - A*A.NM
    */
    for (i=0; i<nxy; i++) dn[i] = DM[i]-zn[i];

    /***** POSITIVITY BY BLOCK CONJUGATE GRADIENTS *****************************
    *****/
    if (ni1 != 0)
      printf("WIPE.......: iter_positiv_gc = 0\n");
    while (ni1 != 0)
      {
      /*** CONJUGATE GRADIENTS INITIALISATION ******************************
      ***/
      /* first residu: rn = D.dn
         first direction of search: dn = rn
      */
      for (i=0; i<nxy; i++) 
          {
          rn[i] = (float)D[i]*dn[i];
          dn[i] = rn[i];
          }

      /*** CONJUGATE GRADIENTS ITERATIONS **********************************
      ***/
      for (it_gc=0; it_gc<NT; it_gc++)
        {
        printf("WIPE(%d)....: iter_gc=%d\n",it_wipe,it_gc);
        /* zn = D.A*A.dn
        */
        zero_padding(dn,nx,ny,TMP1,n1x,n1y);
        dir_fft2D_real(TMP1,n1x,n1y);
        mul_fft2D_real(TMP1,DTP,TMP1,n1x,n1y);
        inv_fft2D_real(TMP1,n1x,n1y);
        zero_padding(TMP1,n1x,n1y,zn,nx,ny);
        for (i=0; i<nxy; i++)  zn[i] *= (float)D[i];
        /* omega
        */
        rn_rn = dn_zn = 0.0;
        for (i=0; i<nxy; i++)
          {
          rn_rn += (rn[i]*rn[i]);
          dn_zn += (dn[i]*zn[i]);
          }
        rn_rn_sav = rn_rn;
        omega = rn_rn/dn_zn;

        /* updating NM & rn
        */
        rn_rn = xn_xn = 0.0;
        for (i=0; i<nxy; i++)
          {
          NM[i] += (omega*dn[i]);
          rn[i] -= (omega*zn[i]);
          rn_rn += (rn[i]*rn[i]);
          xn_xn += (NM[i]*NM[i]);
          }

        /* stop test
        */
        eps_phi = sqrt(rn_rn/xn_xn)/mu;
        if (eps_phi < EPS_POS) break;
        /* gamma
        */
        gamma = rn_rn/rn_rn_sav;
        /* mis a jour de dn
        */
        for (i=0; i<nxy; i++)  dn[i] = rn[i] + gamma*dn[i];
        }

      /*** NEW RESIDU ******************************************************
      ****/
      /* zn = A*A.NM
      */
      zero_padding(NM,nx,ny,TMP1,n1x,n1y);
      dir_fft2D_real(TMP1,n1x,n1y);
      for (i=0; i<n1xy; i++)  TMP2[i] = TMP1[i];
      mul_fft2D_real(TMP1,DTP,TMP1,n1x,n1y);
      inv_fft2D_real(TMP1,n1x,n1y);
      zero_padding(TMP1,n1x,n1y,zn,nx,ny);

      /* dn = DM - A*A.NM
      */
      for (i=0; i<nxy; i++)  dn[i] = DM[i] - zn[i];
    
      /*** NEW NUMBER OF FORBIDDEN POINTS **********************************
      ***/
      ni1 = 0;
      for (i=0; i<nxy; i++)
        {
        if (ID[i] == 1)
          {
          if ((float)D[i] < 0.5) {if (dn[i] > 0.0) ni1++;}
          else {if (NM[i] < 0.0) ni1++;}
          }
        }

      if (ni1 != 0)
        {
        /*** UPDATIND SUPPORT D ********************************************
        ***/
        /*
        */
        if (ni1 < ni0)
          {
          ni0 = ni1;
          it_posgc = 0;
          /* */
          tmp1 = tmp2 = 0.0;
          for (i=0; i<ny; i++)
          for (j=0; j<nx; j++)
            {
            Dsav[nx*i+j]  =  D[nx*i+j];   /* Saving D in Dsav   */
            NMsav[nx*i+j] =  NM[nx*i+j];  /* Saving NM in NMsav */
            if (ID[nx*i+j] == 1)
              {
              if ((float)D[nx*i+j] < 0.5)
                {
                if (dn[nx*i+j] > tmp1) 
		   {tmp1 = dn[nx*i+j]; i1 = i; j1 = j;}
                }
              else
                {
                if (NM[nx*i+j] < tmp2) 
		   {tmp2 = NM[nx*i+j]; i2 = i; j2 = j;}
                }
              } 
            }
          if (tmp1 > -tmp2)
            {
            Dsav[nx*i1+j1] = 1;    /* add pixel (i1,j1) to Dsav    */ 
            NMsav[nx*i1+j1] = 0.0; /* set to 0 corresponding NMsav */
            }
          else
            {
            Dsav[nx*i2+j2] = 0;    /* extract pixel (i2,j2) from Dsav */
            NMsav[nx*i2+j2] = 0.0; /* set to 0 corresponding NMsav    */
            }
          }
        /* updating support
        */
        if (it_posgc < itmax_posgc)
          {
          it_posgc++;
          printf("WIPE.......: iter_positiv_gc = %d\n",it_posgc);

          for (i=0; i<nxy; i++)
            {
            if (ID[i] == 1)
              {
              if ((float)D[i] < 0.5)
                { /* if positif residu: add pixel (i,j) to D */
                if (dn[i] > 0.0) {NM[i] = 0.0; D[i] = 1;}
                }
              else
                { /* if NM negatif: extract pixel (i,j) from D */
                if (NM[i] < 0.0) {NM[i] = D[i] = 0;}
                }
              }
            }
          }
        /* if too many iterations without effect, restore previous situation
        */
        else
          {
          for (i=0; i<nxy; i++)
            {
            D[i] = Dsav[i];
            NM[i] = NMsav[i];
            }
          }
        /* new residu */
        /* zn = A*A.NM */
        zero_padding(NM,nx,ny,TMP1,n1x,n1y);
        dir_fft2D_real(TMP1,n1x,n1y);
        for (i=0; i<n1xy; i++)  TMP2[i] = TMP1[i];
        mul_fft2D_real(TMP1,DTP,TMP1,n1x,n1y);
        inv_fft2D_real(TMP1,n1x,n1y);
        zero_padding(TMP1,n1x,n1y,zn,nx,ny);
        /*** dn = DM - A*A.NM
        ***/
        for (i=0; i<nxy; i++)  dn[i] = DM[i]-zn[i];
        }
      }

    /***** UPDATING CRITERIA ********************************************
    *****/
    /*** global criterion: c = Vis_RI2 - (2.DM - A*A.NM | NM)
    ***/
    c = Vis_RI2;
    for (i=0; i<nxy; i++)  c -= (2.0*DM[i] - zn[i])*NM[i];
    /*** experimental criterion: Ce = Vis_RI2 - (2.DM - Ae*Ae.NM | NM)
         experimental residu: rn = DM - Ae*Ae.NM
    ***/
    mul_fft2D_real(TMP2,DB,TMP1,n1x,n1y);
    inv_fft2D_real(TMP1,n1x,n1y);
    zero_padding(TMP1,n1x,n1y,zn,nx,ny);  /* zn = Ae*Ae.NM  (not troncated) */
    Ce = Vis_RI2;
    for (i=0; i<nxy; i++)
      {
      Ce -= (2.0*DM[i] - zn[i])*NM[i];
      rn[i] = DM[i] - zn[i];
      }
    /*** regularization criterion: Cr 
    ***/
    Cr = c - Ce;
    /*** normalized criteria: Ce et Cr 
    ***/
    Ce = sqrt(Ce/Vis_N2);
    Cr = sqrt(Cr/Vis_N2);

    gdf_c2f_plot("_wipe_nm");
    gdf_c2f_plot("_wipe_eres");
    gdf_c2f_plot("_wipe_res");

    /* SAVE NM + Res
    */
    for (i=0; i<nxy; i++)
        NMR[i] = NM[i] + rn[i];
    gdf_c2f_plot("_wipe_nmres");

    /* Flux
    */
    XX =  0.0;
    for (i=0; i<nxy; i++)
      XX += (NM[i]+rn[i]*(float)D[i]);
/*
      XX += NM[i];
*/
    YY = (XX - flux0)/XX;
    flux0 = XX;
    printf("\nI-WIPE,  FLUX:%E; RELATIVE_VARIATION:%E\n\n", XX, YY);


    /***** INITIALISATION OF PRINCIPAL CHOICES OF WIPE *********************
    *****/
    if (w_switch == 1)
      {
      printf("WIPE.......: wipe_nbr_iter = "); fflush(stdout);
          fscanf(stdin,"%d",&n_wipe);
      printf("\n");
      size = 1.0;
      tmp1 = size*((float) Ns);
      Ns = ((int) tmp1);
      w_switch = 0;
      }
  
    /*****  MAIN CONTROL POINT OF WIPE
    *****/
    if (flag==1)
      {
      printf("Q-WIPE,  DO YOU WANT TO LEAVE CLEAN & WIPE ? [y,n] "); 
      fflush(stdout); 
      fscanf(stdin,"%s",reponse);
      if (reponse[0] == 'n')
        {
        /* old parameters for the minimisation on the non smoothed support
        */
        flag = 0;
/*
        eps_mu *= 100.0; eps *= 100.0; 
*/
        Ce = Ce_sav;
        it_wipe--;
        /* RESTORE */
        for (i=0; i<nxy; i++)
          {
          NM[i] = sav1[i];
          dn[i] = sav2[i];
          D[i]  = sav3[i];
          }
        gdf_c2f_plot("_wipe_nm");
        gdf_c2f_plot("_wipe_res");
        }
      else break;
      }
    else
      {
      /* updating Ce_inf_sav
      */
      Ce_inf_sav = Ce_inf;
      if (Ce > Ce_inf)
        {
printf("\nI-WIPE,  !!! CRITERION Ce IS ABOVE ITS LOWEST REACHED VALUE !!!");
printf("\nI-WIPE,  Present value: %lf (lowest value: %lf)\n",Ce,Ce_inf_sav);
        }
      else Ce_inf = Ce;
      }
  
    if ((it_wipe >= n_wipe) || (Ce > Ce_inf))
      {
      printf("Q-WIPE,  EXTENSION OF THE IMAGE SUPPORT ? [y, n] "); 
      fflush(stdout); 
      fscanf(stdin,"%s",reponse);
  
      /*** stopping support expansion support
      ***/
      if (reponse[0] == 'n')
        {
        printf("Q-WIPE   DO YOU WANT TO STOP WIPE ? [y, n] "); 
	fflush(stdout); 
        fscanf(stdin,"%s",reponse);
        if (reponse[0] == 'y')
          {
          /*new parameters for the minimization on the smoothed support
          */
          flag = 1;
/*
          eps_mu /= 100.0; eps /= 100.0; 
*/
          Ce_sav = Ce;
          /* SAVE
          */
          for (i=0; i<nxy; i++)
            {
            sav1[i] = NM[i];
            sav2[i] = dn[i];
            sav3[i] =  D[i];
            }
          /* Copy D in mask, then D=0.
          */
          for (i=0; i<nxy; i++)
            {
            if ((float)D[i] > 0.5) mask[i] = 1; else mask[i] = 0;
            D[i] = 0;
            }
          /* Analyzing mask in connex componants
          */
          printf("\nI-WIPE,  SMOOTHING OF THE IMAGE SUPPORT\n");
          i_conv = elimine = 0;
          while (1)
            {
            ic = -1;
            for (i=0; i<ny;i++)
            for (j=0; j<nx;j++)
              {
              if (mask[nx*i+j] == 1) {ic = i; jc = j; i = ny; break;}
              }
            if (ic == -1) break;  /* mask is 0 everywhere */
            /* number of pixels in the connex region
            */
            nbre = connex(mask,nx,ic,jc);
            i_conv++;
            printf("I-WIPE,  region %d = %d pixels\n",i_conv,nbre);
            /* interior smoothing of connex region (if large enough)
            */
            if (nbre < Ms) /* region not retained cause too small */
              elimine++;
            else
              { /* interior smoothing of connex region */
              kk = 0;
              for (i=0; i<ny; i++)
              for (j=0; j<nx; j++)
                {
                if (mask[nx*i+j] == 2)
                  { /* troncated support s is centered in (i,j); if it is contained
                       in the area where mask=2, then the trace of s support is 
                       left in D, and k=1;  otherwise k=0 */
                  k = cell(D, mask, nx, motif, m1,m2, i,j);
                  if (k == 1) kk = 1;
                  }
                }
              if (kk == 0) elimine++; /* region not retained */
              }
            /* discarding the connex region of mask
            */
            for (i=0; i<nxy; i++)  if (mask[i] == 2)  mask[i] = 0;
            }
  
          printf("I-WIPE,  %d Discarded connected region(s)\n",elimine);
          printf("I-WIPE,  MINIMIZATION ON THE SMOOTHED SUPPORT\n");
          }
        else
          { /* What needs to be done if we want to pursue Wipe  */
          k = n_wipe;
          fflush(stdout);
          printf("WIPE.............: wipe_iter = "); 
          fflush(stdout);
          fscanf(stdin,"%d", &n_wipe);
          if (n_wipe <= k) {wipe = 0; break;}
          }
        }
      }
    it_wipe++;
    }
  }
for (i=0; i<nxy; i++)  NMR[i] = rn[i]+NM[i];
/* restoring input variables to their original values
*/
if (arg != 0.0)
   {
   for (i=0; i<n1xy; i++)
       DB[i] /= arg;
   for (i=0; i<nxy; i++)
       DM[i] /= arg;
   for (i=0; i<nf; i++) Vk[i] /= arg;
   }
inv_fft2D_real(DB,n1x,n1y);
}
