#include <stdio.h>
#include <math.h>
#include <string.h>
#include "wipe.h"



/******************************************************************************/
/******************************************************************************/
void     wipe_dirty_map(
int      *P_nx,     /* number of pixels in the x direction                    */
int      *P_ny,     /* number of pixels in the y direction                    */
int      *P_nf,     /* number of visibilities in Le+ (DC included)            */
int      *P_M,      /* oversampling factor of the object workspace            */
float    *DM,       /* Dusty Map (nx*ny)                                      */
float    *Vu,       /* points of the frequency list Le (2*nf) in the uv plane */
float    *Vv,       /*        in the u and v directions                       */
float    *Vr,       /* experimental visibilities on Le+ (nf)                  */
float    *Vi,       /*        real and imaginary parts                        */
float    *Vs,
float    *Vk,       /* weighting function on Le+ (nf)                         */
char     *work)     /* workspace: 18*NX*NY+2*NX+2*NY+9*NV                     */
/******************************************************************************/
/******************************************************************************/
{
extern   void     inv_fft2D_real(), ffs1D_taper();
extern   int      inv_ffs2D_real();
extern   float    tab_tfbssf();
/**/             
register int      i,j,k;
int      nx=*P_nx, ny=*P_ny, nxy=nx*ny;
int      nf=*P_nf, M=*P_M;
int      nxs2=nx/2, nys2=ny/2;
int      n1x=2*nx, n1y=2*ny;
float    dux0, duy0, dux, duy, *tape_x, *tape_y, *tmp;

/******* computing sizes and addresses of the different internal arrays ***
*******/
k = 0;
tape_x =  (float*) &work[k]; k += n1x*sizeof(float);
tape_y =  (float*) &work[k]; k += n1y*sizeof(float);
tmp    =  (float*) &work[k]; k += n1x*n1y*sizeof(float);

dux0 = 1.0/((float) nx); dux = M*dux0/2.0;
duy0 = 1.0/((float) ny); duy = M*duy0/2.0;


/* Preprocessing and regularizing data
*/
for (k=0; k<nf; k++)
    {
    Vr[k] *= Vs[k];
    Vi[k] *= Vs[k];
    }

/******* computing Dusty Map ************************************************
*******/
ffs1D_taper(tape_x,nx,4,1.505064,80);
for (i=0; i<nx; i++) tape_x[i] = ((float) n1x)*dux/tape_x[i];
ffs1D_taper(tape_y,ny,4,1.505064,80);
for (i=0; i<ny; i++) tape_y[i] = ((float) n1y)*duy/tape_y[i];

for (k=0; k<nf; k++) {Vr[k] *= Vk[k]; Vi[k] *= Vk[k];}
Vr[0] *= 0.5;  /* DC component */
inv_ffs2D_real(tmp,n1x,n1y,Vr,Vi,Vu,Vv,nf,4,1.505064);
Vr[0] *= 2.0;  /* DC component */
for (k=0; k<nf; k++) if (Vk[k] != 0.0) {Vr[k] /= Vk[k]; Vi[k] /= Vk[k];}
inv_fft2D_real(tmp,n1x,n1y);

for (i=0; i<ny; i++)
for (j=0; j<nx; j++) 
    DM[nx*i+j] = tmp[n1x*(nys2+i)+nxs2+j]*(tape_x[i]*tape_y[j]);
}
