subroutine wipe_error(line,error)
  use gildas_def
  use gkernel_types
  use gkernel_interfaces
  use clean_def
  use clean_arrays
  use clean_types
  use gbl_format
  use gbl_message
  !----------------------------------------------------------------------
  ! GILDAS:  DECONV Internal routine
  !          Implementation of WIPE deconvolution algorithm,
  !          developed by Lannes et al. at OMP (Toulouse-France)
  !----------------------------------------------------------------------
  character(len=*) line
  logical error
  !
  real(8), parameter :: pi=3.14159265358979323846d0
  include 'gbl_memory.inc'
  !
  ! Local variables:
  logical found
  integer map_size(2),proc,ier,ndim
  integer(kind=index_length) :: dim(4)
  integer nx,ny,nv,nt,overs,size,size_l,size_ipw
  integer word
  parameter(word=1)
  real f_major,f_minor,ratio,major,minor,angle,nscale
  real map_cell(2),vref,voff,vinc,eps_mu,eps_phi
  !
  real(kind=4) :: loff,boff
  real(kind=8), parameter :: clight=299792458d-6
  !
  integer(kind=address_length) mode,theta,lambda
  type(sic_descriptor_t) :: desc
  integer(kind=address_length) hr,nb,dm,db,nm,d,ipw
  integer(kind=address_length) addr_l,addr_ipw
  character(len=4) :: rname = 'WIPE'
  !
  ! Have we passed WIPE_APER, WIPE_BEAM and WIPE_SOLVE already?
  !
  if (.not.sic_varexist('WIPE%NM')) then
    call map_message(seve%e,rname,'run WIPE_APER, WIPE_BEAM & WIPE_SOLVE.')
    error = .true.
    return
  endif
  !
  call sic_get_inte('MAP_SIZE[1]',map_size(1),error)
  if (error) then
    call map_message(seve%e,rname,'MAP_SIZE[1] undefined')
    return
  endif
  call sic_get_inte('MAP_SIZE[2]',map_size(2),error)
  if (error) then
    call map_message(seve%e,rname,'MAP_SIZE[2] undefined')
    return
  endif
  call sic_get_real('MAP_CELL[1]',map_cell(1),error)
  if ((map_cell(1).eq.0).or.error) then
    call map_message(seve%e,rname,'MAP_CELL undefined')
    return
  endif
  call sic_get_real('MAP_CELL[2]',map_cell(2),error)
  if ((map_cell(2).eq.0).or.error) then
    call map_message(seve%e,rname,'MAP_CELL undefined')
    return
  endif
  call sic_get_inte('WIPE%PROC',proc,error)
  if (proc.lt.0) then
    call map_message(seve%e,rname,'Current Working Plane: PROC undefined')
    return
  endif
  call sic_get_inte('WIPE%OVERS',overs,error)
  if (overs.lt.2) then
    call map_message(seve%e,rname,'WIPE%OVERS not correct')
    return
  endif
  call sic_get_inte('WIPE%KRY',nt,error)
  if (nt.le.0) then
    call map_message(seve%e,rname,'WIPE%KRY uninitialized')
    return
  endif
  call sic_get_real('WIPE%PMU',eps_mu,error)
  if (eps_mu.le.0) then
    call map_message(seve%e,rname,'WIPE%PMU uninitialized')
    return
  endif
  call sic_get_real('WIPE%PPHI',eps_phi,error)
  if (eps_phi.le.0) then
    call map_message(seve%e,rname,'WIPE%PPHI uninitialized')
    return
  endif
  call sic_get_real('WIPE%MAJOR',major,error)
  if (major.le.0) then
    call map_message(seve%e,rname,'MAJOR AXIS OF NB MUST BE > 0')
    return
  endif
  call sic_get_real('WIPE%MINOR',minor,error)
  if (minor.le.0) then
    call map_message(seve%e,rname,'MINOR AXIS OF NB MUST BE > 0')
    return
  endif
  !
  call sic_get_real('WIPE%ANGLE',angle,error)
  !
  nx = map_size(1)
  ny = map_size(2)
  map_cell(1) = map_cell(1)*pi/180.0/3600.0   ! In radians
  map_cell(2) = map_cell(2)*pi/180.0/3600.0
  !
  ! Compute image attributes
  nv = huv%gil%dim(2)
  !
  ! copy header info from Z buffer
  call gdf_copy_header(huv,hneat,error)
  !
  ! Neat Map allocation
  hneat%gil%ndim = 2
  hneat%gil%dim(1) = nx
  hneat%gil%dim(2) = ny
  hneat%gil%dim(3) = 1
  hneat%gil%dim(4) = 1
  vref = huv%gil%convert(1,1)
  voff = huv%gil%convert(2,1)
  vinc = huv%gil%convert(3,1)
  hneat%gil%convert(1,1) = nx/2+1
  hneat%gil%convert(1,2) = ny/2+1
  hneat%gil%convert(2,1) = 0
  hneat%gil%convert(2,2) = 0
  hneat%gil%convert(3,1) = -map_cell(1)    ! Assume EQUATORIAL system
  hneat%gil%convert(3,2) = map_cell(2)
  hneat%gil%proj_words = 9
  hneat%gil%extr_words = 0
  hneat%gil%convert(1,3) = vref-proc+1
  hneat%gil%convert(2,3) = voff
  hneat%gil%convert(3,3) = vinc
  hneat%char%code(1) = 'ANGLE'
  hneat%char%code(2) = 'ANGLE'
  hneat%loca%size = nx*ny
  hneat%char%type = 'GILDAS_IMAGE'
  !
  ! Projection information (assume equatorial system)
  hneat%char%code(1) = 'RA'
  hneat%char%code(2) = 'DEC'
  hneat%char%code(3) = 'VELOCITY'
  call equ_to_gal(hneat%gil%ra,hneat%gil%dec,0.0,0.0,   &
                  hneat%gil%epoc,hneat%gil%lii,hneat%gil%bii,loff,boff,error)
  if (hneat%gil%ptyp.eq.p_none) then
    hneat%gil%ptyp = p_azimuthal         ! Azimuthal (Sin)
    hneat%gil%a0 = hneat%gil%ra
    hneat%gil%d0 = hneat%gil%dec
  else
    hneat%gil%ptyp = p_azimuthal
  endif
  hneat%char%syst = 'EQUATORIAL'
  hneat%gil%xaxi = 1
  hneat%gil%yaxi = 2
  !
  ! New Neat place
  if (allocated(dneat)) then
    call sic_delvariable ('NEAT',.false.,error)
    deallocate(dneat)
  endif
  allocate(dneat(nx,ny,1),stat=ier)
  call sic_def_real ('NEAT',dneat,hneat%gil%ndim,   &
     &    hneat%gil%dim,.true.,error)
  hneat%r3d => dneat
  !
  ! Allocating Work Space and pointers
  size_ipw = (11*nx*ny+nt*nx*ny+nt*nt+4*nt)*word
  if (sic_getvm(size_ipw,addr_ipw).ne.1) goto 99
  ipw = gag_pointer(addr_ipw,memory)
  !
  call sic_descriptor ('WIPE%NM',desc,found)
  if (desc%dims(1)*desc%dims(2).ne.nx*ny) then
    call map_message(seve%e,rname,'Map Sizes have changed!')
    error = .true.
    return
  endif
  !
  call sic_descriptor ('WIPE%HR',desc,found)
  if (.not.found) goto 102
  hr = gag_pointer(desc%addr,memory)
  call sic_descriptor ('WIPE%SUP',desc,found)
  if (.not.found) goto 102
  d = gag_pointer(desc%addr,memory)
  call sic_descriptor ('WIPE%DM',desc,error)
  if (.not.found) goto 102
  dm = gag_pointer(desc%addr,memory)
  call sic_descriptor ('WIPE%NM',desc,found)
  if (.not.found) goto 102
  nm = gag_pointer(desc%addr,memory)
  call sic_descriptor ('WIPE%DB',desc,found)
  if (.not.found) goto 102
  db = gag_pointer(desc%addr,memory)
  call sic_descriptor ('WIPE%NB',desc,found)
  if (.not.found) goto 102
  nb = gag_pointer(desc%addr,memory)
  !
  size_l = (nt*nx*ny+2*nt)*word
  if (sic_getvm(size_l,addr_l).ne.1) goto 99
  theta = gag_pointer(addr_l,memory)
  lambda = theta+word*nt
  mode = lambda+word*nt
  !
  call sic_delvariable('WIPE%MP0',.false.,error)
  call sic_delvariable('WIPE%MP1',.false.,error)
  call sic_delvariable('WIPE%MP2',.false.,error)
  ndim = 2
  dim(1) = nx
  dim(2) = ny
  call sic_def_real_addr('WIPE%MP0',memory(mode),ndim,dim,   &
     &    .true.,error)
  call sic_def_real_addr('WIPE%MP1',memory(mode+word*nx*ny),   &
     &    ndim,dim,.true.,error)
  call sic_def_real_addr('WIPE%MP2',memory(mode+2*word*nx*ny),   &
     &    ndim,dim,.true.,error)
  !
  ! WIPE Error Analysis
  !
  call wipe_analysis(   &      !
     &    nx,   &              ! Number of pixels in X
     &    ny,   &              ! Number of pixels in Y
     &    nt,   &              ! maximum dim of the tridiagonal matrix (input)
     &    overs,   &           ! oversampling factor of the object workspace (input)
     &    eps_mu,   &          ! stop test value for relative variation of mu (input)
     &    eps_phi,   &         ! stop test value for relative variation of phi (input)
     &    memory(hr),   &      ! Synthesized aperture (input)
     &    memory(d),   &       ! Support of neat map (input)
     &    memory(dm),   &      ! Dusty Map (input)
     &    memory(nm),   &      ! Neat Map (output)
     &    memory(db),   &      ! Dusty Beam (input)
     &    memory(theta),   &   ! angles between solution and eigenvalues (output)
     &    memory(lambda),   &  ! eigenvalues (output)
     &    memory(mode),   &    ! eigenvectors (output)
     &    memory(ipw))
  !
  ! Freeing working space
  call free_vm(size_ipw,addr_ipw)
  call free_vm(size_l,addr_l)
  !
  ! Get scale (map unit to Jy/Beam) from Neat Beam
  call get_scale(memory(nb),nx,ny,nscale)
  !
  ! Saving final result, in Jy/beam units
  size = nx*ny
  call copy_map(memory(nm),dneat,size,nscale)
  !
  ! computing right parameters for flux calculation
  ratio = major/minor
  nscale = nscale*abs(hneat%gil%convert(3,1)*hneat%gil%convert(3,2))
  call flux_prep(nscale,ratio,f_major,f_minor)
  !
  hneat%gil%majo = f_major
  hneat%gil%mino = f_minor
  hneat%gil%posa = pi*angle/180.0
  save_data(code_save_neat) = .true.           ! Need to be saved
  return
  !
  99    call map_message(seve%e,rname,'Memory allocation failure')
  error = .true.
  return
  !
  102   call map_message(seve%e,rname,'SIC variable not defined')
  error = .true.
  return
end subroutine wipe_error
