#include <math.h>

#ifndef  SWAP
#define  SWAP(x,y,tmp) {tmp = x; x = y; y = tmp;}
#endif
#define  DIM    2048
#ifndef  M_PI
#define  M_PI   3.14159265358979323846
#endif
void   prepare_fft();
void   dir_real_fft1D(), dir_cplx_fft1D();
void   inv_real_fft1D(), inv_cplx_fft1D();

/*******************************************************************************
@
@ FUNCTION......: dir_fft2D_real
@
@ PURPOSE.......: Two dimensions forward Fourier transform of a real matrix
@                 using the Fast Fourier Algorithme of Cooley & Tuckey.
@
@ INPUT.........: lin = number of lines
@                 col = number of columns
@                 mat[0..lin-1][0..col-1] = real matrix
@
@ OUTPUT........: mat[0..lin-1][0..col-1] = real matrix holding complex FFT2D
@
@ VERSION.......: July 1990
@
@ AUTHOR........: Eric ANTERRIEU
@
*******************************************************************************/

/******************************************************************************/
void     dir_fft2D_real( float* mat, int lin, int col)
/******************************************************************************/
{
register int 	i, j, ni, nj;
int    lins2, cols2, ordre_lin[DIM], ordre_col[DIM];
float vec[DIM], wf_lin[DIM/2][2], wf_col[DIM/2][2];

prepare_fft(ordre_lin,wf_lin,lin); lins2 = lin/2;
prepare_fft(ordre_col,wf_col,col); cols2 = col/2;

for (i=0; i<lins2; i++)
for (j=0; j<cols2; j++)
  {
  SWAP(mat[lin*i+j],mat[lin*(i+lins2)+j+cols2],vec[0]);
  SWAP(mat[lin*(i+lins2)+j],mat[lin*i+j+cols2],vec[0]);
  }
  
for (j=0; j<col; j++) 
  {
  for (i=0; i<lin; i++)  
    {
    vec[2*i] = mat[lin*ordre_lin[i]+j];  
    }
  dir_real_fft1D(vec,wf_lin,lin);
  mat[j] = vec[0];  
  mat[lin*lins2+j]= vec[lin];
  for (i=1; i<lins2; i++)
    {  
    ni = lin-i;
    mat[lin*i+j] = vec[2*i];  
    mat[lin*ni+j] = vec[2*i+1];
    }
  }

for (i=0; i<=lins2; i+=lins2)
  { 
  for (j=0; j<col; j++)  
    {
    vec[2*j] = mat[lin*i+ordre_col[j]];
    }
  dir_real_fft1D(vec,wf_col,col);
  mat[lin*i] = vec[0];  
  mat[lin*i+cols2] = vec[col];
  for (j=1; j<cols2; j++) 
    {
    nj = col-j; 
    mat[lin*i+j] = vec[2*j];  
    mat[lin*i+nj] = vec[2*j+1]; 
    }
  }

for (i=1; i<lins2; i++)
  {  
  ni = lin-i;
  for (j=0; j<col; j++)
    { 
    vec[2*j] = mat[lin*i+ordre_col[j]];  
    vec[2*j+1] = mat[lin*ni+ordre_col[j]]; 
    }
  dir_cplx_fft1D(vec,wf_col,col);
  for (j=0; j<col; j++)
    {
    mat[lin*i+j] = vec[2*j]; 
    mat[lin*ni+j] = vec[2*j+1]; 
    } 
  }
}

/*******************************************************************************
@
@ FUNCTION......: inv_fft2D_real
@
@ PURPOSE.......: Two dimensions inverse Fourier transform of a real matrix
@                 using the Fast Fourier Algorithme of Cooley & Tuckey.
@
@ INPUT.........: lin = number of lines
@                 col = number of columns
@                 mat[0..lin-1][0..col-1] = real matrix holding complex FFT2D
@
@ OUTPUT........: mat[0..lin-1][0..col-1] = real matrix
@
@ VERSION.......: July 1990
@
@ AUTHOR........: Eric ANTERRIEU
@
*******************************************************************************/

/******************************************************************************/
void     inv_fft2D_real( float *mat, int lin, int col)
/******************************************************************************/
{
register int 	i, j, ni, nj;
int    lins2, cols2, ordre_lin[DIM], ordre_col[DIM];
float vec[DIM], wf_lin[DIM/2][2], wf_col[DIM/2][2];

prepare_fft(ordre_lin,wf_lin,lin); lins2 = lin/2;
prepare_fft(ordre_col,wf_col,col); cols2 = col/2;

for (i=0; i<=lins2; i+=lins2)
  {
  vec[0] = mat[lin*i];
  vec[1] = 0.;
  vec[col] = mat[lin*i+cols2];
  vec[col+1] = 0.;
  for (j=1; j<cols2; j++)
    {
    nj = col-j;
    vec[2*j] = mat[lin*i+j] ; 
    vec[2*nj+1] = -mat[lin*i+nj];  
    vec[2*j+1] = mat[lin*i+nj]; 
    vec[2*nj] = mat[lin*i+j]; 
    }
  inv_real_fft1D(vec,wf_col,col);
  for (j=0; j<col; j++)  mat[lin*i+j] = vec[2*ordre_col[j]];
  }

for (i=1; i<lins2; i++)
  {
  ni = lin-i;
  for (j=0; j<col; j++)
    { 
    vec[2*j] = mat[lin*i+ordre_col[j]]; 
    vec[2*j+1] = mat[lin*ni+ordre_col[j]];
    }
  inv_cplx_fft1D(vec,wf_col,col);
  for (j=0; j<col; j++)
    { 
    mat[lin*i+j] = vec[2*j]; 
    mat[lin*ni+j] = vec[2*j+1];
    }
  } 

for (j=0; j<col; j++) 
  {
  vec[0] = mat[j];
  vec[1] = 0.;
  vec[lin] = mat[lin*lins2+j]; 
  vec[lin+1] = 0.;
  for (i=1; i<lins2; i++)
    {
    ni = lin-i;
    vec[2*i] = mat[lin*i+j];
    vec[2*ni+1] = - mat[lin*ni+j];
    vec[2*i+1] = mat[lin*ni+j];
    vec[2*ni] = mat[lin*i+j];
    }
  inv_real_fft1D(vec,wf_lin,lin);	
  for (i=0; i<lin; i++)
    {
    mat[lin*i+j] = vec[2*ordre_lin[i]]; 
    mat[lin*i+j] /= (lin*col);
    }
  }

for (i=0; i<lins2; i++)
for (j=0; j<cols2; j++)
  {
  SWAP(mat[lin*i+j],mat[lin*(i+lins2)+j+cols2],vec[0]);
  SWAP(mat[lin*(i+lins2)+j],mat[lin*i+j+cols2],vec[0]);
  }
}

/*******************************************************************************
@
@ FUNCTION......: prepare_fft
@
@ PURPOSE.......: Computes the Fourier coefficients.
@
@ INPUT.........: n = number of samples (must be a power of two and less
@                     than 1024 and greater than 8)
@
@ OUTPUT........: ordre[0..n-1] = bit reverse order
@                 wf[0..n/2-1][0..1] = complex Fourier coefficients
@
@ VERSION.......: July 1990
@
@ AUTHOR........: Eric ANTERRIEU
@
*******************************************************************************/

/******************************************************************************/
void prepare_fft( int ordre[DIM], float wf[DIM/2][2], int n)
/******************************************************************************/
{
register int      i, j, k, ii;
int      ns2, ns4, ns8;
unsigned q, r, s, nn[20];
float   x, y, arg, k_arg;

ns2 = n/2;  ns4 = n/4; ns8 = n/8;  x = 1.0/sqrt(2.0);

/* !!! this function computes the FOURIER components for a      */
/*     forward FFT, i.e. : exp(-2jkPI/n) k=0:n/2-1              */
wf[0][0] = 1.0;     wf[0][1] = 0.0; 
wf[ns4][0] = 0.0;   wf[ns4][1] = -1.0;
wf[ns8][0] = x;    wf[ns8][1] = -x;
wf[3*ns8][0]= -x;  wf[3*ns8][1] = -x;
arg = 2.0*M_PI/((float) (n));
for (k=1; k<ns8; k++)
  {
  k_arg = ((float) (k))*arg;
  x = cos(k_arg); 
  y = sin(k_arg);
  wf[k][0] = x;       wf[k][1] = -y;
  wf[ns2-k][0] = -x;  wf[ns2-k][1] = -y;
  wf[ns4-k][0] = y;   wf[ns4-k][1] = -x;
  wf[ns4+k][0] = -y;  wf[ns4+k][1] = -x;
  } 

j = nn[0] = 1; 
for (i=1; i<=12; ++i)
  {
  j *= 2;
  nn[i] = j;
  if (j == n) break;
  }
ii = i-1; 
for (r=0; r<n; ++r)
  {
  k = 0; q = 0;
  j = ii;
  while (j>=0)
    {
    s = r & nn[j];
    s <<= k; k+=2;
    q = q | s;
    j--;
    }
  ordre[r] = (q >> ii);
  }
}

/*******************************************************************************
@
@ FUNCTION......: mul_fft2D_real
@
@ PURPOSE.......: Product of two FFT2D matrices
@
@ INPUT.........: lin = number of lines
@                 col = number of columns
@                 mat1[0..lin-1][0..col-1] = real matrix holding complex FFT2D
@                 mat2[0..lin-1][0..col-1] = real matrix holding complex FFT2D
@
@ OUTPUT........: mat3[0..lin-1][0..col-1] = mat1.mat2
@
@ VERSION.......: July 1990
@
@ AUTHOR........: Eric ANTERRIEU
@
*******************************************************************************/

/******************************************************************************/
void     mul_fft2D_real( float *mat1, float *mat2, float *mat3, int lin, int col)
/******************************************************************************/
{
register int      i, j;
int      lins2 = lin/2, cols2 = col/2;
float   re1, im1, re2, im2;

mat3[0] = mat1[0]*mat2[0];
mat3[lin*lins2] = mat1[lin*lins2]*mat2[lin*lins2];
mat3[cols2] = mat1[cols2]*mat2[cols2];
mat3[lin*lins2+cols2] = mat1[lin*lins2+cols2]*mat2[lin*lins2+cols2];
for (j=1; j<cols2; j++)
  {
  re1 = mat1[j]; im1 = mat1[col-j];
  re2 = mat2[j]; im2 = mat2[col-j];
  mat3[j]     = re1*re2 - im1*im2; /* re */
  mat3[col-j] = re1*im2 + im1*re2; /* im */
  re1 = mat1[lin*lins2+j]; im1 = mat1[lin*lins2+col-j];
  re2 = mat2[lin*lins2+j]; im2 = mat2[lin*lins2+col-j];
  mat3[lin*lins2+j]     = re1*re2 - im1*im2; /* re */
  mat3[lin*lins2+col-j] = re1*im2 + im1*re2; /* im */
  }
for (i=1; i<lins2; i++)
for (j=0; j<col; j++)
  {
  re1 = mat1[lin*i+j]; im1 = mat1[lin*(lin-i)+j];
  re2 = mat2[lin*i+j]; im2 = mat2[lin*(lin-i)+j];
  mat3[lin*i+j]       = re1*re2 - im1*im2; /* re */
  mat3[lin*(lin-i)+j] = re1*im2 + im1*re2; /* im */
  }
}

/*******************************************************************************
@
@ FUNCTION......: zero_padding
@
@ PURPOSE.......: zero padding
@
@ INPUT.........: lin1 = number of lines
@                 col1 = number of columns
@                 mat1[0..lin1-1][0..col1-1] = real matrix
@
@ OUTPUT........: lin2 = number of lines
@                 col2 = number of columns
@                 mat2[0..lin2-1][0..col2-1] = real matrix
@
@ VERSION.......: July 1990
@
@ AUTHOR........: Eric ANTERRIEU
@
*******************************************************************************/

/******************************************************************************/
void zero_padding( float *mat1, int lin1, int col1, float *mat2, int lin2, int col2)
/******************************************************************************/
{
register int      i, j;
int      lins4, cols4;

if ((lin1 > lin2) && (col1 > col2))
  {
  lins4 = lin1/4;
  cols4 = col1/4;
  for (i=0; i<lin2; i++)
  for (j=0; j<col2; j++) mat2[lin2*i+j] = mat1[lin1*(i+lins4)+j+cols4];
  }
if ((lin1 < lin2) && (col1 < col2))
  {
  for (i=0; i<lin2; i++)
  for (j=0; j<col2; j++) mat2[lin2*i+j] = 0.0;
  lins4 = lin2/4;
  cols4 = col2/4;
  for (i=0; i<lin1; i++)
  for (j=0; j<col1; j++) mat2[lin2*(i+lins4)+j+cols4] = mat1[lin1*i+j];
  }

}

/*******************************************************************************
@
@ FUNCTION......: dir_real_fft1D
@
@ PURPOSE.......: Computes the forward 1D Discrete Fourier Transform of a
@                 real vector using the Fast Fourier Transform algorithm
@                 developed by Cooley & Tuckey.
@
@ INPUT.........: n = number of samples
@                 vec[0..n-1] = real vector holding Re parts of data
@                 wf[0..n/2-1][0..1] = Fourier coefficients
@
@ OUTPUT........: vec[0..n-1] = real vector holding Re and Im parts of DFT
@
@ VERSION.......: July 1990
@
@ AUTHOR........: Andre LANNES
@
*******************************************************************************/

/******************************************************************************/
void   dir_real_fft1D( float *vec, float wf[DIM/2][2], int n)
/******************************************************************************/
{
unsigned   i, j, k, m, m1, m2, p;
float     r, s;
float     *xi_re, *xi_im, *xl_re, *xl_im, wf_re, wf_im;
float     *xic_re, *xic_im, *xlc_re, *xlc_im, *suite;

i=0; 
suite = vec;
while (i < n) 
  { 	
  xi_re = suite; 
  xl_re = xi_re + 2; 
  *(xi_re + 1) = 0.; 
  *(xl_re + 1) = 0.;
  r = (*xl_re);	
  (*xl_re) = (*xi_re) - r;
  (*xi_re) = (*xi_re) + r;
  i += 2; 
  suite += 4;
  }

p = n>>2;
i = 0; 
suite = vec;
while (i < n) 
  {
  xi_re = suite;
  xl_re = xi_re + 4; 
  r = (*xl_re);	
  (*xl_re) = (*xi_re) - r;
  (*xi_re) = (*xi_re) + r;
  xi_re = xi_re + 2; 
  xi_im = xi_re + 1; 
  xl_re = xi_re + 4; 
  xl_im = xl_re + 1; 
  (*xi_im) = -(*xl_re);
  (*xl_im) =  (*xl_re);
  (*xl_re) =  (*xi_re);
  i += 4; 
  suite += 8;   
  }

m = 4; 
m1 = 2;
while (p>1) 
  {
  p = p>>1; 
  m2 = m<<1; 
  i=0; 
  suite = vec;
  while (i < n) 
    {		
    xi_re = suite; 
    xl_re = xi_re + m2; 
    r = (*xl_re);
    (*xl_re) = (*xi_re) - r;
    (*xi_re) = (*xi_re) + r;
    xic_re = suite = xl_re + m2;
    xlc_re = xl_re;
    j = 1; 
    k = p;
    while (j<m1) 
      {
      xi_re += 2; 
      xi_im = xi_re + 1; 
      xl_re = xi_re + m2; 
      xl_im = xl_re + 1; 
      xic_re -= 2;
      xlc_re -= 2;
      wf_re = wf[k][0]; wf_im = wf[k][1];
      r = (*xl_re)*wf_re - (*xl_im)*wf_im;
      s = (*xl_re)*wf_im + (*xl_im)*wf_re;
      (*xl_re) = *xlc_re = (*xi_re) - r;
      (*xi_re) = (*xic_re) = (*xi_re) + r;
      (*xl_im) = (*xi_im) - s;
      xlc_im = xlc_re + 1;
      (*xlc_im) = - (*xl_im);
      (*xi_im) = (*xi_im) + s;
      xic_im = xic_re + 1;
      (*xic_im) = - (*xi_im);
      ++j; 
      k += p;
      }
    xi_re = xi_re + 2; 
    xi_im = xi_re + 1; 
    xl_re = xi_re + m2; 
    xl_im = xl_re + 1; 
    r =  (*xl_im);
    s = - (*xl_re); 
    (*xl_re) = (*xi_re) - r;
    (*xi_re) = (*xi_re) + r;
    (*xl_im) = (*xi_im) - s;
    (*xi_im) = (*xi_im) + s;
    i += m2;
    }
  m1 = m; 
  m = m2;
  }
}

/*******************************************************************************
@
@ FUNCTION......: dir_cplx_fft1D
@
@ PURPOSE.......: Computes the forward 1D Discrete Fourier Transform of a
@                 complex vector using the Fast Fourier Transform algorithm
@                 developed by Cooley & Tuckey.
@
@ INPUT.........: n = number of samples
@                 vec[0..n-1] = real vector holding Re and Im parts of data
@                 wf[0..n/2-1][0..1] = Fourier coefficients
@
@ OUTPUT........: vec[0..n-1] = real vector holding Re and Im parts of DFT
@
@ VERSION.......: July 1990
@
@ AUTHOR........: Andre LANNES
@
*******************************************************************************/

/******************************************************************************/
void   dir_cplx_fft1D( float *vec, float wf[DIM/2][2], int n)
/******************************************************************************/
{
unsigned  i, j, k, m, m1, m2, p;
float    r, s;
float    *xi_re, *xi_im, *xl_re, *xl_im, wf_re, wf_im;

i = 0; 
xl_im = vec - 1;
while (i < n) 
  {
  xi_re = xl_im + 1; 
  xi_im = xi_re + 1; 
  xl_re = xi_re + 2; 
  xl_im = xl_re + 1; 
  r = (*xl_re);
  s = (*xl_im);
  (*xl_re) = (*xi_re) - r;
  (*xi_re) = (*xi_re) + r;
  (*xl_im) = (*xi_im) - s;
  (*xi_im) = (*xi_im) + s;
  i += 2;
  }

m1 = 1; 
m = 2; 
p = n>>1;
while (p>1) 
  {
  p = p>>1; 
  m2 = m<<1; 
  i = 0; 
  xl_im = &(*(vec + n*i - 1));
  while (i < n) 
    {
    xi_re = xl_im + 1; 
    xi_im = xi_re + 1; 
    xl_re = xi_re + m2; 
    xl_im = xl_re + 1; 
    r = (*xl_re);
    s = (*xl_im);
    (*xl_re) = (*xi_re) - r;
    (*xi_re) = (*xi_re) + r;
    (*xl_im) = (*xi_im) - s;
    (*xi_im) = (*xi_im) + s;
    j = 1; 
    k = p;
    while (j<m1) 
      {
      xi_re = xi_re + 2; 
      xi_im = xi_re + 1; 
      xl_re = xi_re + m2; 
      xl_im = xl_re + 1; 
      wf_re = wf[k][0]; 
      wf_im = wf[k][1];
      r = (*xl_re)*(wf_re) -  (*xl_im)*(wf_im);
      s = (*xl_re)*(wf_im) +  (*xl_im)*(wf_re);
      (*xl_re) = (*xi_re) - r;
      (*xi_re) = (*xi_re) + r;
      (*xl_im) = (*xi_im) - s;
      (*xi_im) = (*xi_im) + s;
      ++j; 
      k += p;
      }
    xi_re = xi_re + 2; 
    xi_im = xi_re + 1; 
    xl_re = xi_re + m2; 
    xl_im = xl_re + 1; 
    r =  (*xl_im); 
    s = -(*xl_re);
    (*xl_re) = (*xi_re) - r;
    (*xi_re) = (*xi_re) + r;
    (*xl_im) = (*xi_im) - s;
    (*xi_im) = (*xi_im) + s;
    ++j; 
    k += p;
    while (j<m)
      {
      xi_re = xi_re + 2; 
      xi_im = xi_re + 1; 
      xl_re = xi_re + m2; 
      xl_im = xl_re + 1; 
      wf_re = wf[k][0]; 
      wf_im = wf[k][1];
      r = (*xl_re)*(wf_re) -  (*xl_im)*(wf_im);
      s = (*xl_re)*(wf_im) +  (*xl_im)*(wf_re);
      (*xl_re) = (*xi_re) - r;
      (*xi_re) = (*xi_re) + r;
      (*xl_im) = (*xi_im) - s;
      (*xi_im) = (*xi_im) + s;
      ++j; 
      k += p;
      }
    i += m2;
    }
  m1 = m; 
  m = m2;
  }
}

/*******************************************************************************
@
@ FUNCTION......: inv_real_fft1D
@
@ PURPOSE.......: Computes the inverse 1D Discrete Fourier Transform of a
@                 real vector using the Fast Fourier Transform algorithm
@                 developed by Cooley & Tuckey.
@
@ INPUT.........: n = number of samples
@                 vec[0..n-1] = real vector holding Re and Im parts of DFT
@                 wf[0..n/2-1][0..1] = Fourier coefficients
@
@ OUTPUT........: vec[0..n-1] = real vector holding Re parts of data
@
@ VERSION.......: July 1990
@
@ AUTHOR........: Andre LANNES
@
*******************************************************************************/

/******************************************************************************/
void   inv_real_fft1D( float *vec, float wf[DIM/2][2], int n)
/******************************************************************************/
{
unsigned   i, j, k, m, m1, m2, p;
float     r, s, rr, ss;
float     *xi_re, *xi_im, *xl_re, *xl_im, wf_re, wf_im;
float     *xic_re, *xic_im, *xlc_re, *xlc_im, *suite;


m2 = n; 
m = n>>1;
p = 1;
while (m>2) 
  {
  m1 = m>>1; 
  i = 0; 
  suite = vec + 2*i;
  while (i < n) 
    {
    xi_re = suite;
    xl_re = xi_re + m2; 
    r = (*xl_re);
    (*xl_re) = (*xi_re) - r;
    (*xi_re) = (*xi_re) + r;
    xi_im = xi_re + 1; (*xi_im) = 0;
    xl_im = xl_re + 1; (*xl_im) = 0;
    xic_re = suite = xl_re + m2;
    xlc_re = xl_re;
    j = 1; 
    k = p;
    while (j<m1) 
      {
      xi_re += 2; xi_im = xi_re + 1; 
      xl_re = xi_re + m2; xl_im = xl_re + 1; 
      xic_re -= 2;
      xlc_re -= 2;
      wf_re = wf[k][0]; wf_im = wf[k][1];
      r = (*xi_re) + (*xl_re);
      s = (*xi_im) + (*xl_im);
      rr = (*xi_re) - (*xl_re);
      ss = (*xi_im) - (*xl_im);
      (*xi_re) = *xlc_re = r;
      (*xi_im) = s;
      xlc_im = xlc_re + 1;
      (*xlc_im) = - s;
      (*xl_re) = (*xic_re) = wf_re*rr + wf_im*ss;
      (*xl_im)  = wf_re*ss - wf_im*rr;
      xic_im = xic_re + 1;
      (*xic_im) = - (*xl_im);
      ++j; k += p;
      }
    xi_re = xi_re + 2; xi_im = xi_re + 1; 
    xl_re = xi_re + m2; xl_im = xl_re + 1; 
    r = (*xi_re) + (*xl_re);
    s = (*xi_im) + (*xl_im);
    rr = (*xi_re) - (*xl_re);
    ss = (*xi_im) - (*xl_im);
    (*xi_re) = r;
    (*xi_im) = s;
    (*xl_re)  =  - ss;
    (*xl_im)  =  rr;
    i += m2;
    }
  m2 = m ; 
  m = m1;
  p = p<<1; 
  } 

i = 0; 
suite = vec + 2*i;
while (i < n) 
  {
  xi_re = suite; 
  xl_re = xi_re + 4; 
  r = (*xl_re);
  (*xl_re) = (*xi_re) - r;
  (*xi_re) = (*xi_re) + r;
  *(xi_re + 1) = 0.;
  *(xl_re + 1) = 0.;
  xi_re += 2; xi_im = xi_re + 1; 
  xl_re = xi_re + 4; xl_im = xl_re + 1; 
  r = (*xi_re) + (*xl_re);
  ss = (*xi_im) - (*xl_im);
  (*xi_re) = r; 
  (*xl_re)  = - ss;
  (*xi_im) = 0.; (*xl_im)  =  0.;
  i += 4; 
  suite += 8;
  }

i = 0; 
suite = vec;
while (i < n) 
  {
  xi_re = suite; 
  xl_re = xi_re + 2; 
  r = (*xl_re);
  (*xl_re) = (*xi_re) - r;
  (*xi_re) = (*xi_re) + r;
  i += 2; 
  suite += 4;
  }
}

/*******************************************************************************
@
@ FUNCTION......: inv_cplx_fft1D
@
@ PURPOSE.......: Computes the inverse 1D Discrete Fourier Transform of a
@                 complex vector using the Fast Fourier Transform algorithm
@                 developed by Cooley & Tuckey.
@
@ INPUT.........: n = number of samples
@                 vec[0..n-1] = real vector holding Re and Im parts of DHT
@                 wf[0..n/2-1][0..1] = Fourier coefficients
@
@ OUTPUT........: vec[0..n-1] = real vector holding Re and Im parts of data
@
@ VERSION.......: July 1990
@
@ AUTHOR........: Andre LANNES
@
*******************************************************************************/

/******************************************************************************/
void   inv_cplx_fft1D( float *vec, float wf[DIM/2][2], int n)
/******************************************************************************/
{
unsigned   i, j, k, m, m1, m2, p;
float     r, s;
float     *xi_re, *xi_im, *xl_re, *xl_im, wf_re, wf_im;

i = 0; 
xl_im = vec - 1;
while (i < n) 
  { 
  xi_re = xl_im + 1; xi_im = xi_re + 1; 
  xl_re = xi_re + 2; xl_im = xl_re + 1; 
  r = (*xl_re);
  s = (*xl_im);
  (*xl_re) = (*xi_re) - r;
  (*xi_re) = (*xi_re) + r;
  (*xl_im) = (*xi_im) - s;
  (*xi_im) = (*xi_im) + s;
  i += 2;
  }

m1 = 1; 
m = 2; 
p = n>>1;

while (p>1) 
  {
  p = p>>1;         
  m2 = m<<1;
  i = 0; 
  xl_im = vec + 2*i - 1;
  while (i < n) 
    {		
    xi_re = xl_im + 1; xi_im = xi_re + 1; 
    xl_re = xi_re + m2; xl_im = xl_re + 1; 
    r = (*xl_re);
    s = (*xl_im);
    (*xl_re) = (*xi_re) - r;
    (*xi_re) = (*xi_re) + r;
    (*xl_im) = (*xi_im) - s;
    (*xi_im) = (*xi_im) + s;
    j = 1; 
    k = p;
    while (j<m1) 
      {
      xi_re = xi_re + 2; xi_im = xi_re + 1; 
      xl_re = xi_re + m2; xl_im = xl_re + 1; 
      wf_re = wf[k][0]; wf_im = wf[k][1];
      r =  (*xl_re)*(wf_re) + (*xl_im)*(wf_im);
      s = -(*xl_re)*(wf_im) + (*xl_im)*(wf_re);
      (*xl_re) = (*xi_re) - r;
      (*xi_re) = (*xi_re) + r;
      (*xl_im) = (*xi_im) - s;
      (*xi_im) = (*xi_im) + s;
      ++j; 
      k += p;
      }
    xi_re = xi_re + 2; xi_im = xi_re + 1; 
    xl_re = xi_re + m2; xl_im = xl_re + 1; 
    r = -(*xl_im); 
    s =  (*xl_re);
    (*xl_re) = (*xi_re) - r;
    (*xi_re) = (*xi_re) + r;
    (*xl_im) = (*xi_im) - s;
    (*xi_im) = (*xi_im) + s;
    ++j; 
    k += p;
    while (j<m)
      {
      xi_re = xi_re + 2; xi_im = xi_re + 1; 
      xl_re = xi_re + m2; xl_im = xl_re + 1; 
      wf_re = wf[k][0]; wf_im = wf[k][1];
      r =  (*xl_re)*(wf_re) + (*xl_im)*(wf_im);
      s = -(*xl_re)*(wf_im) + (*xl_im)*(wf_re);
      (*xl_re) = (*xi_re) - r;
      (*xi_re) = (*xi_re) + r;
      (*xl_im) = (*xi_im) - s;
      (*xi_im) = (*xi_im) + s;
      ++j; 
      k += p;
      }
    i += m2;
    }
  m1 = m; 
  m = m2;
  }
}
