#include <stdio.h>
#include <math.h>

#ifndef  SWAP
#define  SWAP(x,y,tmp) {tmp = x; x = y; y = tmp;}
#endif

/******************************************************************************/
/******************************************************************************/
int      ql_tridiag( float *d, float *e, float *z, int n, int it)
/******************************************************************************/
/******************************************************************************/
{
register int      i, j, k, l, m;
float   s, r, p, g, f, dd, c, b;

/* renumber the subdiagonal elements (don't ask why!)
*/
for (i=1; i<n; i++) e[i-1] = e[i];   
e[n-1] = 0.0;

/* initialize basis vectors
*/
if (z != NULL)
  {
  for (i=0; i<n; i++)
  for (j=0; j<n; j++) z[it*i+j] = 0.0;
  for (i=0; i<n; i++) z[it*i+i] = 1.0;
  }

for (k=0; k<n; k++)
  {
  l = 0;
  do {
    /* look for the smallest subdiagonal element (up to roundoff errors)
    */
    for (m=k; m<n-1; m++)
      {
      dd = (float)fabs((double)d[m]) + (float)fabs((double)d[m+1]);
      if (((float)fabs((double)e[m]) + dd) == dd) break;
      }
    /* general case: process the 2x2 matrix located in k,k+1 on the diagonal
    */
    if (m != k)
      {
      if (l++ == it) return(-1);
      /* compute smallest eigenvalue of this 2x2 matrix
      */
      g = (d[k+1] - d[k])/(2.0 * e[k]); /* theta */
      /* implicit shift of this eigenvalue
      */
      if (g < 0.0) 
         g = d[m] - d[k] + e[k]/(g - (float)sqrt((double)(g*g) + 1.0));
      else         
         g = d[m] - d[k] + e[k]/(g + (float)sqrt((double)(g*g) + 1.0));
      s = c = 1.0; /* s stands for sin(phi) and c for cos(phi) */
      p = 0.0;
      /* Givens rotations of basis vectors (from m-1 to k)
      */
      for (i=m-1; i>=k; i--)
        {
        f = s * e[i];
        b = c * e[i];
        if (fabs(f) >= fabs(g))
          {
          c = g/f; /* ctan(phi) */
          r = (float)sqrt((double)(c*c) + 1.0);
          e[i+1] = f * r;
          c *= (s=1.0/r);
          }
        else
          {
          s = f/g; /* tan(phi) */
          r = (float)sqrt((double)(s*s) + 1.0);
          e[i+1] = g * r;
          s *= (c=1.0/r);
          }
        g = d[i+1] - p;
        r = (d[i]-g)*s + 2.0*c*b;
        p = s * r;
        d[i+1] = g + p;
        g = c*r - b;
        /* loop over the eigenvectors
        */
        if (z != NULL)
          {
          for (j=0; j<n; j++)
            {
            f = z[it*j+i+1];
            z[it*j+i+1] = s*z[it*j+i] + c*f;
            z[it*j+i]   = c*z[it*j+i] - s*f;
            }
          }
        }
      d[k] = d[k] - p;
      e[k] = g;
      e[m] = 0.0;
      }
    } while (m != k);
  }

/* sort the eigenvalues and the corresponding eigenvectors
*/
for (i=0; i<n; i++)
  {
  p = d[k=i];
  for (j=i+1; j<n; j++) if (d[j] < p) p = d[k=j];
  if (k != i)
    {
    d[k] = d[i]; d[i] = p;
    if (z != NULL)
      {
      for (j=0; j<n; j++) SWAP(z[it*j+i],z[it*j+k],p);
      }
    }
  }

return(0);
}
