subroutine wipe_solve(line,error)
  use gildas_def
  use gkernel_types
  use gkernel_interfaces
  use clean_def
  use clean_arrays
  use clean_support
  use clean_types
  use gbl_format
  use gbl_message
  !----------------------------------------------------------------------
  ! GILDAS:  DECONV Internal routine
  !          Implementation of WIPE deconvolution algorithm,
  !          developed by Lannes et al. at OMP (Toulouse-France)
  !----------------------------------------------------------------------
  character(len=*) line
  logical error
  !
  real(8), parameter :: pi=3.14159265358979323846d0
  include 'gbl_memory.inc'
  !
  ! Local variables:
  logical found
  integer map_size(2),box(4),nplanes
  integer(kind=index_length) :: dim(4)
  integer nx,ny,nc,nv,ier,proc,ndim,size,word,size_ipw
  integer nt,overs,mnwi,mnci,mncg,mnpos
  parameter(word=1)
  real nscale,vref,voff,vinc,map_cell(2),eps_pos
  real f_major,f_minor,ratio,major,minor,angle
  real trse,eps_mu,eps_phi,lmnb,umnb
  real cpu1,cpu0
  character(len=message_length) :: chain
  !
  integer f_nclean, f_nwipe
  real f_cgain
  logical query
  real(kind=4) :: loff,boff
  !
  integer(kind=address_length) addr
  integer(kind=address_length) hr,nb,dm,db,vr,vi,vn,vs,vk,vu,vv
  integer(kind=address_length) nm,nmr,res,d,cb,ipw,addr_ipw
  type(sic_descriptor_t) :: desc
  character(len=4) :: rname = 'WIPE'
  !
  call gag_cpu(cpu0)
  !
  ! /INTERACTIVE option
  query = sic_present(1,0)
  !
  ! Have we passed WIPE_APER and WIPE_BEAM already?
  !
  call sic_descriptor('WIPE%NB',desc,found)
  if (.not.found) then
     call map_message(seve%e,rname,'run WIPE_APER & WIPE_BEAM first.')
     error = .true.
     return
  endif
  !
  ! reading SIC variables
  !
  call sic_get_inte('MAP_SIZE[1]',map_size(1),error)
  if (error) then
     call map_message(seve%e,rname,'MAP_SIZE[1] undefined')
     return
  endif
  call sic_get_inte('MAP_SIZE[2]',map_size(2),error)
  if (error) then
     call map_message(seve%e,rname,'MAP_SIZE[2] undefined')
     return
  endif
  call sic_get_real('MAP_CELL[1]',map_cell(1),error)
  if ((map_cell(1).eq.0).or.error) then
     call map_message(seve%e,rname,'MAP_CELL undefined')
     return
  endif
  call sic_get_real('MAP_CELL[2]',map_cell(2),error)
  if ((map_cell(2).eq.0).or.error) then
     call map_message(seve%e,rname,'MAP_CELL undefined')
     return
  endif
  call sic_get_inte('WIPE%PROC',proc,error)
  if (proc.lt.0) then
     call map_message(seve%e,rname,'Current Working Plane: WIPE%PROC undefined')
     return
  endif
  call sic_get_inte('WIPE%KRY',nt,error)
  if (nt.le.0) then
     call map_message(seve%e,rname,'WIPE%KRY uninitialized')
     return
  endif
  call sic_get_inte('WIPE%OVERS',overs,error)
  if (overs.lt.2) then
     call map_message(seve%e,rname,'WIPE%OVERS not correct')
     return
  endif
  call sic_get_inte('WIPE%MNWI',mnwi,error)
  if (mnwi.lt.1) then
     call map_message(seve%e,rname,'max number wipe iterations too small')
     return
  endif
  call sic_get_inte('WIPE%MNCI',mnci,error)
  if (mnci.lt.1) then
     call map_message(seve%e,rname,'max number clean iterations too small')
     return
  endif
  call sic_get_inte('WIPE%MNCG',mncg,error)
  if (mncg.lt.1) then
     call map_message(seve%e,rname,'max number CG iterations too small')
     return
  endif
  call sic_get_inte('WIPE%MNPOS',mnpos,error)
  if (mnpos.lt.1) then
     call map_message(seve%e,rname,'max number positivity iterations < 1?')
     return
  endif
  call sic_get_real('WIPE%EPSPOS',eps_pos,error)
  if (eps_pos.le.0) then
     call map_message(seve%e,rname,'WIPE%EPSPOS must be > 0')
     return
  endif
  call sic_get_real('WIPE%MAJOR',major,error)
  if (major.le.0) then
     call map_message(seve%e,rname,'MAJOR AXIS OF NB MUST BE > 0')
     return
  endif
  call sic_get_real('WIPE%MINOR',minor,error)
  if (minor.le.0) then
     call map_message(seve%e,rname,'MINOR AXIS OF NB MUST BE > 0')
     return
  endif
  !
  call sic_get_real('WIPE%ANGLE',angle,error)
  !
  call sic_get_real('WIPE%TRSE',trse,error)
  if (trse.le.0) then
     call map_message(seve%e,rname,'WIPE%TRSE uninitialized')
     return
  endif
  call sic_get_real('WIPE%PMU',eps_mu,error)
  if (eps_mu.le.0) then
     call map_message(seve%e,rname,'WIPE%PMU uninitialized')
     return
  endif
  call sic_get_real('WIPE%PPHI',eps_phi,error)
  if (eps_phi.le.0) then
     call map_message(seve%e,rname,'WIPE%PPHI uninitialized')
     return
  endif
  call sic_get_real('WIPE%LMNB',lmnb,error)
  if (lmnb.lt.0.or.lmnb.gt.1) then
     call map_message(seve%e,rname,'WIPE%LMNB uninitialized')
     return
  endif
  call sic_get_real('WIPE%UMNB',umnb,error)
  if (lmnb.lt.0.or.umnb.gt.1) then
     call map_message(seve%e,rname,'WIPE%UMNB uninitialized')
     return
  endif
  !
  nx = map_size(1)
  ny = map_size(2)
  map_cell(1) = map_cell(1)*pi/180.0/3600.0   ! In radians
  map_cell(2) = map_cell(2)*pi/180.0/3600.0
  !
  ! copy header info to Z buffer
  nc = huv%gil%nchan
  nplanes = huv%gil%dim(1)
  nv = huv%gil%dim(2)
  call sic_get_inte('WIPE%PROC',proc,error)
  if ((proc.lt.0).or.(proc.gt.nc)) then
     call map_message(seve%e,rname,'Current Working Plane: PROC undefined')
     return
  elseif  (proc.eq.0) then
     proc = 1
  endif
  !
  ! copy header info from Z buffer
  call gdf_copy_header(huv,hneat,error)
  !
  ! Neat Map allocation
  ! Compute image attributes
  hneat%gil%ndim = 2
  hneat%gil%dim(1) = nx
  hneat%gil%dim(2) = ny
  hneat%gil%dim(3) = 1
  hneat%gil%dim(4) = 1
  vref = huv%gil%convert(1,1)
  voff = huv%gil%convert(2,1)
  vinc = huv%gil%convert(3,1)
  hneat%gil%convert(1,1) = nx/2+1
  hneat%gil%convert(1,2) = ny/2+1
  hneat%gil%convert(2,1) = 0
  hneat%gil%convert(2,2) = 0
  hneat%gil%convert(3,1) = -map_cell(1)    ! Assume EQUATORIAL system
  hneat%gil%convert(3,2) = map_cell(2)
  hneat%gil%proj_words = 9
  hneat%gil%extr_words = 0
  hneat%gil%convert(1,3) = vref-proc+1
  hneat%gil%convert(2,3) = voff
  hneat%gil%convert(3,3) = vinc
  hneat%char%code(1) = 'ANGLE'
  hneat%char%code(2) = 'ANGLE'
  hneat%loca%size = nx*ny
  hneat%char%type = 'GILDAS_IMAGE'
  !
  ! Projection information (assume equatorial system)
  hneat%char%code(1) = 'RA'
  hneat%char%code(2) = 'DEC'
  hneat%char%code(3) = 'VELOCITY'
  call equ_to_gal(hneat%gil%ra,hneat%gil%dec,0.0,0.0,   &
                  hneat%gil%epoc,hneat%gil%lii,hneat%gil%bii,loff,boff,error)
  if (hneat%gil%ptyp.eq.p_none) then
     hneat%gil%ptyp = p_azimuthal  ! Azimuthal (Sin)
     hneat%gil%a0 = hneat%gil%ra
     hneat%gil%d0 = hneat%gil%dec
  else
     hneat%gil%ptyp = p_azimuthal
  endif
  hneat%char%syst = 'EQUATORIAL'
  hneat%gil%xaxi = 1
  hneat%gil%yaxi = 2
  !
  ! New Neat place
  if (allocated(dneat)) then
     call sic_delvariable ('NEAT',.false.,error)
     deallocate(dneat)
  endif
  allocate(dneat(nx,ny,1),stat=ier)
  call sic_def_real ('NEAT',dneat,hneat%gil%ndim,   &
       &    hneat%gil%dim,.true.,error)
  hneat%r3d => dneat
  !
  ! Dirty Map allocation
  call gdf_copy_header(hneat,hdirty,error)   ! IPD
  if (allocated(ddirty)) then
     call sic_delvariable ('DIRTY',.false.,error)
     deallocate(ddirty)
  endif
  allocate(ddirty(nx,ny,1),stat=ier)
  hdirty%loca%size = nx*ny
  call sic_def_real ('DIRTY',ddirty,hdirty%gil%ndim,   &
       &    hdirty%gil%dim,.true.,error)
  hdirty%r3d => ddirty
  !
  ! Residual Map allocation
  call gdf_copy_header(hneat,hresid,error)   ! IPR
  if (allocated(dresid)) then
     call sic_delvariable ('RESIDUAL',.false.,error)
     deallocate(dresid)
  endif
  allocate(dresid(nx,ny,1),stat=ier)
  hresid%loca%size = nx*ny
  call sic_def_real ('RESIDUAL',dresid,hresid%gil%ndim,   &
       &    hresid%gil%dim,.true.,error)
  hresid%r3d => dresid
  !
  ! NMR Map allocation
  call gdf_copy_header(hneat,hnmr,error) ! IPN
  if (allocated(dnmr)) then
     call sic_delvariable ('NMR',.false.,error)
     deallocate(dnmr)
  endif
  allocate(dnmr(nx,ny,1),stat=ier)
  hnmr%loca%size = nx*ny
  call sic_def_real ('NMR',dnmr,hnmr%gil%ndim,   &
       &    hnmr%gil%dim,.true.,error)
  hnmr%r3d => dnmr
  !
  ! Allocating Work Space and pointers
  !
  size_ipw = (27*nx*ny+4*nt)*word
  if (sic_getvm(size_ipw,addr_ipw).ne.1) goto 99
  ipw = gag_pointer(addr_ipw,memory)
  !
  ! grabbing and allocating additional SIC  variables
  call sic_descriptor ('WIPE%NB',desc,found)
  if (desc%dims(1)*desc%dims(2).ne.nx*ny) then
     call map_message(seve%e,rname,'Map Sizes have changed!')
     error = .true.
     return
  endif
  !
  call sic_descriptor ('WIPE%VK',desc,found)
  if (desc%dims(1).ne.nv) then
     call map_message(seve%e,rname,'uv coverage size have changed!')
     error = .true.
     return
  endif
  !
  call sic_descriptor ('WIPE%HR',desc,found)
  if (.not.found) goto 102
  hr = gag_pointer(desc%addr,memory)
  call sic_descriptor ('WIPE%DB',desc,found)
  if (.not.found) goto 102
  db = gag_pointer(desc%addr,memory)
  call sic_descriptor ('WIPE%NB',desc,found)
  if (.not.found) goto 102
  nb = gag_pointer(desc%addr,memory)
  call sic_descriptor ('WIPE%VU',desc,found)
  if (.not.found) goto 102
  vu = gag_pointer(desc%addr,memory)
  call sic_descriptor ('WIPE%VV',desc,found)
  if (.not.found) goto 102
  vv = gag_pointer(desc%addr,memory)
  call sic_descriptor ('WIPE%VN',desc,found)
  if (.not.found) goto 102
  vn = gag_pointer(desc%addr,memory)
  call sic_descriptor ('WIPE%VS',desc,found)
  if (.not.found) goto 102
  vs = gag_pointer(desc%addr,memory)
  call sic_descriptor ('WIPE%VK',desc,found)
  if (.not.found) goto 102
  vk = gag_pointer(desc%addr,memory)
  !
  call sic_descriptor ('WIPE%NM',desc,found)
  ! we have 6 arrays!
  size = 6*desc%dims(1)*desc%dims(2)
  if (found) then
     if (size.ne.6*nx*ny) then
        call sic_delvariable('WIPE%NM',.false.,error)
        call sic_delvariable('WIPE%DM',.false.,error)
        call sic_delvariable('WIPE%NMR',.false.,error)
        call sic_delvariable('WIPE%RES',.false.,error)
        call sic_delvariable('WIPE%SUP',.false.,error)
        call free_vm(size,desc%addr)
        size = word*(6*nx*ny)
        if (sic_getvm(size,addr).ne.1) goto 99
        nm = gag_pointer(addr,memory)
        dm = nm+word*nx*ny
        nmr = dm+word*nx*ny
        res = nmr+word*nx*ny
        d  = res+word*nx*ny
        cb = d+word*nx*ny
        !
        ndim = 2
        dim(1) = nx
        dim(2) = ny
        call sic_def_real_addr('WIPE%NM', memory(nm), ndim,dim,.true.,error)
        call sic_def_real_addr('WIPE%DM', memory(dm), ndim,dim,.true.,error)
        call sic_def_real_addr('WIPE%RES',memory(res),ndim,dim,.true.,error)
        call sic_def_real_addr('WIPE%NMR',memory(nmr),ndim,dim,.true.,error)
        call sic_def_inte_addr('WIPE%SUP',memory(d),  ndim,dim,.true.,error)
        !
     else
        call sic_descriptor ('WIPE%NM',desc,found)
        nm = gag_pointer(desc%addr,memory)
        dm = nm+word*nx*ny
        nmr = dm+word*nx*ny
        res = nmr+word*nx*ny
        d  = res+word*nx*ny
        cb = d+word*nx*ny
     endif
  else
     size = word*(6*nx*ny)
     if (sic_getvm(size,addr).ne.1) goto 99
     nm = gag_pointer(addr,memory)
     dm = nm+word*nx*ny
     nmr = dm+word*nx*ny
     res = nmr+word*nx*ny
     d  = res+word*nx*ny
     cb = d+word*nx*ny
     !
     ndim = 2
     dim(1) = nx
     dim(2) = ny
     call sic_def_real_addr('WIPE%NM', memory(nm), ndim,dim,.true.,error)
     call sic_def_real_addr('WIPE%DM', memory(dm), ndim,dim,.true.,error)
     call sic_def_real_addr('WIPE%RES',memory(res),ndim,dim,.true.,error)
     call sic_def_real_addr('WIPE%NMR',memory(nmr),ndim,dim,.true.,error)
     call sic_def_inte_addr('WIPE%SUP',memory(d),  ndim,dim,.true.,error)
  endif
  !
  call sic_descriptor ('WIPE%VR',desc,found)
  if (found) then
     if (size.ne.2*nv) then
        call sic_delvariable('WIPE%VR',.false.,error)
        call sic_delvariable('WIPE%VI',.false.,error)
        call free_vm(size,desc%addr)
        size = word*(2*nv)
        if (sic_getvm(size,addr).ne.1) goto 99
        vr = gag_pointer(addr,memory)
        vi = vr+word*nv
        !
        ndim = 1
        dim(1) = nv
        call sic_def_real_addr('WIPE%VR',memory(vr),ndim,dim,.true.,error)
        call sic_def_real_addr('WIPE%VI',memory(vi),ndim,dim,.true.,error)
        !
     else
        call sic_descriptor ('WIPE%VR',desc,found)
        vr = gag_pointer(desc%addr,memory)
        vi = vr+word*nv
     endif
  else
     size = word*(2*nv)
     if (sic_getvm(size,addr).ne.1) goto 99
     vr = gag_pointer(addr,memory)
     vi = vr+word*nv
     !
     ndim = 1
     dim(1) = nv
     call sic_def_real_addr('WIPE%VR',memory(vr),ndim,dim,.true.,error)
     call sic_def_real_addr('WIPE%VI',memory(vi),ndim,dim,.true.,error)
  endif
  !
  ! Reading visibilities
  write(chain,101) 'Processing plane number ',proc
  call map_message(seve%i,rname,chain)
  call get_uvd(nv,memory(vr),memory(vi),duv,nplanes,proc)
  !
  ! Compute Dusty Map
  !
  call wipe_dirty_map(   &     !
       &    nx,   &              ! Number of pixels in X (input)
       &    ny,   &              ! Number of pixels in Y (input)
       &    nv,   &              ! number of visibilities (input)
       &    overs,   &           ! oversampling factor of the object workspace (input)
       &    memory(dm),   &      ! Dusty map (input)
       &    memory(vu),   &      !  (input)
       &    memory(vv),   &      !  (input)
       &    memory(vr),   &      ! Real part (input)
       &    memory(vi),   &      ! Imaginary part (input)
       &    memory(vs),   &      !
       &    memory(vk),   &      ! weight function (input)
       &    memory(ipw))
  !
  ! finding a mask from the defined polygon
  !
  size = nx*ny
  nscale = 1
  call copy_map(memory(dm),ddirty,size,nscale)
  call exec_program ('@ p_plot DIRTY')
  call gr_exec('BOX /ABSOLUTE')
  if (query) then
     call gagout(' ')
     call gagout('I-WIPE,  Use mouse to define initial support:')
     call gagout(' ')
     call gagout(' 1) Left button for the polygon points')
     call gagout('    then right button to finish.')
     call gagout(' 2) Press directly the right button')
     call gagout('    for no support change.')
     call gagout(' ')
     call greg_poly_define(rname,'',.false.,supportpol,supportvar,error)
     if (error)  return
  endif
  call greg_poly_plot(supportpol,error)
  if (error)  return
  call gr8_gimsk(supportpol,memory(cb),nx,ny,hneat%gil%convert(1,1), &
        hneat%gil%convert(1,2),box)
  !
  ! Compute Neat Map (Wipe's hearth)
  !
  call map_message(seve%i,rname,'Computing Neat Map...')
  call gag_cpu(cpu1)
  write(chain,'(A,F9.2)') 'Start Wiping ',cpu1-cpu0
  call map_message(seve%i,rname,chain)
  if (query) then
     call wipe_cw(   &          !
          &      nx,   &            ! Number of pixels in X (input)
          &      ny,   &            ! Number of pixels in Y (input)
          &      nv,   &            ! number of visibilities (input)
          &      nt,   &            ! maximum dimension of krylov space (input)
          &      overs,   &         ! oversampling factor of the object workspace (input)
          &      mnwi,   &          ! maximum number of wipe iterations (input)
          &      mnci,   &          ! maximum number of clean iterations (input)
          &      mncg,   &          ! maximum number of conjugate gradients iterations (input)
          &      mnpos,   &         ! maximum number of positivity constraint iterations (input)
          &      eps_pos,   &       ! Precision on the positivity constraint stop (input)
          &      trse,   &          ! threshold on the residu for support expansion (input)
          &      eps_mu,   &        ! stop test value for relative variation of mu (input)
          &      eps_phi,   &       ! stop test value for relative variation of phi (input)
          &      lmnb,   &          ! Lower margin on Neat Beam (input)
          &      umnb,   &          ! Upper margin on Neat Beam (input)
          &      memory(hr),   &    ! Synthesized aperture (input)
          &      memory(db),   &    ! Dusty beam (input)
          &      memory(nb),   &    ! Neat beam (input)
          &      memory(dm),   &    ! Dusty map (input)
          &      memory(vr),   &    ! Real part (input)
          &      memory(vi),   &    ! Imaginary part (input)
          &      memory(vn),   &    ! Sigma on visibility 1/sqrt(2)  (input)
          &      memory(vk),   &    ! weight function (input)
          &      memory(cb),   &    ! initial support (input)
          &      memory(nm),   &    ! Neat map (output)
          &      memory(res),   &   ! Residu (output)
          &      memory(nmr),   &   ! Neat Map+Residu (output)
          &      memory(d),   &     ! Support of neat map (output)
          &      memory(ipw))       ! pointer on the memory working space (input)
  else
     call sic_get_inte('WIPE%NCLEAN',f_nclean,error)
     call sic_get_real('WIPE%GAIN',f_cgain,error)
     call sic_get_inte('WIPE%CYCLE',f_nwipe,error)
     f_nclean = min(f_nclean,mnci)
     f_nwipe  = min(f_nclean,mnwi)
     call wipe_sg(   &          !
          &      nx,   &            ! Number of pixels in X (input)
          &      ny,   &            ! Number of pixels in Y (input)
          &      nv,   &            ! number of visibilities (input)
          &      nt,   &            ! maximum dimension of krylov space (input)
          &      overs,   &         ! oversampling factor of the object workspace (input)
          &      mnwi,   &          ! maximum number of wipe iterations (input)
          &      mnci,   &          ! maximum number of clean iterations (input)
          &      mncg,   &          ! maximum number of conjugate gradients iterations (input)
          &      mnpos,   &         ! maximum number of positivity constraint iterations (input)
          &      eps_pos,   &       ! Precision on the positivity constraint stop (input)
          &      trse,   &          ! threshold on the residu for support expansion (input)
          &      eps_mu,   &        ! stop test value for relative variation of mu (input)
          &      eps_phi,   &       ! stop test value for relative variation of phi (input)
          &      lmnb,   &          ! Lower margin on Neat Beam (input)
          &      umnb,   &          ! Upper margin on Neat Beam (input)
          &      memory(hr),   &    ! Synthesized aperture (input)
          &      memory(db),   &    ! Dusty beam (input)
          &      memory(nb),   &    ! Neat beam (input)
          &      memory(dm),   &    ! Dusty map (input)
          &      memory(vr),   &    ! Real part (input)
          &      memory(vi),   &    ! Imaginary part (input)
          &      memory(vn),   &    ! Sigma on visibility 1/sqrt(2)  (input)
          &      memory(vk),   &    ! weight function (input)
          &      memory(cb),   &    ! initial support (input)
          &      memory(nm),   &    ! Neat map (output)
          &      memory(res),   &   ! Residu (output)
          &      memory(nmr),   &   ! Neat Map+Residu (output)
          &      memory(d),   &     ! Support of neat map (output)
          &      memory(ipw),   &   ! pointer on the memory working space (input)
          &      f_nclean,f_cgain,f_nwipe)  ! WIPE parameters
  endif
  call gag_cpu(cpu1)
  write(chain,'(A,F9.2)') 'Finished Wiping ',cpu1-cpu0
  call map_message(seve%i,rname,chain)
  !
  ! Freeing working space
  call free_vm(size_ipw,addr_ipw)
  !
  ! Get scale (map unit to Jy/Beam) from Neat Beam
  call get_scale(memory(nb),nx,ny,nscale)
  !
  ! Saving final result, in Jy/beam units
  size = nx*ny
  call copy_map(memory(nm),dneat,size,nscale)
  call copy_map(memory(dm),ddirty,size,nscale)
  call copy_map(memory(res),dresid,size,nscale)
  call copy_map(memory(nmr),dnmr,size,nscale)
  !
  ! computing right parameters for flux calculation
  ratio = major/minor
  nscale = nscale*abs(hneat%gil%convert(3,1)*hneat%gil%convert(3,2))
  hneat%gil%reso_words = 3
  hnmr%gil%reso_words = 3
  call flux_prep(nscale,ratio,f_major,f_minor)
  !
  hneat%gil%majo = f_major
  hneat%gil%mino = f_minor
  hneat%gil%posa = pi*angle/180.0
  !
  hnmr%gil%majo = f_major
  hnmr%gil%mino = f_minor
  hnmr%gil%posa = pi*angle/180.0
  !
  save_data(code_save_neat) = .true.
  save_data(code_save_dirty) = .true.
  save_data(code_save_resid) = .true.
  save_data(code_save_nmr) = .true.
  !
  return
  !
99 call map_message(seve%e,rname,'Memory allocation failure')
  error = .true.
  return
  !
101 format(a,i10)
102 call map_message(seve%e,rname,'SIC variable not defined')
  error = .true.
  return
end subroutine wipe_solve
