program uv_model
  use gildas_def
  use image_def
  use gkernel_interfaces
  !---------------------------------------------------------------------
  ! GILDAS
  !   Compute a UV data set from an image model
  !    Input : a UV table, created for example by ASTRO
  !    Input : an image.
  !    Output: the UV table, where the visibilities have been computed.
  !---------------------------------------------------------------------
  ! Local
  type(gildas) :: x,y,z
  integer(kind=index_length) :: nf,nx,ny
  integer(kind=4) :: n, i, ier
  character(len=filename_length) :: uvdata,image,name,uvmodel
  character(len=132) :: mess
  real(kind=4) :: freq
  real(kind=4), allocatable :: mapx(:), mapy(:)
  logical :: error
  !
  call gagout('F-UV_MODEL,  This task is currently broken')
  call gagout('F-UV_MODEL,  You can use UV_FMODEL as a replacement for the time being')
  goto 999
  !
  ! Code:
  call gildas_open
  call gildas_char('UVSAMPLE$',uvdata)
  call gildas_char('IMAGE$',image)
  call gildas_char('UVMODEL$',uvmodel)
  call gildas_close
  !
  ! Input file
  n = lenc(uvdata)
  if (n.le.0) goto 999
  call gildas_null (z, type = 'UVT')
  name = uvdata(1:n)
  call gdf_read_gildas (z, name, '.uvt', error)
  if (error) then
    call gagout ('F-UV_MODEL,  Cannot read input table')
    goto 999
  endif
  !
  ! Image
  n = lenc(image)
  if (n.le.0) goto 999
  call gildas_null (y, type = 'IMAGE')
  name = image(1:n)
  call gdf_read_gildas (y, name, '.vlm', error, rank=3)
  if (error) then
    mess = 'E-UV_TABLE,  Cannot read input image '//y%file
    call gagout (mess)
    goto 999
  endif
  if (y%char%code(1).eq.'FREQUENCY' .or.  &
      y%char%code(1).eq.'VELOCITY'  .or.  &
      y%char%code(1).eq.'UNKNOWN') then
    nf = y%gil%dim(1)
  else
    call gagout('E-UV_MODEL,  First axis unit not FREQUENCY')
    goto 999
  endif
  nx = y%gil%dim(2)
  ny = y%gil%dim(3)
  mess = 'I-UV_TABLE,  '//trim(y%file)//' succesfully opened'
  call gagout(mess)
  ! Define mean observing frequency
  freq = y%gil%freq+y%gil%fres*(nf*0.5-y%gil%ref(1))
  if (freq.lt.0.1) then        ! 100 kHz threshold for warning
    call gagout('W-UV_MODEL,  VERY low frequency. Please check!')
    write(6,*) 'Value is ',freq,' MHz'
    goto 999
  endif
  !
  ! Get Virtual Memory to compute the positions
  allocate (mapx(nx), mapy(ny), stat=ier)
  if (ier.ne.0) goto 999
  call docoor (nx,y%gil%convert(3,2),mapx)
  call docoor (ny,y%gil%convert(3,3),mapy)
  !
  call gildas_null (x, type = 'UVT') 
  call gdf_copy_header (y,x,error)            ! x%file is defined LATER
  n = lenc(uvmodel)
  if (n.le.0) then
    mess = 'E-UV_TABLE,  Output file name is empty.'
    call gagout (mess)
    goto 999
  endif
  name = uvmodel(1:n)
  call sic_parsef(name,x%file,' ','.uvt')
  x%gil%dim(1) = 7+3*nf
  x%gil%dim(2) = z%gil%dim(2)
  x%gil%dim(3) = 1
  x%gil%dim(4) = 1
  x%gil%ndim = 2
  !
  x%gil%convert = z%gil%convert
  !
  x%gil%ref(1)  = y%gil%ref(1)
  x%gil%val(1)  = y%gil%freq
  x%gil%inc(1)  = y%gil%fres
  x%char%unit = z%char%unit
  do i=1,4
    x%char%code(i) = z%char%code(i)
  enddo
  x%gil%faxi = 1
  !
  call gdf_create_image (x, error)
  if (error) then
    mess = 'E-UV_MODEL,  Cannot create output UV model '//x%file
    call gagout (mess)
    goto 999
  endif
  call gdf_allocate (x,error)
  if (error) then
    mess = 'E-UV_MODEL,  Cannot obtain memory slot for UV model'
    call gagout (mess)
    goto 999
  endif
  !
  mess = 'I-UV_TABLE,  Output table'//trim(x%file)//' succesfully opened'
  call gagout(mess)
  !
  call copyuv (x%gil%dim(1),x%gil%dim(2),x%r2d,z%gil%dim(1),z%r2d)
  !
  ! OK, compute the model
  call do_model(x%r2d,x%gil%dim(1),x%gil%dim(2),y%r3d,nf,nx,ny,mapx,mapy,freq)
  call gdf_write_data (x, x%r2d, error)
  call gagout('S-UV_MODEL,  Successful completion')
  call sysexi(1)
  !
999  call sysexi(fatale)
end program uv_model
!
subroutine do_model (visi,nc,nv,map,nf,nx,ny,mapx,mapy,freq)
  use gildas_def
  !---------------------------------------------------------------------
  !
  !---------------------------------------------------------------------
  integer(kind=index_length), intent(in)    :: nc             !
  integer(kind=index_length), intent(in)    :: nv             !
  real(kind=4),               intent(inout) :: visi(nc,nv)    !
  integer(kind=index_length), intent(in)    :: nf             !
  integer(kind=index_length), intent(in)    :: nx             !
  integer(kind=index_length), intent(in)    :: ny             !
  real(kind=4),               intent(in)    :: map(nf,nx,ny)  !
  real(kind=4),               intent(in)    :: mapx(nx)       !
  real(kind=4),               intent(in)    :: mapy(ny)       !
  real(kind=4),               intent(in)    :: freq           !
  ! Local
  real(kind=8), parameter :: pi=3.14159265358979323846d0
  real(kind=4), parameter :: f_to_k = 2.d0*pi/299792458.d-6  ! (2*pi/clight)*1d6
  real(kind=4) :: kwx,kwy,y,z,cosi,sinu
  integer(kind=index_length) :: i,ix,iy,if
  !
  do i = 1, nv                 ! loop on visibility points
    kwx = - freq * f_to_k * visi(1,i)  ! Prepare exponential coefficients
    kwy = - freq * f_to_k * visi(2,i)
    do iy=1, ny                ! Loop on pixels
      y = kwy*mapy(iy)
      do ix=1, nx
        z = kwx*mapx(ix) + y
        cosi = cos(z)
        sinu = sin(z)
        !               write(6,*) cosi,sinu
        do if = 1,nf
          visi(5+3*if,i) = visi(5+3*if,i)+map(if,ix,iy)*cosi
          visi(6+3*if,i) = visi(6+3*if,i)-map(if,ix,iy)*sinu
        enddo
      enddo
    enddo
  enddo
end subroutine do_model
!
subroutine docoor (n,xinc,x)
  use gildas_def
  !---------------------------------------------------------------------
  ! GILDAS UVMAP
  !  Compute FFT grid coordinates in U or V
  !---------------------------------------------------------------------
  integer(kind=index_length), intent(in)  :: n     !
  real(kind=8),               intent(in)  :: xinc  !
  real(kind=4),               intent(out) :: x(n)  !
  ! Local
  integer(kind=index_length) :: i
  do i=1,n
    x(i) = dble(i-n/2-1)*xinc
  enddo
end subroutine docoor
!
subroutine copyuv (nco,nv,out,nci,in)
  use gildas_def
  !---------------------------------------------------------------------
  !
  !---------------------------------------------------------------------
  integer(kind=index_length), intent(in)  :: nco          !
  integer(kind=index_length), intent(in)  :: nv           !
  real(kind=4),               intent(out) :: out(nco,nv)  !
  integer(kind=index_length), intent(in)  :: nci          !
  real(kind=4),               intent(in)  :: in(nci,nv)   !
  ! Local
  integer(kind=index_length) :: i,j
  do i=1,nv
    do j=1,7
      out(j,i) = in(j,i)
    enddo
    do j=8,nco,3
      out(j,i) = 0.
      out(j+1,i) = 0.
      out(j+2,i) = in(10,i)
    enddo
  enddo
end subroutine copyuv
